@@ -57,10 +57,7 @@ static const GimpActionEntry image_actions[] =
 
   { "image-menu",                  NULL, NC_("image-action", "_Image")     },
   { "image-mode-menu",             NULL, NC_("image-action", "_Mode")      },
-  { "image-precision-menu",        NULL, NC_("image-action", "Pr_ecision") },
-#if PENDING_TRANSLATION
   { "image-precision-menu",        NULL, NC_("image-action", "_Encoding")  },
-#endif
   { "image-color-management-menu", NULL, NC_("image-action",
                                              "Color Ma_nagement")          },
   { "image-transform-menu",        NULL, NC_("image-action", "_Transform") },
@@ -180,14 +177,14 @@ static const GimpActionEntry image_actions[] =
 
 static const GimpToggleActionEntry image_toggle_actions[] =
 {
-  { "image-color-management-enabled", NULL,
-    NC_("image-action", "_Enable Color Management"), NULL,
-    NC_("image-action", "Whether the image is color managed. Disabling "
-        "color management is equivalent to assigning a built-in sRGB "
-        "color profile. Better leave color management enabled."),
-    image_color_management_enabled_cmd_callback,
+  { "image-color-profile-use-srgb", NULL,
+    NC_("image-action", "Use _sRGB Profile"), NULL,
+    NC_("image-action", "Temporarily use an sRGB profile for the image. "
+        "This is the same as discarding the image's color profile, but "
+        "allows to easily restore the profile."),
+    image_color_profile_use_srgb_cmd_callback,
     TRUE,
-    GIMP_HELP_IMAGE_COLOR_MANAGEMENT_ENABLED }
+    GIMP_HELP_IMAGE_COLOR_PROFILE_USE_SRGB }
 };
 
 static const GimpRadioActionEntry image_convert_base_type_actions[] =
@@ -247,19 +244,25 @@ static const GimpRadioActionEntry image_convert_precision_actions[] =
     GIMP_COMPONENT_TYPE_DOUBLE, GIMP_HELP_IMAGE_CONVERT_DOUBLE }
 };
 
-static const GimpRadioActionEntry image_convert_gamma_actions[] =
+static const GimpRadioActionEntry image_convert_trc_actions[] =
 {
-  { "image-convert-gamma", NULL,
-    NC_("image-convert-action", "Perceptual gamma (sRGB)"), NULL,
-    NC_("image-convert-action",
-        "Convert the image to perceptual (sRGB) gamma"),
-    FALSE, GIMP_HELP_IMAGE_CONVERT_GAMMA },
-
   { "image-convert-linear", NULL,
     NC_("image-convert-action", "Linear light"), NULL,
     NC_("image-convert-action",
         "Convert the image to linear light"),
-    TRUE, GIMP_HELP_IMAGE_CONVERT_GAMMA }
+    GIMP_TRC_LINEAR, GIMP_HELP_IMAGE_CONVERT_GAMMA },
+
+  { "image-convert-non-linear", NULL,
+    NC_("image-convert-action", "Non-Linear"), NULL,
+    NC_("image-convert-action",
+        "Convert the image to non-linear gamma from the color profile"),
+    GIMP_TRC_NON_LINEAR, GIMP_HELP_IMAGE_CONVERT_GAMMA },
+
+  { "image-convert-perceptual", NULL,
+    NC_("image-convert-action", "Perceptual (sRGB)"), NULL,
+    NC_("image-convert-action",
+        "Convert the image to perceptual (sRGB) gamma"),
+    GIMP_TRC_PERCEPTUAL, GIMP_HELP_IMAGE_CONVERT_GAMMA }
 };
 
 static const GimpEnumActionEntry image_flip_actions[] =
@@ -323,10 +326,10 @@ image_actions_setup (GimpActionGroup *group)
                                        image_convert_precision_cmd_callback);
 
   gimp_action_group_add_radio_actions (group, "image-convert-action",
-                                       image_convert_gamma_actions,
-                                       G_N_ELEMENTS (image_convert_gamma_actions),
+                                       image_convert_trc_actions,
+                                       G_N_ELEMENTS (image_convert_trc_actions),
                                        NULL, 0,
-                                       image_convert_gamma_cmd_callback);
+                                       image_convert_trc_cmd_callback);
 
   gimp_action_group_add_enum_actions (group, "image-action",
                                       image_flip_actions,
@@ -352,16 +355,26 @@ void
 image_actions_update (GimpActionGroup *group,
                       gpointer         data)
 {
-  GimpImage *image         = action_data_get_image (data);
-  gboolean   is_indexed    = FALSE;
-  gboolean   is_u8_gamma   = FALSE;
-  gboolean   is_double     = FALSE;
-  gboolean   aux           = FALSE;
-  gboolean   lp            = FALSE;
-  gboolean   sel           = FALSE;
-  gboolean   groups        = FALSE;
-  gboolean   color_managed = FALSE;
-  gboolean   profile       = FALSE;
+  GimpImage *image          = action_data_get_image (data);
+  gboolean   is_indexed     = FALSE;
+  gboolean   is_u8_gamma    = FALSE;
+  gboolean   is_double      = FALSE;
+  gboolean   aux            = FALSE;
+  gboolean   lp             = FALSE;
+  gboolean   sel            = FALSE;
+  gboolean   groups         = FALSE;
+  gboolean   profile_srgb   = FALSE;
+  gboolean   profile_hidden = FALSE;
+  gboolean   profile        = FALSE;
+
+#define SET_LABEL(action,label) \
+        gimp_action_group_set_action_label (group, action, (label))
+#define SET_SENSITIVE(action,condition) \
+        gimp_action_group_set_action_sensitive (group, action, (condition) != 0)
+#define SET_ACTIVE(action,condition) \
+        gimp_action_group_set_action_active (group, action, (condition) != 0)
+#define SET_VISIBLE(action,condition) \
+        gimp_action_group_set_action_visible (group, action, (condition) != 0)
 
   if (image)
     {
@@ -370,10 +383,13 @@ image_actions_update (GimpActionGroup *group,
       GimpImageBaseType  base_type;
       GimpPrecision      precision;
       GimpComponentType  component_type;
+      GimpTRCType        trc;
 
       base_type      = gimp_image_get_base_type (image);
       precision      = gimp_image_get_precision (image);
       component_type = gimp_image_get_component_type (image);
+      trc            = gimp_babl_format_get_trc
+                         (gimp_image_get_layer_format (image, FALSE));
 
       switch (base_type)
         {
@@ -382,7 +398,7 @@ image_actions_update (GimpActionGroup *group,
         case GIMP_INDEXED: action = "image-convert-indexed";   break;
         }
 
-      gimp_action_group_set_action_active (group, action, TRUE);
+      SET_ACTIVE (action, TRUE);
 
       switch (component_type)
         {
@@ -394,22 +410,30 @@ image_actions_update (GimpActionGroup *group,
         case GIMP_COMPONENT_TYPE_DOUBLE: action = "image-convert-double"; break;
         }
 
-      gimp_action_group_set_action_active (group, action, TRUE);
+      SET_ACTIVE (action, TRUE);
 
-      if (gimp_babl_format_get_linear (gimp_image_get_layer_format (image,
-                                                                    FALSE)))
+      switch (trc)
         {
-          gimp_action_group_set_action_active (group, "image-convert-linear",
-                                               TRUE);
-        }
-      else
-        {
-          gimp_action_group_set_action_active (group, "image-convert-gamma",
-                                               TRUE);
+        case GIMP_TRC_LINEAR:
+          action = "image-convert-linear";
+          SET_VISIBLE ("image-convert-perceptual", FALSE);
+          break;
+
+        case GIMP_TRC_NON_LINEAR:
+          action = "image-convert-non-linear";
+          SET_VISIBLE ("image-convert-perceptual", FALSE);
+          break;
+
+        case GIMP_TRC_PERCEPTUAL:
+          action = "image-convert-perceptual";
+          SET_VISIBLE ("image-convert-perceptual", TRUE);
+          break;
         }
 
+      SET_ACTIVE (action, TRUE);
+
       is_indexed  = (base_type == GIMP_INDEXED);
-      is_u8_gamma = (precision == GIMP_PRECISION_U8_GAMMA);
+      is_u8_gamma = (precision == GIMP_PRECISION_U8_NON_LINEAR);
       is_double   = (component_type == GIMP_COMPONENT_TYPE_DOUBLE);
       aux         = (gimp_image_get_active_channel (image) != NULL);
       lp          = ! gimp_image_is_empty (image);
@@ -419,18 +443,13 @@ image_actions_update (GimpActionGroup *group,
 
       groups = ! gimp_item_stack_is_flat (GIMP_ITEM_STACK (layers));
 
-      color_managed = gimp_image_get_is_color_managed (image);
-      profile       = (gimp_image_get_color_profile (image) != NULL);
+      profile_srgb = gimp_image_get_use_srgb_profile (image, &profile_hidden);
+      profile      = (gimp_image_get_color_profile (image) != NULL);
+    }
+  else
+    {
+      SET_VISIBLE ("image-convert-perceptual", FALSE);
     }
-
-#define SET_LABEL(action,label) \
-        gimp_action_group_set_action_label (group, action, (label))
-#define SET_SENSITIVE(action,condition) \
-        gimp_action_group_set_action_sensitive (group, action, (condition) != 0)
-#define SET_ACTIVE(action,condition) \
-        gimp_action_group_set_action_active (group, action, (condition) != 0)
-#define SET_VISIBLE(action,condition) \
-        gimp_action_group_set_action_visible (group, action, (condition) != 0)
 
   SET_SENSITIVE ("image-duplicate", image);
 
@@ -461,11 +480,12 @@ image_actions_update (GimpActionGroup *group,
   SET_SENSITIVE ("image-convert-double", image && !is_indexed);
   SET_VISIBLE   ("image-convert-double", is_double);
 
-  SET_SENSITIVE ("image-convert-gamma",  image);
-  SET_SENSITIVE ("image-convert-linear", image && !is_indexed);
+  SET_SENSITIVE ("image-convert-linear",     image && !is_indexed);
+  SET_SENSITIVE ("image-convert-non-linear", image);
+  SET_SENSITIVE ("image-convert-perceptual", image && !is_indexed);
 
-  SET_SENSITIVE ("image-color-management-enabled", image);
-  SET_ACTIVE    ("image-color-management-enabled", image && color_managed);
+  SET_SENSITIVE ("image-color-profile-use-srgb", image && (profile || profile_hidden));
+  SET_ACTIVE    ("image-color-profile-use-srgb", image && profile_srgb);
 
   SET_SENSITIVE ("image-color-profile-assign",  image);
   SET_SENSITIVE ("image-color-profile-convert", image);
