@@ -376,8 +376,6 @@ windows_actions_image_notify (GimpDisplay      *display,
 
       gimp_action_group_add_actions (group, NULL, &entry, 1);
 
-      gimp_action_group_set_action_always_show_image (group, action_name,
-                                                      TRUE);
       action = gimp_action_group_get_action (group, action_name);
 
       g_object_set_data (G_OBJECT (action), "display", display);
@@ -395,7 +393,7 @@ windows_actions_image_notify (GimpDisplay      *display,
       escaped = gimp_escape_uline (display_name);
 
       title = g_strdup_printf ("%s-%d.%d", escaped,
-                               gimp_image_get_ID (image),
+                               gimp_image_get_id (image),
                                gimp_display_get_instance (display));
       g_free (escaped);
 
