@@ -886,7 +886,8 @@ user_install_migrate_files (GimpUserInstall *install)
               g_str_has_prefix (basename, "gimpswap.") ||
               strcmp (basename, "pluginrc") == 0       ||
               strcmp (basename, "themerc") == 0        ||
-              strcmp (basename, "toolrc") == 0)
+              strcmp (basename, "toolrc") == 0         ||
+              strcmp (basename, "gtkrc") == 0)
             {
               goto next_file;
             }
