@@ -773,6 +773,71 @@ gimp_file_with_new_extension (GFile *file,
   return ret;
 }
 
+/**
+ * gimp_file_delete_recursive:
+ * @file: #GFile to delete from file system.
+ * @error:
+ *
+ * Delete @file. If file is a directory, it will delete its children as
+ * well recursively. It will not follow symlinks so you won't end up in
+ * infinite loops, not will you be at risk of deleting your whole file
+ * system (unless you pass the root of course!).
+ * Such function unfortunately does not exist in glib, which only allows
+ * to delete single files or empty directories by default.
+ *
+ * Returns: %TRUE if @file was successfully deleted and all its
+ * children, %FALSE otherwise with @error filled.
+ */
+gboolean
+gimp_file_delete_recursive (GFile   *file,
+                            GError **error)
+{
+  gboolean success = TRUE;
+
+  if (g_file_query_exists (file, NULL))
+    {
+      if (g_file_query_file_type (file, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+                                  NULL) == G_FILE_TYPE_DIRECTORY)
+        {
+          GFileEnumerator *enumerator;
+
+          enumerator = g_file_enumerate_children (file,
+                                                  G_FILE_ATTRIBUTE_STANDARD_NAME ","
+                                                  G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN ","
+                                                  G_FILE_ATTRIBUTE_TIME_MODIFIED,
+                                                  G_FILE_QUERY_INFO_NONE,
+                                                  NULL, NULL);
+          if (enumerator)
+            {
+              GFileInfo *info;
+
+              while ((info = g_file_enumerator_next_file (enumerator, NULL, NULL)))
+                {
+                  GFile *child;
+
+                  child = g_file_enumerator_get_child (enumerator, info);
+                  g_object_unref (info);
+
+                  if (! gimp_file_delete_recursive (child, error))
+                    success = FALSE;
+
+                  g_object_unref (child);
+                  if (! success)
+                    break;
+                }
+
+              g_object_unref (enumerator);
+            }
+        }
+
+      if (success)
+        /* Non-directory or empty directory. */
+        success = g_file_delete (file, NULL, error);
+    }
+
+  return success;
+}
+
 gchar *
 gimp_data_input_stream_read_line_always (GDataInputStream  *stream,
                                          gsize             *length,
@@ -882,6 +947,36 @@ gimp_g_list_compare (GList *list1,
   return 0;
 }
 
+GimpTRCType
+gimp_suggest_trc_for_component_type (GimpComponentType component_type,
+                                     GimpTRCType       old_trc)
+{
+  GimpTRCType new_trc = old_trc;
+
+  switch (component_type)
+    {
+    case GIMP_COMPONENT_TYPE_U8:
+      /* default to non-linear when converting 8 bit */
+      new_trc = GIMP_TRC_NON_LINEAR;
+      break;
+
+    case GIMP_COMPONENT_TYPE_U16:
+    case GIMP_COMPONENT_TYPE_U32:
+    default:
+      /* leave TRC alone by default when converting to 16/32 bit int */
+      break;
+
+    case GIMP_COMPONENT_TYPE_HALF:
+    case GIMP_COMPONENT_TYPE_FLOAT:
+    case GIMP_COMPONENT_TYPE_DOUBLE:
+      /* default to linear when converting to floating point */
+      new_trc = GIMP_TRC_LINEAR;
+      break;
+    }
+
+  return new_trc;
+}
+
 typedef struct
 {
   gint              ref_count;
@@ -1049,51 +1144,6 @@ gimp_idle_run_async_full (gint             priority,
   return g_object_ref (data->async);
 }
 
-#if defined(G_OS_WIN32)
-
-gboolean
-gimp_win32_have_wintab (void)
-{
-  gunichar2 wchars_buffer[MAX_PATH + 1];
-  UINT      wchars_count = 0;
-
-  memset (wchars_buffer, 0, sizeof (wchars_buffer));
-  wchars_count = GetSystemDirectoryW (wchars_buffer, MAX_PATH);
-  if (wchars_count > 0 && wchars_count < MAX_PATH)
-    {
-      char *system32_directory = g_utf16_to_utf8 (wchars_buffer, -1, NULL, NULL, NULL);
-
-      if (system32_directory)
-        {
-          GFile    *file   = g_file_new_build_filename (system32_directory, "Wintab32.dll", NULL);
-          gboolean  exists = g_file_query_exists (file, NULL);
-
-          g_object_unref (file);
-          g_free (system32_directory);
-
-          return exists;
-        }
-    }
-
-  return FALSE;
-}
-
-gboolean
-gimp_win32_have_windows_ink (void)
-{
-  wchar_t buf[100];
-  DWORD ret;
-
-  memset (buf, 0, sizeof (buf));
-  ret = GetEnvironmentVariableW (L"GDK_WIN32_FEATURES", buf, sizeof (buf) - 1);
-  if (ret > 0 && ret < 100)
-    return wcsstr (buf, L"winpointer") != NULL;
-
-  return FALSE;
-}
-
-#endif
-
 
 /*  debug stuff  */
 
@@ -1133,7 +1183,7 @@ gimp_create_image_from_buffer (Gimp        *gimp,
                                            NULL /* same image */);
   gimp_image_add_layer (image, layer, NULL, -1, FALSE);
 
-  gimp_create_display (gimp, image, GIMP_UNIT_PIXEL, 1.0, NULL, 0);
+  gimp_create_display (gimp, image, GIMP_UNIT_PIXEL, 1.0, NULL);
 
   /* unref the image unconditionally, even when no display was created */
   g_object_add_weak_pointer (G_OBJECT (image), (gpointer) &image);
