@@ -50,6 +50,7 @@ static void            gimp_layer_mask_bounding_box_changed (GimpDrawable      *
 static void            gimp_layer_mask_convert_type         (GimpDrawable      *drawable,
                                                              GimpImage         *dest_image,
                                                              const Babl        *new_format,
+                                                             GimpColorProfile  *src_profile,
                                                              GimpColorProfile  *dest_profile,
                                                              GeglDitherMethod   layer_dither_type,
                                                              GeglDitherMethod   mask_dither_type,
@@ -215,6 +216,7 @@ static void
 gimp_layer_mask_convert_type (GimpDrawable      *drawable,
                               GimpImage         *dest_image,
                               const Babl        *new_format,
+                              GimpColorProfile  *src_profile,
                               GimpColorProfile  *dest_profile,
                               GeglDitherMethod   layer_dither_type,
                               GeglDitherMethod   mask_dither_type,
@@ -226,6 +228,7 @@ gimp_layer_mask_convert_type (GimpDrawable      *drawable,
 
   GIMP_DRAWABLE_CLASS (parent_class)->convert_type (drawable, dest_image,
                                                     new_format,
+                                                    src_profile,
                                                     dest_profile,
                                                     layer_dither_type,
                                                     mask_dither_type,
