@@ -43,10 +43,10 @@ struct _Selection
 {
   GimpDisplayShell *shell;            /*  shell that owns the selection     */
 
-  GimpSegment      *segs_in;          /*  gdk segments of area boundary     */
+  GimpSegment      *segs_in;          /*  segments of area boundary         */
   gint              n_segs_in;        /*  number of segments in segs_in     */
 
-  GimpSegment      *segs_out;         /*  gdk segments of area boundary     */
+  GimpSegment      *segs_out;         /*  segments of area boundary         */
   gint              n_segs_out;       /*  number of segments in segs_out    */
 
   guint             index;            /*  index of current stipple pattern  */
@@ -63,6 +63,8 @@ struct _Selection
 static void      selection_start          (Selection          *selection);
 static void      selection_stop           (Selection          *selection);
 
+static void      selection_draw           (Selection          *selection,
+                                           cairo_t            *cr);
 static void      selection_undraw         (Selection          *selection);
 
 static void      selection_render_mask    (Selection          *selection);
@@ -70,12 +72,11 @@ static void      selection_render_mask    (Selection          *selection);
 static void      selection_zoom_segs      (Selection          *selection,
                                            const GimpBoundSeg *src_segs,
                                            GimpSegment        *dest_segs,
-                                           gint                n_segs,
-                                           gint                canvas_offset_x,
-                                           gint                canvas_offset_y);
+                                           gint                n_segs);
 static void      selection_generate_segs  (Selection          *selection);
 static void      selection_free_segs      (Selection          *selection);
 
+static gboolean  selection_start_timeout  (Selection          *selection);
 static gboolean  selection_timeout        (Selection          *selection);
 
 static gboolean  selection_window_state_event      (GtkWidget           *shell,
@@ -103,7 +104,6 @@ gimp_display_shell_selection_init (GimpDisplayShell *shell)
   selection->show_selection = gimp_display_shell_get_show_selection (shell);
 
   shell->selection = selection;
-  shell->selection_update = g_get_monotonic_time ();
 
   g_signal_connect (shell, "window-state-event",
                     G_CALLBACK (selection_window_state_event),
@@ -229,22 +229,10 @@ selection_start (Selection *selection)
   selection_stop (selection);
 
   /*  If this selection is paused, do not start it  */
-  if (selection->paused == 0                             &&
-      gimp_display_get_image (selection->shell->display) &&
-      selection->show_selection)
+  if (selection->paused == 0)
     {
-      /*  Draw the ants once  */
-      selection_timeout (selection);
-
-      if (selection->segs_in && selection->shell_visible)
-        {
-          GimpDisplayConfig *config = selection->shell->display->config;
-
-          selection->timeout = g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE,
-                                                   config->marching_ants_speed,
-                                                   (GSourceFunc) selection_timeout,
-                                                   selection, NULL);
-        }
+      selection->timeout = g_idle_add ((GSourceFunc) selection_start_timeout,
+                                       selection);
     }
 }
 
@@ -258,41 +246,15 @@ selection_stop (Selection *selection)
     }
 }
 
-void
-gimp_display_shell_selection_draw (GimpDisplayShell *shell,
-                                   cairo_t          *cr)
+static void
+selection_draw (Selection *selection,
+                cairo_t   *cr)
 {
-  if (gimp_display_get_image (shell->display) &&
-      shell->selection && shell->selection->show_selection)
+  if (selection->segs_in)
     {
-      GimpDisplayConfig *config = shell->display->config;
-      gint64             time   = g_get_monotonic_time ();
-
-      if ((time - shell->selection_update) / 1000 > config->marching_ants_speed &&
-          shell->selection->paused == 0)
-        {
-          shell->selection_update = time;
-          shell->selection->index++;
-        }
-
-      selection_generate_segs (shell->selection);
-
-      if (shell->selection->segs_in)
-        {
-          gimp_display_shell_draw_selection_in (shell->selection->shell, cr,
-                                                shell->selection->segs_in_mask,
-                                                shell->selection->index % 8);
-        }
-
-      if (shell->selection->segs_out)
-        {
-          if (shell->selection->shell->rotate_transform)
-            cairo_transform (cr, shell->selection->shell->rotate_transform);
-
-          gimp_display_shell_draw_selection_out (shell->selection->shell, cr,
-                                                 shell->selection->segs_out,
-                                                 shell->selection->n_segs_out);
-        }
+      gimp_display_shell_draw_selection_in (selection->shell, cr,
+                                            selection->segs_in_mask,
+                                            selection->index % 8);
     }
 }
 
@@ -321,7 +283,7 @@ selection_render_mask (Selection *selection)
   cairo_surface_t *surface;
   cairo_t         *cr;
 
-  window = gtk_widget_get_window (GTK_WIDGET (selection->shell));
+  window = gtk_widget_get_window (selection->shell->canvas);
   surface = gdk_window_create_similar_surface (window, CAIRO_CONTENT_ALPHA,
                                                gdk_window_get_width  (window),
                                                gdk_window_get_height (window));
@@ -348,9 +310,7 @@ static void
 selection_zoom_segs (Selection          *selection,
                      const GimpBoundSeg *src_segs,
                      GimpSegment        *dest_segs,
-                     gint                n_segs,
-                     gint                canvas_offset_x,
-                     gint                canvas_offset_y)
+                     gint                n_segs)
 {
   const gint xclamp = selection->shell->disp_width + 1;
   const gint yclamp = selection->shell->disp_height + 1;
@@ -364,11 +324,11 @@ selection_zoom_segs (Selection          *selection,
     {
       if (! selection->shell->rotate_transform)
         {
-          dest_segs[i].x1 = CLAMP (dest_segs[i].x1, -1, xclamp) + canvas_offset_x;
-          dest_segs[i].y1 = CLAMP (dest_segs[i].y1, -1, yclamp) + canvas_offset_y;
+          dest_segs[i].x1 = CLAMP (dest_segs[i].x1, -1, xclamp);
+          dest_segs[i].y1 = CLAMP (dest_segs[i].y1, -1, yclamp);
 
-          dest_segs[i].x2 = CLAMP (dest_segs[i].x2, -1, xclamp) + canvas_offset_x;
-          dest_segs[i].y2 = CLAMP (dest_segs[i].y2, -1, yclamp) + canvas_offset_y;
+          dest_segs[i].x2 = CLAMP (dest_segs[i].x2, -1, xclamp);
+          dest_segs[i].y2 = CLAMP (dest_segs[i].y2, -1, yclamp);
         }
 
       /*  If this segment is a closing segment && the segments lie inside
@@ -399,10 +359,6 @@ selection_generate_segs (Selection *selection)
   GimpImage          *image = gimp_display_get_image (selection->shell->display);
   const GimpBoundSeg *segs_in;
   const GimpBoundSeg *segs_out;
-  gint                canvas_offset_x = 0;
-  gint                canvas_offset_y = 0;
-
-  selection_free_segs (selection);
 
   /*  Ask the image for the boundary of its selected region...
    *  Then transform that information into a new buffer of GimpSegments
@@ -416,19 +372,25 @@ selection_generate_segs (Selection *selection)
     {
       selection->segs_in = g_new (GimpSegment, selection->n_segs_in);
       selection_zoom_segs (selection, segs_in,
-                           selection->segs_in, selection->n_segs_in,
-                           canvas_offset_x, canvas_offset_y);
+                           selection->segs_in, selection->n_segs_in);
 
       selection_render_mask (selection);
     }
+  else
+    {
+      selection->segs_in = NULL;
+    }
 
   /*  Possible secondary boundary representation  */
   if (selection->n_segs_out)
     {
       selection->segs_out = g_new (GimpSegment, selection->n_segs_out);
       selection_zoom_segs (selection, segs_out,
-                           selection->segs_out, selection->n_segs_out,
-                           canvas_offset_x, canvas_offset_y);
+                           selection->segs_out, selection->n_segs_out);
+    }
+  else
+    {
+      selection->segs_out = NULL;
     }
 }
 
@@ -439,30 +401,102 @@ selection_free_segs (Selection *selection)
   selection->n_segs_in = 0;
 
   g_clear_pointer (&selection->segs_out, g_free);
-  selection->n_segs_out = 0;
+  selection->segs_out = NULL;
 
   g_clear_pointer (&selection->segs_in_mask, cairo_pattern_destroy);
 }
 
 static gboolean
-selection_timeout (Selection *selection)
+selection_start_timeout (Selection *selection)
 {
-  GimpDisplayConfig *config = selection->shell->display->config;
-  gint64             time   = g_get_monotonic_time ();
+  selection_free_segs (selection);
+  selection->timeout = 0;
 
-  if ((time - selection->shell->selection_update) / 1000 > config->marching_ants_speed)
+  if (! gimp_display_get_image (selection->shell->display))
+    return FALSE;
+
+  selection_generate_segs (selection);
+
+  selection->index = 0;
+
+  /*  Draw the ants  */
+  if (selection->show_selection)
     {
-      GdkWindow *window;
+      GdkWindow             *window;
+      GdkDrawingContext     *context;
+      cairo_rectangle_int_t  rect;
+      cairo_region_t        *region;
+      cairo_t               *cr;
 
-      window = gtk_widget_get_window (GTK_WIDGET (selection->shell));
+      window = gtk_widget_get_window (selection->shell->canvas);
 
-      gtk_widget_queue_draw_area (GTK_WIDGET (selection->shell),
-                                  0, 0,
-                                  gdk_window_get_width (window),
-                                  gdk_window_get_height (window));
+      rect.x      = 0;
+      rect.y      = 0;
+      rect.width  = gdk_window_get_width  (window);
+      rect.height = gdk_window_get_height (window);
+
+      region = cairo_region_create_rectangle (&rect);
+      context = gdk_window_begin_draw_frame (window, region);
+      cairo_region_destroy (region);
+
+      cr = gdk_drawing_context_get_cairo_context (context);
+
+      selection_draw (selection, cr);
+
+      if (selection->segs_out)
+        {
+          if (selection->shell->rotate_transform)
+            cairo_transform (cr, selection->shell->rotate_transform);
+
+          gimp_display_shell_draw_selection_out (selection->shell, cr,
+                                                 selection->segs_out,
+                                                 selection->n_segs_out);
+        }
+
+      gdk_window_end_draw_frame (window, context);
+
+      if (selection->segs_in && selection->shell_visible)
+        {
+          GimpDisplayConfig *config = selection->shell->display->config;
+
+          selection->timeout = g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE,
+                                                   config->marching_ants_speed,
+                                                   (GSourceFunc) selection_timeout,
+                                                   selection, NULL);
+        }
     }
 
-  return G_SOURCE_CONTINUE;
+  return FALSE;
+}
+
+static gboolean
+selection_timeout (Selection *selection)
+{
+  GdkWindow             *window;
+  GdkDrawingContext     *context;
+  cairo_rectangle_int_t  rect;
+  cairo_region_t        *region;
+  cairo_t               *cr;
+
+  window = gtk_widget_get_window (selection->shell->canvas);
+
+  rect.x      = 0;
+  rect.y      = 0;
+  rect.width  = gdk_window_get_width  (window);
+  rect.height = gdk_window_get_height (window);
+
+  region = cairo_region_create_rectangle (&rect);
+  context = gdk_window_begin_draw_frame (window, region);
+  cairo_region_destroy (region);
+
+  cr = gdk_drawing_context_get_cairo_context (context);
+
+  selection->index++;
+  selection_draw (selection, cr);
+
+  gdk_window_end_draw_frame (window, context);
+
+  return TRUE;
 }
 
 static void
