@@ -21,6 +21,9 @@
 
 #include <gegl.h>
 #include <gtk/gtk.h>
+#ifdef GDK_WINDOWING_WAYLAND
+#include <gdk/gdkwayland.h>
+#endif
 
 #include "libgimpbase/gimpbase.h"
 #include "libgimpmath/gimpmath.h"
@@ -29,6 +32,8 @@
 
 #include "gui-types.h"
 
+#include "core/gimp.h"
+
 #include "widgets/gimpwidgets-utils.h"
 
 #include "splash.h"
@@ -47,7 +52,7 @@ typedef struct
   gint            width;
   gint            height;
   GtkWidget      *progress;
-  GdkColor        color;
+  GdkRGBA         color;
   PangoLayout    *upper;
   gint            upper_x;
   gint            upper_y;
@@ -71,31 +76,27 @@ static void        splash_position_layouts     (GimpSplash     *splash,
                                                 const gchar    *text1,
                                                 const gchar    *text2,
                                                 GdkRectangle   *area);
-static gboolean    splash_area_expose          (GtkWidget      *widget,
-                                                GdkEventExpose *event,
+static gboolean    splash_area_draw            (GtkWidget      *widget,
+                                                cairo_t        *cr,
                                                 GimpSplash     *splash);
 static void        splash_rectangle_union      (GdkRectangle   *dest,
                                                 PangoRectangle *pango_rect,
                                                 gint            offset_x,
                                                 gint            offset_y);
-static gboolean    splash_average_text_area    (GimpSplash     *splash,
+static void        splash_average_text_area    (GimpSplash     *splash,
                                                 GdkPixbuf      *pixbuf,
-                                                GdkColor       *color);
+                                                GdkRGBA        *rgba);
 
 static GdkPixbufAnimation *
-                   splash_image_load           (gint            max_width,
-                                                gint            max_height,
-                                                gboolean        be_verbose);
-static GdkPixbufAnimation *
-                   splash_image_load_from_path (const gchar    *filename,
+                   splash_image_load           (Gimp           *gimp,
                                                 gint            max_width,
                                                 gint            max_height,
                                                 gboolean        be_verbose);
 static GdkPixbufAnimation *
-                   splash_image_load_from_file (GFile    *file,
-                                                gint      max_width,
-                                                gint      max_height,
-                                                gboolean  be_verbose);
+                   splash_image_load_from_file (GFile          *file,
+                                                gint            max_width,
+                                                gint            max_height,
+                                                gboolean        be_verbose);
 static GdkPixbufAnimation *
                    splash_image_pick_from_dirs (GList          *dirs,
                                                 gint            max_width,
@@ -108,23 +109,52 @@ static void        splash_timer_elapsed        (void);
 /*  public functions  */
 
 void
-splash_create (gboolean   be_verbose,
-               GdkScreen *screen,
-               gint       monitor)
+splash_create (Gimp       *gimp,
+               gboolean    be_verbose,
+               GdkMonitor *monitor)
 {
   GtkWidget          *frame;
   GtkWidget          *vbox;
   GdkPixbufAnimation *pixbuf;
   PangoRectangle      ink;
+  GdkRectangle        workarea;
   gint                max_width;
   gint                max_height;
 
   g_return_if_fail (splash == NULL);
-  g_return_if_fail (GDK_IS_SCREEN (screen));
+  g_return_if_fail (GDK_IS_MONITOR (monitor));
 
-  max_width  = gdk_screen_get_width (screen) / 2;
-  max_height = gdk_screen_get_height (screen) / 2;
-  pixbuf = splash_image_load (max_width, max_height, be_verbose);
+  gdk_monitor_get_workarea (monitor, &workarea);
+
+#ifdef GDK_WINDOWING_WAYLAND
+  if (GDK_IS_WAYLAND_DISPLAY (gdk_display_get_default ()))
+    {
+      /* This is completely extra ugly. Basically we cannot rely on
+       * gdk_monitor_get_workarea() on Wayland because the application
+       * cannot get trustworthy values. Indeed it is supposed to return
+       * a value in "application pixels" not "device pixels", in other
+       * words already inverse-scaled value. It turns out that on
+       * Wayland, under some conditions (but maybe not even all the
+       * time? I'm still unclear if there are conditions where returned
+       * value is properly scaled), it returns the device dimensions.
+       * E.g. on high-density display x2, making a splash for half the
+       * value, we ended up actually with the size of the whole display.
+       * This is why I special-case Wayland with a max of a third of the
+       * work area so that it would end up 2/3 maximum (at least not
+       * filling the screen!).
+       * This is ugly but for now I don't see the right solution. FIXME!
+       * See #5322.
+       */
+      max_width  = workarea.width  / 3;
+      max_height = workarea.height / 3;
+    }
+  else
+#endif
+    {
+      max_width  = workarea.width  / 2;
+      max_height = workarea.height / 2;
+    }
+  pixbuf = splash_image_load (gimp, max_width, max_height, be_verbose);
 
   if (! pixbuf)
     return;
@@ -137,25 +167,18 @@ splash_create (gboolean   be_verbose,
                   "type-hint",       GDK_WINDOW_TYPE_HINT_SPLASHSCREEN,
                   "title",           _("GIMP Startup"),
                   "role",            "gimp-startup",
-                  "screen",          screen,
                   "window-position", GTK_WIN_POS_CENTER,
                   "resizable",       FALSE,
                   NULL);
 
-  /* Don't remove this call, it's necessary to remove decorations on Windows
-   * (which is the natural state of splash-screens). Looks like the
-   * GDK_WINDOW_TYPE_HINT_SPLASHSCREEN hint is not used on some platforms.
-   */
-  gtk_window_set_decorated (GTK_WINDOW (splash->window), FALSE);
-
   g_signal_connect_swapped (splash->window, "delete-event",
                             G_CALLBACK (exit),
                             GINT_TO_POINTER (0));
 
   splash->width  = MIN (gdk_pixbuf_animation_get_width (pixbuf),
-                        gdk_screen_get_width (screen));
+                        workarea.width);
   splash->height = MIN (gdk_pixbuf_animation_get_height (pixbuf),
-                        gdk_screen_get_height (screen));
+                        workarea.height);
 
   frame = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
@@ -184,7 +207,7 @@ splash_create (gboolean   be_verbose,
   else if (splash->width > 2 * ink.width)
     gimp_pango_layout_set_scale (splash->upper, PANGO_SCALE_MEDIUM);
   else
-    gimp_pango_layout_set_scale (splash->upper, PANGO_SCALE_SMALL);
+    gimp_pango_layout_set_scale (splash->upper, PANGO_SCALE_MEDIUM);
 
   splash->lower = gtk_widget_create_pango_layout (splash->area,
                                                   MEASURE_LOWER);
@@ -208,8 +231,8 @@ splash_create (gboolean   be_verbose,
 
   g_object_unref (pixbuf);
 
-  g_signal_connect_after (splash->area, "expose-event",
-                          G_CALLBACK (splash_area_expose),
+  g_signal_connect_after (splash->area, "draw",
+                          G_CALLBACK (splash_area_draw),
                           splash);
 
   /*  add a progress bar  */
@@ -304,16 +327,11 @@ splash_update (const gchar *text1,
 /*  private functions  */
 
 static gboolean
-splash_area_expose (GtkWidget      *widget,
-                    GdkEventExpose *event,
-                    GimpSplash     *splash)
+splash_area_draw (GtkWidget  *widget,
+                  cairo_t    *cr,
+                  GimpSplash *splash)
 {
-  cairo_t *cr = gdk_cairo_create (event->window);
-
-  gdk_cairo_region (cr, event->region);
-  cairo_clip (cr);
-
-  gdk_cairo_set_source_color (cr, &splash->color);
+  gdk_cairo_set_source_rgba (cr, &splash->color);
 
   cairo_move_to (cr, splash->upper_x, splash->upper_y);
   pango_cairo_show_layout (cr, splash->upper);
@@ -321,8 +339,6 @@ splash_area_expose (GtkWidget      *widget,
   cairo_move_to (cr, splash->lower_x, splash->lower_y);
   pango_cairo_show_layout (cr, splash->lower);
 
-  cairo_destroy (cr);
-
   return FALSE;
 }
 
@@ -426,10 +442,10 @@ splash_rectangle_union (GdkRectangle   *dest,
 /* This function chooses a gray value for the text color, based on
  * the average luminance of the text area of the splash image.
  */
-static gboolean
+static void
 splash_average_text_area (GimpSplash *splash,
                           GdkPixbuf  *pixbuf,
-                          GdkColor   *color)
+                          GdkRGBA    *color)
 {
   const guchar *pixels;
   gint          rowstride;
@@ -439,8 +455,8 @@ splash_average_text_area (GimpSplash *splash,
   GdkRectangle  image     = { 0, 0, 0, 0 };
   GdkRectangle  area      = { 0, 0, 0, 0 };
 
-  g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), FALSE);
-  g_return_val_if_fail (gdk_pixbuf_get_bits_per_sample (pixbuf) == 8, FALSE);
+  g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
+  g_return_if_fail (gdk_pixbuf_get_bits_per_sample (pixbuf) == 8);
 
   image.width  = gdk_pixbuf_get_width (pixbuf);
   image.height = gdk_pixbuf_get_height (pixbuf);
@@ -484,35 +500,41 @@ splash_average_text_area (GimpSplash *splash,
 
     }
 
-  color->red = color->green = color->blue = (luminance << 8 | luminance);
-
-  return gdk_colormap_alloc_color (gtk_widget_get_colormap (splash->area),
-                                   color, FALSE, TRUE);
+  color->red = color->green = color->blue = luminance / 255.0;
+  color->alpha = 1.0;
 }
 
 static GdkPixbufAnimation *
-splash_image_load (gint      max_width,
+splash_image_load (Gimp     *gimp,
+                   gint      max_width,
                    gint      max_height,
                    gboolean  be_verbose)
 {
   GdkPixbufAnimation *animation = NULL;
-  gchar              *filename;
   GFile              *file;
   GList              *list;
 
-  /* File "gimp-splash.png" in personal configuration directory. */
-  filename = gimp_personal_rc_file ("gimp-splash.png");
-  animation = splash_image_load_from_path (filename,
+  /* Random image in splash extensions. */
+  g_object_get (gimp->extension_manager,
+                "splash-paths", &list,
+                NULL);
+  animation = splash_image_pick_from_dirs (list,
                                            max_width, max_height,
                                            be_verbose);
-  g_free (filename);
+  if (animation)
+    return animation;
+
+  /* File "gimp-splash.png" in personal configuration directory. */
+  file = gimp_directory_file ("gimp-splash.png", NULL);
+  animation = splash_image_load_from_file (file,
+                                           max_width, max_height,
+                                           be_verbose);
+  g_object_unref (file);
   if (animation)
     return animation;
 
   /* Random image under splashes/ directory in personal config dir. */
-  filename = gimp_personal_rc_file ("splashes");
-  file = g_file_new_for_path (filename);
-  g_free (filename);
+  file = gimp_directory_file ("splashes", NULL);
   list = NULL;
   list = g_list_prepend (list, file);
   animation = splash_image_pick_from_dirs (list,
@@ -523,19 +545,16 @@ splash_image_load (gint      max_width,
     return animation;
 
   /* Release splash image. */
-  filename = g_build_filename (gimp_data_directory (),
-                               "images", "gimp-splash.png", NULL);
-  animation = splash_image_load_from_path (filename,
+  file = gimp_data_directory_file ("images", "gimp-splash.png", NULL);
+  animation = splash_image_load_from_file (file,
                                            max_width, max_height,
                                            be_verbose);
-  g_free (filename);
+  g_object_unref (file);
   if (animation)
     return animation;
 
   /* Random release image in installed splashes/ directory. */
-  filename = g_build_filename (gimp_data_directory (), "splashes", NULL);
-  file = g_file_new_for_path (filename);
-  g_free (filename);
+  file = gimp_data_directory_file ("splashes", NULL);
   list = NULL;
   list = g_list_prepend (list, file);
   animation = splash_image_pick_from_dirs (list,
@@ -546,24 +565,6 @@ splash_image_load (gint      max_width,
   return animation;
 }
 
-static GdkPixbufAnimation *
-splash_image_load_from_path (const gchar *filename,
-                             gint         max_width,
-                             gint         max_height,
-                             gboolean     be_verbose)
-{
-  GdkPixbufAnimation *animation;
-  GFile              *file;
-
-  file = g_file_new_for_path (filename);
-  animation = splash_image_load_from_file (file,
-                                           max_width, max_height,
-                                           be_verbose);
-  g_object_unref (file);
-
-  return animation;
-}
-
 static GdkPixbufAnimation *
 splash_image_load_from_file (GFile    *file,
                              gint      max_width,
@@ -576,15 +577,7 @@ splash_image_load_from_file (GFile    *file,
   gboolean            is_svg = FALSE;
 
   if (be_verbose)
-    {
-      gchar *path;
-
-      path = g_file_get_path (file);
-
-      g_printerr ("Trying splash '%s' ... ", path);
-
-      g_free (path);
-    }
+    g_printerr ("Trying splash '%s' ... ", g_file_peek_path (file));
 
   info = g_file_query_info (file,
                             G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
