@@ -351,14 +351,14 @@ gimp_font_get_standard (void)
 
 
 static inline gboolean
-gimp_font_covers_string (PangoFcFont *font,
+gimp_font_covers_string (PangoFont   *font,
                          const gchar *sample)
 {
   const gchar *p;
 
   for (p = sample; *p; p = g_utf8_next_char (p))
     {
-      if (! pango_fc_font_has_char (font, g_utf8_get_char (p)))
+      if (! pango_font_has_char (font, g_utf8_get_char (p)))
         return FALSE;
     }
 
@@ -389,6 +389,7 @@ gimp_font_get_sample_string (PangoContext         *context,
 {
   PangoFont        *font;
   hb_face_t        *hb_face;
+  hb_font_t        *hb_font;
   FT_Face           face;
   TT_OS2           *os2;
   PangoOTTableType  tt;
@@ -424,12 +425,40 @@ gimp_font_get_sample_string (PangoContext         *context,
     gint         bit;
     const gchar *sample;
   } scripts[] = {
-    /* Han is first because fonts that support it presumably are primarily
-     * designed for it.
+    /* CJK are first because fonts that support these characters
+     * presumably are primarily designed for it.
+     *
+     * Though not a universal rules, most fonts targetting Korean will
+     * also have the ideographs as they were used historically (and
+     * still in some cases). As for Japanese, ideographs are still
+     * definitely used daily. After all the block is called "*CJK*
+     * (Chinese-Japanese-Korean) Unified Ideographs". So let's give the
+     * Chinese representation less priority towards finding
+     * Korean/Japanese targetting.
+     * Then we prioritize Korean because many fonts for Korean also
+     * design Hiragana/Katakana blocks. On the other hand, I could see
+     * very few (none so far) Japanese fonts designing also Hangul
+     * characters. So if we see both Hiragana and Hangul, we can assume
+     * this is a font for Korean market, whereas Hiragana only is for
+     * Japanese.
+     * XXX: of course we could probably come with a better algorithm to
+     * determine the target language. Probably looking at more than the
+     * existence of a single block would help (since some languages are
+     * spread on several blocks).
      */
     {
-      "hani",                   /* Han Ideographic */
-      59,
+      "hang",                   /* Korean alphabet (Hangul)   */
+      56,                       /* Hangul Syllables block     */
+      "\355\225\234"            /* U+D55C HANGUL SYLLABLE HAN */
+    },
+    {
+      "japa",                   /* Japanese                   */
+      49,                       /* Hiragana block             */
+      "\343\201\202"            /* U+3042 HIRAGANA LETTER A   */
+    },
+    {
+      "hani",                   /* Han Ideographic            */
+      59,                       /* CJK Unified Ideographs     */
       "\346\260\270"            /* U+6C38 "forever". Ed Trager says
                                  * this is a "pan-stroke" often used
                                  * in teaching Chinese calligraphy. It
@@ -576,13 +605,6 @@ gimp_font_get_sample_string (PangoContext         *context,
                                   * U+10D0 GEORGIAN LETTER AN
                                   */
     },
-    {
-      "hang",                   /* Hangul */
-      28,
-      "\341\204\200\341\204\201"/* U+1100 HANGUL CHOSEONG KIYEOK
-                                 * U+1101 HANGUL CHOSEONG SSANGKIYEOK
-                                 */
-    },
     {
       "ethi",                   /* Ethiopic */
       75,
@@ -672,8 +694,28 @@ gimp_font_get_sample_string (PangoContext         *context,
 
   g_return_val_if_fail (PANGO_IS_FC_FONT (font), "Aa");
 
-  face = pango_fc_font_lock_face (PANGO_FC_FONT (font));
+  hb_font = pango_font_get_hb_font (font);
+  g_return_val_if_fail (hb_font != NULL, "Aa");
+
+  /* These are needed to set hb_font to the right internal format, which
+   * can only work if the font is not immutable. Hence we make a copy.
+   */
+  hb_font = hb_font_create_sub_font (hb_font);
+  hb_ft_font_set_funcs (hb_font);
+  /* TODO: use hb_ft_font_lock_face/hb_ft_font_unlock_face() when we
+   * bump to harfbuzz >= 2.6.5.
+   */
+  face = hb_ft_font_get_face (hb_font);
+
+  /* Are there actual cases where this function could return NULL while
+   * it's not a bug in the code?
+   * For instance if the font file is broken, we don't want to return a
+   * CRITICAL, but properly address the issue (removing the font with a
+   * warning or whatnot).
+   * See #5922.
+   */
   g_return_val_if_fail (face != NULL, "Aa");
+
   hb_face = hb_ft_face_create (face, NULL);
 
   /* First check what script(s), if any, the font has GSUB or GPOS
@@ -712,8 +754,7 @@ gimp_font_get_sample_string (PangoContext         *context,
 #define TAG(s) FT_MAKE_TAG (s[0], s[1], s[2], s[3])
 
                   if (slist[j] == TAG (scripts[i].script) &&
-                      gimp_font_covers_string (PANGO_FC_FONT (font),
-                                               scripts[i].sample))
+                      gimp_font_covers_string (font, scripts[i].sample))
                     {
                       ot_alts[n_ot_alts++] = i;
                       DEBUGPRINT (("%.4s ", scripts[i].script));
@@ -744,8 +785,7 @@ gimp_font_get_sample_string (PangoContext         *context,
         {
           if (scripts[i].bit >= 0 &&
               (&os2->ulUnicodeRange1)[scripts[i].bit/32] & (1 << (scripts[i].bit % 32)) &&
-              gimp_font_covers_string (PANGO_FC_FONT (font),
-                                       scripts[i].sample))
+              gimp_font_covers_string (font, scripts[i].sample))
             {
               sr_alts[n_sr_alts++] = i;
               DEBUGPRINT (("%.4s ", scripts[i].script));
@@ -753,8 +793,7 @@ gimp_font_get_sample_string (PangoContext         *context,
         }
     }
 
-  pango_fc_font_unlock_face (PANGO_FC_FONT (font));
-
+  hb_font_destroy (hb_font);
   g_object_unref (font);
 
   if (n_ot_alts > 2)
