@@ -93,7 +93,7 @@ gimp_generic_transform_tool_dialog (GimpTransformGridTool *tg_tool)
   GimpGenericTransformTool *generic = GIMP_GENERIC_TRANSFORM_TOOL (tg_tool);
   GtkWidget                *frame;
   GtkWidget                *vbox;
-  GtkWidget                *table;
+  GtkWidget                *grid;
   GtkWidget                *label;
   GtkSizeGroup             *size_group;
   gint                      x, y;
@@ -109,12 +109,12 @@ gimp_generic_transform_tool_dialog (GimpTransformGridTool *tg_tool)
 
   size_group = gtk_size_group_new (GTK_SIZE_GROUP_BOTH);
 
-  table = generic->matrix_table = gtk_table_new (3, 3, FALSE);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 2);
-  gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 0);
-  gtk_size_group_add_widget (size_group, table);
-  gtk_widget_show (table);
+  grid = generic->matrix_grid = gtk_grid_new ();
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 2);
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 2);
+  gtk_box_pack_start (GTK_BOX (vbox), grid, TRUE, TRUE, 0);
+  gtk_size_group_add_widget (size_group, grid);
+  gtk_widget_show (grid);
 
   for (y = 0; y < 3; y++)
     {
@@ -126,8 +126,7 @@ gimp_generic_transform_tool_dialog (GimpTransformGridTool *tg_tool)
           gimp_label_set_attributes (GTK_LABEL (label),
                                      PANGO_ATTR_SCALE, PANGO_SCALE_SMALL,
                                      -1);
-          gtk_table_attach (GTK_TABLE (table), label,
-                            x, x + 1, y, y + 1, GTK_EXPAND, GTK_FILL, 0, 0);
+          gtk_grid_attach (GTK_GRID (grid), label, x, y, 1, 1);
           gtk_widget_show (label);
         }
     }
@@ -156,7 +155,7 @@ gimp_generic_transform_tool_dialog_update (GimpTransformGridTool *tg_tool)
     {
       gint x, y;
 
-      gtk_widget_show (generic->matrix_table);
+      gtk_widget_show (generic->matrix_grid);
       gtk_widget_hide (generic->invalid_label);
 
       for (y = 0; y < 3; y++)
@@ -174,7 +173,7 @@ gimp_generic_transform_tool_dialog_update (GimpTransformGridTool *tg_tool)
   else
     {
       gtk_widget_show (generic->invalid_label);
-      gtk_widget_hide (generic->matrix_table);
+      gtk_widget_hide (generic->matrix_grid);
     }
 }
 
