@@ -25,6 +25,7 @@
 #include <gegl.h>
 #include <gtk/gtk.h>
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpcolor/gimpcolor.h"
 #include "libgimpconfig/gimpconfig.h"
 #include "libgimpwidgets/gimpwidgets.h"
@@ -71,10 +72,17 @@ static void     gimp_fg_bg_editor_get_property      (GObject          *object,
                                                      GValue           *value,
                                                      GParamSpec       *pspec);
 
-static void     gimp_fg_bg_editor_style_set         (GtkWidget        *widget,
-                                                     GtkStyle         *prev_style);
-static gboolean gimp_fg_bg_editor_expose            (GtkWidget        *widget,
-                                                     GdkEventExpose   *eevent);
+static GtkSizeRequestMode
+                gimp_fg_bg_editor_get_request_mode  (GtkWidget        *widget);
+static void
+   gimp_fg_bg_editor_get_preferred_width_for_height (GtkWidget        *widget,
+                                                     gint              height,
+                                                     gint             *minimum_width,
+                                                     gint             *natural_width);
+
+static void     gimp_fg_bg_editor_style_updated     (GtkWidget        *widget);
+static gboolean gimp_fg_bg_editor_draw              (GtkWidget        *widget,
+                                                     cairo_t          *cr);
 static gboolean gimp_fg_bg_editor_button_press      (GtkWidget        *widget,
                                                      GdkEventButton   *bevent);
 static gboolean gimp_fg_bg_editor_button_release    (GtkWidget        *widget,
@@ -133,8 +141,7 @@ gimp_fg_bg_editor_class_init (GimpFgBgEditorClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (GimpFgBgEditorClass, color_clicked),
-                  NULL, NULL,
-                  gimp_marshal_VOID__ENUM,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 1,
                   GIMP_TYPE_ACTIVE_COLOR);
 
@@ -153,12 +160,14 @@ gimp_fg_bg_editor_class_init (GimpFgBgEditorClass *klass)
   object_class->set_property         = gimp_fg_bg_editor_set_property;
   object_class->get_property         = gimp_fg_bg_editor_get_property;
 
-  widget_class->style_set            = gimp_fg_bg_editor_style_set;
-  widget_class->expose_event         = gimp_fg_bg_editor_expose;
-  widget_class->button_press_event   = gimp_fg_bg_editor_button_press;
-  widget_class->button_release_event = gimp_fg_bg_editor_button_release;
-  widget_class->drag_motion          = gimp_fg_bg_editor_drag_motion;
-  widget_class->query_tooltip        = gimp_fg_bg_editor_query_tooltip;
+  widget_class->get_request_mode               = gimp_fg_bg_editor_get_request_mode;
+  widget_class->get_preferred_width_for_height = gimp_fg_bg_editor_get_preferred_width_for_height;
+  widget_class->style_updated                  = gimp_fg_bg_editor_style_updated;
+  widget_class->draw                           = gimp_fg_bg_editor_draw;
+  widget_class->button_press_event             = gimp_fg_bg_editor_button_press;
+  widget_class->button_release_event           = gimp_fg_bg_editor_button_release;
+  widget_class->drag_motion                    = gimp_fg_bg_editor_drag_motion;
+  widget_class->query_tooltip                  = gimp_fg_bg_editor_query_tooltip;
 
   g_object_class_install_property (object_class, PROP_CONTEXT,
                                    g_param_spec_object ("context",
@@ -172,6 +181,8 @@ gimp_fg_bg_editor_class_init (GimpFgBgEditorClass *klass)
                                                       GIMP_TYPE_ACTIVE_COLOR,
                                                       GIMP_ACTIVE_COLOR_FOREGROUND,
                                                       GIMP_PARAM_READWRITE));
+
+  gtk_widget_class_set_css_name (widget_class, "GimpFgBgEditor");
 }
 
 static void
@@ -179,6 +190,7 @@ gimp_fg_bg_editor_init (GimpFgBgEditor *editor)
 {
   editor->active_color = GIMP_ACTIVE_COLOR_FOREGROUND;
 
+  gtk_widget_set_can_focus (GTK_WIDGET (editor), FALSE);
   gtk_event_box_set_visible_window (GTK_EVENT_BOX (editor), FALSE);
 
   gtk_widget_add_events (GTK_WIDGET (editor),
@@ -192,7 +204,9 @@ gimp_fg_bg_editor_init (GimpFgBgEditor *editor)
 
   gimp_widget_track_monitor (GTK_WIDGET (editor),
                              G_CALLBACK (gimp_fg_bg_editor_destroy_transform),
-                             NULL);
+                             NULL, NULL);
+
+  gtk_widget_set_size_request (GTK_WIDGET (editor), 32, 24);
 }
 
 static void
@@ -253,57 +267,84 @@ gimp_fg_bg_editor_get_property (GObject    *object,
     }
 }
 
+static GtkSizeRequestMode
+gimp_fg_bg_editor_get_request_mode (GtkWidget *widget)
+{
+  return GTK_SIZE_REQUEST_WIDTH_FOR_HEIGHT;
+}
+
 static void
-gimp_fg_bg_editor_style_set (GtkWidget *widget,
-                             GtkStyle  *prev_style)
+gimp_fg_bg_editor_get_preferred_width_for_height (GtkWidget *widget,
+                                                  gint       height,
+                                                  gint      *minimum_width,
+                                                  gint      *natural_width)
+{
+  *minimum_width = height;
+  *natural_width = height;
+}
+
+static void
+gimp_fg_bg_editor_style_updated (GtkWidget *widget)
 {
   GimpFgBgEditor *editor = GIMP_FG_BG_EDITOR (widget);
 
-  GTK_WIDGET_CLASS (parent_class)->style_set (widget, prev_style);
+  GTK_WIDGET_CLASS (parent_class)->style_updated (widget);
 
   g_clear_object (&editor->default_icon);
   g_clear_object (&editor->swap_icon);
 }
 
 static gboolean
-gimp_fg_bg_editor_expose (GtkWidget      *widget,
-                          GdkEventExpose *eevent)
+gimp_fg_bg_editor_draw (GtkWidget *widget,
+                        cairo_t   *cr)
 {
-  GimpFgBgEditor *editor = GIMP_FG_BG_EDITOR (widget);
-  cairo_t        *cr;
-  GtkAllocation   allocation;
-  gint            width, height;
-  gint            default_w, default_h;
-  gint            swap_w, swap_h;
-  gint            rect_w, rect_h;
-  GimpRGB         color;
+  GimpFgBgEditor  *editor = GIMP_FG_BG_EDITOR (widget);
+  GtkStyleContext *style  = gtk_widget_get_style_context (widget);
+  GtkBorder        border;
+  GtkBorder        padding;
+  GdkRectangle     rect;
+  gint             scale_factor;
+  gint             width, height;
+  gint             default_w, default_h;
+  gint             swap_w, swap_h;
+  GimpRGB          color;
 
-  if (! gtk_widget_is_drawable (widget))
-    return FALSE;
+  gtk_style_context_save (style);
 
-  cr = gdk_cairo_create (eevent->window);
-  gdk_cairo_region (cr, eevent->region);
-  cairo_clip (cr);
+  width  = gtk_widget_get_allocated_width  (widget);
+  height = gtk_widget_get_allocated_height (widget);
 
-  gtk_widget_get_allocation (widget, &allocation);
+  gtk_style_context_get_border (style, gtk_style_context_get_state (style),
+                                &border);
+  gtk_style_context_get_padding (style, gtk_style_context_get_state (style),
+                                 &padding);
 
-  width  = allocation.width;
-  height = allocation.height;
+  border.left   += padding.left;
+  border.right  += padding.right;
+  border.top    += padding.top;
+  border.bottom += padding.bottom;
 
-  cairo_translate (cr, allocation.x, allocation.y);
+  scale_factor = gtk_widget_get_scale_factor (widget);
 
   /*  draw the default colors pixbuf  */
   if (! editor->default_icon)
     editor->default_icon = gimp_widget_load_icon (widget,
                                                   GIMP_ICON_COLORS_DEFAULT, 12);
 
-  default_w = gdk_pixbuf_get_width  (editor->default_icon);
-  default_h = gdk_pixbuf_get_height (editor->default_icon);
+  default_w = gdk_pixbuf_get_width  (editor->default_icon) / scale_factor;
+  default_h = gdk_pixbuf_get_height (editor->default_icon) / scale_factor;
 
   if (default_w < width / 2 && default_h < height / 2)
     {
-      gdk_cairo_set_source_pixbuf (cr, editor->default_icon,
-                                   0, height - default_h);
+      cairo_surface_t *surface;
+
+      surface = gdk_cairo_surface_create_from_pixbuf (editor->default_icon,
+                                                      scale_factor, NULL);
+      cairo_set_source_surface (cr, surface,
+                                border.left,
+                                height - border.bottom - default_h);
+      cairo_surface_destroy (surface);
+
       cairo_paint (cr);
     }
   else
@@ -316,13 +357,20 @@ gimp_fg_bg_editor_expose (GtkWidget      *widget,
     editor->swap_icon = gimp_widget_load_icon (widget,
                                                GIMP_ICON_COLORS_SWAP, 12);
 
-  swap_w = gdk_pixbuf_get_width  (editor->swap_icon);
-  swap_h = gdk_pixbuf_get_height (editor->swap_icon);
+  swap_w = gdk_pixbuf_get_width  (editor->swap_icon) / scale_factor;
+  swap_h = gdk_pixbuf_get_height (editor->swap_icon) / scale_factor;
 
   if (swap_w < width / 2 && swap_h < height / 2)
     {
-      gdk_cairo_set_source_pixbuf (cr, editor->swap_icon,
-                                   width - swap_w, 0);
+      cairo_surface_t *surface;
+
+      surface = gdk_cairo_surface_create_from_pixbuf (editor->swap_icon,
+                                                      scale_factor, NULL);
+      cairo_set_source_surface (cr, surface,
+                                width - border.right - swap_w,
+                                border.top);
+      cairo_surface_destroy (surface);
+
       cairo_paint (cr);
     }
   else
@@ -330,15 +378,15 @@ gimp_fg_bg_editor_expose (GtkWidget      *widget,
       swap_w = swap_h = 0;
     }
 
-  rect_h = height - MAX (default_h, swap_h) - 2;
-  rect_w = width  - MAX (default_w, swap_w) - 4;
+  rect.width  = width  - MAX (default_w, swap_w) - 4 - border.top  - border.bottom;
+  rect.height = height - MAX (default_h, swap_h) - 2 - border.left - border.right;
 
-  if (rect_h > (height * 3 / 4))
-    rect_w = MAX (rect_w - (rect_h - ((height * 3 / 4))),
-                  width * 2 / 3);
+  if (rect.height > (height * 3 / 4))
+    rect.width = MAX (rect.width - (rect.height - ((height * 3 / 4))),
+                      width * 2 / 3);
 
-  editor->rect_width  = rect_w;
-  editor->rect_height = rect_h;
+  editor->rect_width  = rect.width;
+  editor->rect_height = rect.height;
 
   if (! editor->transform)
     gimp_fg_bg_editor_create_transform (editor);
@@ -347,21 +395,24 @@ gimp_fg_bg_editor_expose (GtkWidget      *widget,
     {
       /*  draw the background frame  */
       gimp_context_get_background (editor->context, &color);
+      rect.x = width  - rect.width  - border.right;
+      rect.y = height - rect.height - border.bottom;
       gimp_fg_bg_editor_draw_color_frame (editor, cr, &color,
-                                          width - rect_w, height - rect_h,
-                                          rect_w,         rect_h,
-                                          +1,             +1);
-
+                                          rect.x,     rect.y,
+                                          rect.width, rect.height,
+                                          +1,         +1);
 
       /*  draw the foreground frame  */
       gimp_context_get_foreground (editor->context, &color);
+      rect.x = border.left;
+      rect.y = border.top;
       gimp_fg_bg_editor_draw_color_frame (editor, cr, &color,
-                                          0,              0,
-                                          rect_w,         rect_h,
-                                          -1,             -1);
+                                          rect.x,     rect.y,
+                                          rect.width, rect.height,
+                                          +1,         +1);
     }
 
-  cairo_destroy (cr);
+  gtk_style_context_restore (style);
 
   return TRUE;
 }
@@ -371,33 +422,58 @@ gimp_fg_bg_editor_target (GimpFgBgEditor *editor,
                           gint            x,
                           gint            y)
 {
-  GtkAllocation allocation;
-  gint          width;
-  gint          height;
-  gint          rect_w = editor->rect_width;
-  gint          rect_h = editor->rect_height;
+  GtkWidget       *widget = GTK_WIDGET (editor);
+  GtkStyleContext *style  = gtk_widget_get_style_context (widget);
+  GtkBorder        border;
+  GtkBorder        padding;
+  gint             width;
+  gint             height;
+  gint             rect_w = editor->rect_width;
+  gint             rect_h = editor->rect_height;
+  gint             button_width;
+  gint             button_height;
 
-  gtk_widget_get_allocation (GTK_WIDGET (editor), &allocation);
+  width  = gtk_widget_get_allocated_width  (widget);
+  height = gtk_widget_get_allocated_height (widget);
 
-  width  = allocation.width;
-  height = allocation.height;
+  gtk_style_context_get_border (style, gtk_style_context_get_state (style),
+                                &border);
+  gtk_style_context_get_padding (style, gtk_style_context_get_state (style),
+                                 &padding);
 
-  if (x > 0 && x < rect_w && y > 0 && y < rect_h)
+  border.left   += padding.left;
+  border.right  += padding.right;
+  border.top    += padding.top;
+  border.bottom += padding.bottom;
+
+  button_width  = width  - border.left - border.right  - rect_w;
+  button_height = height - border.top  - border.bottom - rect_h;
+
+  if (x > border.left          &&
+      x < border.left + rect_w &&
+      y > border.top           &&
+      y < border.top + rect_h)
     {
       return GIMP_FG_BG_TARGET_FOREGROUND;
     }
-  else if (x > (width - rect_w)  && x < width  &&
-           y > (height - rect_h) && y < height)
+  else if (x > width  - border.right - rect_w  &&
+           x < width  - border.right           &&
+           y > height - border.bottom - rect_h &&
+           y < height - border.bottom)
     {
       return GIMP_FG_BG_TARGET_BACKGROUND;
     }
-  else if (x > 0      && x < (width - rect_w) &&
-           y > rect_h && y < height)
+  else if (x > border.left                &&
+           x < border.left + button_width &&
+           y > border.top + rect_h        &&
+           y < height - border.bottom)
     {
       return GIMP_FG_BG_TARGET_DEFAULT;
     }
-  else if (x > rect_w && x < width &&
-           y > 0      && y < (height - rect_h))
+  else if (x > border.left + rect_w &&
+           x < width - border.right &&
+           y > border.top           &&
+           y < border.top + button_height)
     {
       return GIMP_FG_BG_TARGET_SWAP;
     }
@@ -796,17 +872,19 @@ gimp_fg_bg_editor_draw_color_frame (GimpFgBgEditor *editor,
          ABS (color->r - color->b) > CHANNEL_EPSILON ||
          ABS (color->g - color->b) > CHANNEL_EPSILON))))
     {
-      gint corner_x = x + 0.5 * (1.0 + corner_dx) * width;
-      gint corner_y = y + 0.5 * (1.0 + corner_dy) * height;
-      gint side     = MIN (width, height) * 2 / 3;
+      gint    corner_x = x + 0.5 * (1.0 + corner_dx) * width;
+      gint    corner_y = y + 0.5 * (1.0 + corner_dy) * height;
+      gint    side     = MIN (width, height) * 2 / 3;
+      GimpRGB out_of_gamut_color;
 
       cairo_move_to (cr, corner_x, corner_y);
       cairo_line_to (cr, corner_x + side * corner_dx, corner_y);
       cairo_line_to (cr, corner_x, corner_y + side * corner_dy);
       cairo_close_path (cr);
 
-      gimp_cairo_set_source_rgb (cr,
-                                 &editor->color_config->out_of_gamut_color);
+      gimp_color_config_get_out_of_gamut_color (editor->color_config,
+                                                &out_of_gamut_color);
+      gimp_cairo_set_source_rgb (cr, &out_of_gamut_color);
       cairo_fill (cr);
     }
 
