@@ -25,6 +25,7 @@
 #include <gtk/gtk.h>
 
 #include "libgimpbase/gimpbase.h"
+#include "libgimpcolor/gimpcolor.h"
 #include "libgimpwidgets/gimpwidgets.h"
 
 #include "widgets-types.h"
@@ -72,6 +73,8 @@ static void        gimp_image_prop_view_undo_event   (GimpImage         *image,
                                                       GimpImagePropView *view);
 static void        gimp_image_prop_view_update       (GimpImagePropView *view);
 static void        gimp_image_prop_view_file_update  (GimpImagePropView *view);
+static void        gimp_image_prop_view_realize      (GimpImagePropView *view,
+                                                      gpointer           user_data);
 
 
 G_DEFINE_TYPE (GimpImagePropView, gimp_image_prop_view, GTK_TYPE_GRID)
@@ -126,6 +129,8 @@ gimp_image_prop_view_init (GimpImagePropView *view)
 
   gtk_label_set_ellipsize (GTK_LABEL (view->filename_label),
                            PANGO_ELLIPSIZE_MIDDLE);
+  /* See gimp_image_prop_view_realize(). */
+  gtk_label_set_max_width_chars (GTK_LABEL (view->filename_label), 25);
 
   view->filesize_label =
     gimp_image_prop_view_add_label (grid, row++, _("File Size:"));
@@ -158,6 +163,9 @@ gimp_image_prop_view_init (GimpImagePropView *view)
   view->vectors_label =
     gimp_image_prop_view_add_label (grid, row++, _("Number of paths:"));
 
+  g_signal_connect (view, "realize",
+                    G_CALLBACK (gimp_image_prop_view_realize),
+                    NULL);
 }
 
 static void
@@ -304,6 +312,9 @@ gimp_image_prop_view_label_set_filename (GtkWidget *label,
     {
       gtk_label_set_text (GTK_LABEL (label),
                           gimp_file_get_utf8_name (file));
+      /* In case the label is ellipsized. */
+      gtk_widget_set_tooltip_text (GTK_WIDGET (label),
+                                   gimp_file_get_utf8_name (file));
     }
   else
     {
@@ -409,6 +420,7 @@ static void
 gimp_image_prop_view_update (GimpImagePropView *view)
 {
   GimpImage         *image = view->image;
+  GimpColorProfile  *profile;
   GimpImageBaseType  type;
   GimpPrecision      precision;
   GimpUnit           unit;
@@ -453,9 +465,8 @@ gimp_image_prop_view_update (GimpImagePropView *view)
               unit == GIMP_UNIT_INCH ? _("ppi") : format_buf);
   gtk_label_set_text (GTK_LABEL (view->resolution_label), buf);
 
-  /*  color type  */
+  /*  color space  */
   type = gimp_image_get_base_type (image);
-
   gimp_enum_get_value (GIMP_TYPE_IMAGE_BASE_TYPE, type,
                        NULL, NULL, &desc, NULL);
 
@@ -463,16 +474,21 @@ gimp_image_prop_view_update (GimpImagePropView *view)
     {
     case GIMP_RGB:
     case GIMP_GRAY:
-      g_snprintf (buf, sizeof (buf), "%s", desc);
+      profile = gimp_color_managed_get_color_profile (GIMP_COLOR_MANAGED (image));
+      g_snprintf (buf, sizeof (buf), "%s: %s", desc,
+                  gimp_color_profile_get_label (profile));
       break;
     case GIMP_INDEXED:
       g_snprintf (buf, sizeof (buf),
-                  "%s (%d %s)", desc, gimp_image_get_colormap_size (image),
-                  _("colors"));
+                  ngettext ("Indexed color (monochrome)",
+                            "Indexed color (%d colors)",
+                            gimp_image_get_colormap_size (image)),
+                  gimp_image_get_colormap_size (image));
       break;
     }
 
   gtk_label_set_text (GTK_LABEL (view->colorspace_label), buf);
+  gtk_label_set_line_wrap (GTK_LABEL (view->colorspace_label), TRUE);
 
   /*  precision  */
   precision = gimp_image_get_precision (image);
@@ -528,3 +544,18 @@ gimp_image_prop_view_file_update (GimpImagePropView *view)
   /*  filetype  */
   gimp_image_prop_view_label_set_filetype (view->filetype_label, image);
 }
+
+static void
+gimp_image_prop_view_realize (GimpImagePropView *view,
+                              gpointer           user_data)
+{
+  /* Ugly trick to avoid extra-wide dialog at construction because of
+   * overlong file path. Basically I give a reasonnable max size at
+   * construction (if the path is longer, it is just ellipsized per set
+   * rules), then once the widget is realized, I remove the max size,
+   * allowing the widget to grow wider if ever the dialog were
+   * manually resized (we don't want to keep the label short and
+   * ellipsized if the dialog is explicitly made to have enough place).
+   */
+  gtk_label_set_max_width_chars (GTK_LABEL (view->filename_label), -1);
+}
