@@ -170,7 +170,7 @@ xcf_load_image (Gimp     *gimp,
   gint                width;
   gint                height;
   gint                image_type;
-  GimpPrecision       precision = GIMP_PRECISION_U8_GAMMA;
+  GimpPrecision       precision = GIMP_PRECISION_U8_NON_LINEAR;
   gint                num_successful_elements = 0;
   gint                n_broken_layers         = 0;
   gint                n_broken_channels       = 0;
@@ -183,19 +183,10 @@ xcf_load_image (Gimp     *gimp,
   xcf_read_int32 (info, (guint32 *) &width, 1);
   xcf_read_int32 (info, (guint32 *) &height, 1);
   xcf_read_int32 (info, (guint32 *) &image_type, 1);
-  if (image_type < GIMP_RGB || image_type > GIMP_INDEXED)
+  if (image_type < GIMP_RGB || image_type > GIMP_INDEXED ||
+      width <= 0 || height <= 0)
     goto hard_error;
 
-  /* Be lenient with corrupt image dimensions.
-   * Hopefully layer dimensions will be valid. */
-  if (width <= 0 || height <= 0 ||
-      width > GIMP_MAX_IMAGE_SIZE || height > GIMP_MAX_IMAGE_SIZE)
-    {
-      GIMP_LOG (XCF, "Invalid image size %d x %d, setting to 1x1.", width, height);
-      width  = 1;
-      height = 1;
-    }
-
   if (info->file_version >= 4)
     {
       gint p;
@@ -206,11 +197,11 @@ xcf_load_image (Gimp     *gimp,
         {
           switch (p)
             {
-            case 0: precision = GIMP_PRECISION_U8_GAMMA;     break;
-            case 1: precision = GIMP_PRECISION_U16_GAMMA;    break;
-            case 2: precision = GIMP_PRECISION_U32_LINEAR;   break;
-            case 3: precision = GIMP_PRECISION_HALF_LINEAR;  break;
-            case 4: precision = GIMP_PRECISION_FLOAT_LINEAR; break;
+            case 0: precision = GIMP_PRECISION_U8_NON_LINEAR;  break;
+            case 1: precision = GIMP_PRECISION_U16_NON_LINEAR; break;
+            case 2: precision = GIMP_PRECISION_U32_LINEAR;     break;
+            case 3: precision = GIMP_PRECISION_HALF_LINEAR;    break;
+            case 4: precision = GIMP_PRECISION_FLOAT_LINEAR;   break;
             default:
               goto hard_error;
             }
@@ -220,16 +211,16 @@ xcf_load_image (Gimp     *gimp,
         {
           switch (p)
             {
-            case 100: precision = GIMP_PRECISION_U8_LINEAR; break;
-            case 150: precision = GIMP_PRECISION_U8_GAMMA; break;
-            case 200: precision = GIMP_PRECISION_U16_LINEAR; break;
-            case 250: precision = GIMP_PRECISION_U16_GAMMA; break;
-            case 300: precision = GIMP_PRECISION_U32_LINEAR; break;
-            case 350: precision = GIMP_PRECISION_U32_GAMMA; break;
-            case 400: precision = GIMP_PRECISION_HALF_LINEAR; break;
-            case 450: precision = GIMP_PRECISION_HALF_GAMMA; break;
-            case 500: precision = GIMP_PRECISION_FLOAT_LINEAR; break;
-            case 550: precision = GIMP_PRECISION_FLOAT_GAMMA; break;
+            case 100: precision = GIMP_PRECISION_U8_LINEAR;        break;
+            case 150: precision = GIMP_PRECISION_U8_NON_LINEAR;    break;
+            case 200: precision = GIMP_PRECISION_U16_LINEAR;       break;
+            case 250: precision = GIMP_PRECISION_U16_NON_LINEAR;   break;
+            case 300: precision = GIMP_PRECISION_U32_LINEAR;       break;
+            case 350: precision = GIMP_PRECISION_U32_NON_LINEAR;   break;
+            case 400: precision = GIMP_PRECISION_HALF_LINEAR;      break;
+            case 450: precision = GIMP_PRECISION_HALF_NON_LINEAR;  break;
+            case 500: precision = GIMP_PRECISION_FLOAT_LINEAR;     break;
+            case 550: precision = GIMP_PRECISION_FLOAT_NON_LINEAR; break;
             default:
               goto hard_error;
             }
@@ -427,16 +418,10 @@ xcf_load_image (Gimp     *gimp,
                                             xmp_length - 10,
                                             &my_error))
             {
-              /* XMP metadata from 2.8.x or earlier can be really messed up.
-               * Let's make the message more user friendly so they will
-               * understand that we can't do anything about it.
-               * See issue #987. */
               gimp_message (gimp, G_OBJECT (info->progress),
                             GIMP_MESSAGE_WARNING,
-                            _("Corrupt XMP metadata saved by an older version of "
-                              "GIMP could not be converted and will be ignored.\n"
-                              "If you don't know what XMP is, you most likely don't "
-                              "need it. Reported error: %s."),
+                            _("Corrupt 'gimp-metadata' parasite discovered.\n"
+                              "XMP data could not be migrated: %s"),
                             my_error->message);
               g_clear_error (&my_error);
             }
@@ -485,13 +470,6 @@ xcf_load_image (Gimp     *gimp,
        */
       saved_pos = info->cp;
 
-      if (offset < saved_pos)
-        {
-          GIMP_LOG (XCF, "Invalid layer offset: %" G_GOFFSET_FORMAT
-                    " at offset: %" G_GOFFSET_FORMAT, offset, saved_pos);
-          goto error;
-        }
-
       /* seek to the layer offset */
       if (! xcf_seek_pos (info, offset, NULL))
         goto error;
@@ -632,13 +610,6 @@ xcf_load_image (Gimp     *gimp,
        */
       saved_pos = info->cp;
 
-      if (offset < saved_pos)
-        {
-          GIMP_LOG (XCF, "Invalid channel offset: %" G_GOFFSET_FORMAT
-                    " at offset: % "G_GOFFSET_FORMAT, offset, saved_pos);
-          goto error;
-        }
-
       /* seek to the channel offset */
       if (! xcf_seek_pos (info, offset, NULL))
         goto error;
@@ -648,7 +619,6 @@ xcf_load_image (Gimp     *gimp,
       if (!channel)
         {
           n_broken_channels++;
-          GIMP_LOG (XCF, "Failed to load channel.");
 
           if (! xcf_seek_pos (info, saved_pos, NULL))
             goto error;
@@ -678,10 +648,22 @@ xcf_load_image (Gimp     *gimp,
     xcf_load_add_masks (image);
 
   if (info->floating_sel && info->floating_sel_drawable)
-    floating_sel_attach (info->floating_sel, info->floating_sel_drawable);
+    {
+      /* we didn't fix the loaded floating selection's format before
+       * because we didn't know if it needed the layer space
+       */
+      if (GIMP_IS_LAYER (info->floating_sel_drawable) &&
+          gimp_drawable_is_gray (GIMP_DRAWABLE (info->floating_sel)))
+        gimp_layer_fix_format_space (info->floating_sel, TRUE, FALSE);
 
-  if (info->active_layer)
-    gimp_image_set_active_layer (image, info->active_layer);
+      floating_sel_attach (info->floating_sel, info->floating_sel_drawable);
+    }
+
+  if (info->selected_layers)
+    {
+      gimp_image_set_selected_layers (image, info->selected_layers);
+      g_clear_pointer (&info->selected_layers, g_list_free);
+    }
 
   if (info->active_channel)
     gimp_image_set_active_channel (image, info->active_channel);
@@ -1028,15 +1010,7 @@ xcf_load_image_props (XcfInfo   *info,
                 GError       *error = NULL;
 
                 if (! p)
-                  {
-                    gimp_message (info->gimp, G_OBJECT (info->progress),
-                                  GIMP_MESSAGE_WARNING,
-                                  "Invalid image parasite found. "
-                                  "Possibly corrupt XCF file.");
-
-                    xcf_seek_pos (info, base + prop_size, NULL);
-                    continue;
-                  }
+                  return FALSE;
 
                 if (! gimp_image_parasite_validate (image, p, &error))
                   {
@@ -1084,12 +1058,7 @@ xcf_load_image_props (XcfInfo   *info,
           break;
 
         case PROP_PATHS:
-          {
-            goffset base = info->cp;
-
-            if (! xcf_load_old_paths (info, image))
-              xcf_seek_pos (info, base + prop_size, NULL);
-          }
+          xcf_load_old_paths (info, image);
           break;
 
         case PROP_USER_UNIT:
@@ -1205,7 +1174,7 @@ xcf_load_layer_props (XcfInfo    *info,
           return TRUE;
 
         case PROP_ACTIVE_LAYER:
-          info->active_layer = *layer;
+          info->selected_layers = g_list_prepend (info->selected_layers, *layer);
           break;
 
         case PROP_FLOATING_SELECTION:
@@ -1484,15 +1453,15 @@ xcf_load_layer_props (XcfInfo    *info,
         case PROP_GROUP_ITEM:
           {
             GimpLayer *group;
-            gboolean   is_active_layer;
+            gboolean   is_selected_layer;
 
             /* We're going to delete *layer, Don't leave its pointers
              * in @info.  After that, we'll restore them back with the
              * new pointer. See bug #767873.
              */
-            is_active_layer = (*layer == info->active_layer);
-            if (is_active_layer)
-              info->active_layer = NULL;
+            is_selected_layer = (g_list_find (info->selected_layers, *layer ) != NULL);
+            if (is_selected_layer)
+              info->selected_layers = g_list_remove (info->selected_layers, *layer);
 
             if (*layer == info->floating_sel)
               info->floating_sel = NULL;
@@ -1506,8 +1475,8 @@ xcf_load_layer_props (XcfInfo    *info,
             g_object_unref (*layer);
             *layer = group;
 
-            if (is_active_layer)
-              info->active_layer = *layer;
+            if (is_selected_layer)
+              info->selected_layers = g_list_prepend (info->selected_layers, *layer);
 
             /* Don't restore info->floating_sel because group layers
              * can't be floating selections
@@ -1877,8 +1846,6 @@ xcf_load_prop (XcfInfo  *info,
   if (G_UNLIKELY (xcf_read_int32 (info, (guint32 *) prop_size, 1) != 4))
     return FALSE;
 
-  GIMP_LOG (XCF, "prop type=%d size=%u", *prop_type, *prop_size);
-
   return TRUE;
 }
 
@@ -1894,7 +1861,7 @@ xcf_load_layer (XcfInfo    *info,
   gboolean           apply_mask = TRUE;
   gboolean           edit_mask  = FALSE;
   gboolean           show_mask  = FALSE;
-  gboolean           active;
+  GList             *selected;
   gboolean           floating;
   guint32            group_layer_flags = 0;
   guint32            text_layer_flags = 0;
@@ -1906,7 +1873,6 @@ xcf_load_layer (XcfInfo    *info,
   const Babl        *format;
   gboolean           is_fs_drawable;
   gchar             *name;
-  goffset            cur_offset;
 
   /* check and see if this is the drawable the floating selection
    *  is attached to. if it is then we'll do the attachment in our caller.
@@ -1958,8 +1924,7 @@ xcf_load_layer (XcfInfo    *info,
       return NULL;
     }
 
-  if (width <= 0 || height <= 0 ||
-      width > GIMP_MAX_IMAGE_SIZE || height > GIMP_MAX_IMAGE_SIZE)
+  if (width <= 0 || height <= 0)
     {
       gboolean is_group_layer = FALSE;
       gboolean is_text_layer  = FALSE;
@@ -1983,12 +1948,20 @@ xcf_load_layer (XcfInfo    *info,
         }
     }
 
-  /* do not use gimp_image_get_layer_format() because it might
-   * be the floating selection of a channel or mask
-   */
-  format = gimp_image_get_format (image, base_type,
-                                  gimp_image_get_precision (image),
-                                  has_alpha);
+  if (base_type == GIMP_GRAY)
+    {
+      /* do not use gimp_image_get_layer_format() because it might
+       * be the floating selection of a channel or mask
+       */
+      format = gimp_image_get_format (image, base_type,
+                                      gimp_image_get_precision (image),
+                                      has_alpha,
+                                      NULL /* we will fix the space later */);
+    }
+  else
+    {
+      format = gimp_image_get_layer_format (image, has_alpha);
+    }
 
   /* create a new layer */
   layer = gimp_layer_new (image, width, height,
@@ -2009,7 +1982,7 @@ xcf_load_layer (XcfInfo    *info,
   xcf_progress_update (info);
 
   /* call the evil text layer hack that might change our layer pointer */
-  active   = (info->active_layer == layer);
+  selected = g_list_find (info->selected_layers, layer);
   floating = (info->floating_sel == layer);
 
   if (gimp_text_layer_xcf_load_hack (&layer))
@@ -2017,14 +1990,23 @@ xcf_load_layer (XcfInfo    *info,
       gimp_text_layer_set_xcf_flags (GIMP_TEXT_LAYER (layer),
                                      text_layer_flags);
 
-      if (active)
-        info->active_layer = layer;
+      if (selected)
+        {
+          info->selected_layers = g_list_delete_link (info->selected_layers, selected);
+          info->selected_layers = g_list_prepend (info->selected_layers, layer);
+        }
       if (floating)
         info->floating_sel = layer;
     }
 
+  /* if this is not the floating selection, we can fix the layer's
+   * space already now, the function will do nothing if we already
+   * created the layer with the right format
+   */
+  if (! floating && base_type == GIMP_GRAY)
+    gimp_layer_fix_format_space (layer, FALSE, FALSE);
+
   /* read the hierarchy and layer mask offsets */
-  cur_offset = info->cp;
   xcf_read_offset (info, &hierarchy_offset,  1);
   xcf_read_offset (info, &layer_mask_offset, 1);
 
@@ -2034,11 +2016,6 @@ xcf_load_layer (XcfInfo    *info,
    */
   if (! gimp_viewable_get_children (GIMP_VIEWABLE (layer)))
     {
-      if (hierarchy_offset < cur_offset)
-        {
-          GIMP_LOG (XCF, "Invalid layer hierarchy offset!");
-          goto error;
-        }
       if (! xcf_seek_pos (info, hierarchy_offset, NULL))
         goto error;
 
@@ -2062,11 +2039,6 @@ xcf_load_layer (XcfInfo    *info,
   /* read in the layer mask */
   if (layer_mask_offset != 0)
     {
-      if (layer_mask_offset < cur_offset)
-        {
-          GIMP_LOG (XCF, "Invalid layer mask offset!");
-          goto error;
-        }
       if (! xcf_seek_pos (info, layer_mask_offset, NULL))
         goto error;
 
@@ -2098,8 +2070,7 @@ xcf_load_layer (XcfInfo    *info,
   return layer;
 
  error:
-  if (info->active_layer == layer)
-    info->active_layer = NULL;
+  info->selected_layers = g_list_remove (info->selected_layers, layer);
 
   if (info->floating_sel == layer)
     info->floating_sel = NULL;
@@ -2123,7 +2094,6 @@ xcf_load_channel (XcfInfo   *info,
   gboolean     is_fs_drawable;
   gchar       *name;
   GimpRGB      color = { 0.0, 0.0, 0.0, GIMP_OPACITY_OPAQUE };
-  goffset      cur_offset;
 
   /* check and see if this is the drawable the floating selection
    *  is attached to. if it is then we'll do the attachment in our caller.
@@ -2133,16 +2103,10 @@ xcf_load_channel (XcfInfo   *info,
   /* read in the layer width, height and name */
   xcf_read_int32 (info, (guint32 *) &width,  1);
   xcf_read_int32 (info, (guint32 *) &height, 1);
-  if (width <= 0 || height <= 0 ||
-      width > GIMP_MAX_IMAGE_SIZE || height > GIMP_MAX_IMAGE_SIZE)
-    {
-      GIMP_LOG (XCF, "Invalid channel size %d x %d.", width, height);
-      return NULL;
-    }
+  if (width <= 0 || height <= 0)
+    return NULL;
 
   xcf_read_string (info, &name, 1);
-  GIMP_LOG (XCF, "Channel width=%d, height=%d, name='%s'",
-            width, height, name);
 
   /* create a new channel */
   channel = gimp_channel_new (image, width, height, name, &color);
@@ -2156,16 +2120,9 @@ xcf_load_channel (XcfInfo   *info,
 
   xcf_progress_update (info);
 
-  /* read the hierarchy offset */
-  cur_offset = info->cp;
+  /* read the hierarchy and layer mask offsets */
   xcf_read_offset (info, &hierarchy_offset, 1);
 
-  if (hierarchy_offset < cur_offset)
-    {
-      GIMP_LOG (XCF, "Invalid hierarchy offset!");
-      goto error;
-    }
-
   /* read in the hierarchy */
   if (! xcf_seek_pos (info, hierarchy_offset, NULL))
     goto error;
@@ -2209,7 +2166,6 @@ xcf_load_layer_mask (XcfInfo   *info,
   gboolean       is_fs_drawable;
   gchar         *name;
   GimpRGB        color = { 0.0, 0.0, 0.0, GIMP_OPACITY_OPAQUE };
-  goffset        cur_offset;
 
   /* check and see if this is the drawable the floating selection
    *  is attached to. if it is then we'll do the attachment in our caller.
@@ -2219,16 +2175,10 @@ xcf_load_layer_mask (XcfInfo   *info,
   /* read in the layer width, height and name */
   xcf_read_int32 (info, (guint32 *) &width,  1);
   xcf_read_int32 (info, (guint32 *) &height, 1);
-  if (width <= 0 || height <= 0 ||
-      width > GIMP_MAX_IMAGE_SIZE || height > GIMP_MAX_IMAGE_SIZE)
-    {
-      GIMP_LOG (XCF, "Invalid layer mask size %d x %d.", width, height);
-      return NULL;
-    }
+  if (width <= 0 || height <= 0)
+    return NULL;
 
   xcf_read_string (info, &name, 1);
-  GIMP_LOG (XCF, "Layer mask width=%d, height=%d, name='%s'",
-            width, height, name);
 
   /* create a new layer mask */
   layer_mask = gimp_layer_mask_new (image, width, height, name, &color);
@@ -2243,16 +2193,9 @@ xcf_load_layer_mask (XcfInfo   *info,
 
   xcf_progress_update (info);
 
-  /* read the hierarchy offset */
-  cur_offset = info->cp;
+  /* read the hierarchy and layer mask offsets */
   xcf_read_offset (info, &hierarchy_offset, 1);
 
-  if (hierarchy_offset < cur_offset)
-    {
-      GIMP_LOG (XCF, "Invalid hierarchy offset!");
-      goto error;
-    }
-
   /* read in the hierarchy */
   if (! xcf_seek_pos (info, hierarchy_offset, NULL))
     goto error;
@@ -2290,7 +2233,6 @@ xcf_load_buffer (XcfInfo    *info,
   gint        width;
   gint        height;
   gint        bpp;
-  goffset     cur_offset;
 
   format = gegl_buffer_get_format (buffer);
 
@@ -2306,15 +2248,8 @@ xcf_load_buffer (XcfInfo    *info,
       bpp    != babl_format_get_bytes_per_pixel (format))
     return FALSE;
 
-  cur_offset = info->cp;
   xcf_read_offset (info, &offset, 1); /* top level */
 
-  if (offset < cur_offset)
-    {
-      GIMP_LOG (XCF, "Invalid buffer offset!");
-      return FALSE;
-    }
-
   /* seek to the level offset */
   if (! xcf_seek_pos (info, offset, NULL))
     return FALSE;
@@ -2768,18 +2703,16 @@ xcf_load_tile_zlib (XcfInfo       *info,
 static GimpParasite *
 xcf_load_parasite (XcfInfo *info)
 {
-  GimpParasite *parasite = NULL;
+  GimpParasite *parasite;
   gchar        *name;
   guint32       flags;
-  guint32       size, size_read;
+  guint32       size;
   gpointer      data;
 
   xcf_read_string (info, &name,  1);
   xcf_read_int32  (info, &flags, 1);
   xcf_read_int32  (info, &size,  1);
 
-  GIMP_LOG (XCF, "Parasite name: %s, flags: %d, size: %d", name, flags, size);
-
   if (size > MAX_XCF_PARASITE_DATA_LEN)
     {
       g_printerr ("Maximum parasite data length (%ld bytes) exceeded. "
@@ -2788,25 +2721,10 @@ xcf_load_parasite (XcfInfo *info)
       return NULL;
     }
 
-  if (!name)
-    {
-      g_printerr ("Parasite has no name! Possibly corrupt XCF file.\n");
-      return NULL;
-    }
-
   data = g_new (gchar, size);
-  size_read = xcf_read_int8 (info, data, size);
+  xcf_read_int8 (info, data, size);
 
-  if (size_read != size)
-    {
-      g_printerr ("Incorrect parasite data size: read %u bytes instead of %u. "
-                  "Possibly corrupt XCF file.\n",
-                  size_read, size);
-    }
-  else
-    {
-      parasite = gimp_parasite_new (name, flags, size, data);
-    }
+  parasite = gimp_parasite_new (name, flags, size, data);
 
   g_free (name);
   g_free (data);
@@ -2825,11 +2743,8 @@ xcf_load_old_paths (XcfInfo   *info,
   xcf_read_int32 (info, &last_selected_row, 1);
   xcf_read_int32 (info, &num_paths,         1);
 
-  GIMP_LOG (XCF, "Number of old paths: %u", num_paths);
-
   while (num_paths-- > 0)
-    if (! xcf_load_old_path (info, image))
-      return FALSE;
+    xcf_load_old_path (info, image);
 
   active_vectors =
     GIMP_VECTORS (gimp_container_get_child_by_index (gimp_image_get_vectors (image),
@@ -2880,7 +2795,7 @@ xcf_load_old_path (XcfInfo   *info,
     }
   else if (version != 1)
     {
-      g_printerr ("Unknown path type (version: %u). Possibly corrupt XCF file.\n", version);
+      g_printerr ("Unknown path type. Possibly corrupt XCF file");
 
       return FALSE;
     }
@@ -3058,7 +2973,7 @@ xcf_load_vector (XcfInfo   *info,
       guint32      num_axes;
       guint32      num_control_points;
       guint32      type;
-      gfloat       coords[10] = GIMP_COORDS_DEFAULT_VALUES;
+      gfloat       coords[13] = GIMP_COORDS_DEFAULT_VALUES;
       GimpStroke  *stroke;
       gint         j;
 
