@@ -445,7 +445,7 @@ gimp_procedure_config_set_parasite (GimpProcedureConfig *config,
  * If @run_mode is %GIMP_RUN_INTERACTIVE or %GIMP_RUN_WITH_LAST_VALS,
  * the saved values from the procedure's last run() are loaded and set
  * on @config. If @image is not %NULL, the last used values for this
- * image are tried first, and if no image-spesicic values are found
+ * image are tried first, and if no image-specific values are found
  * the globally saved last used values are used. If no saved last used
  * values are found, the procedure's default argument values are used.
  *
@@ -461,7 +461,7 @@ gimp_procedure_config_set_parasite (GimpProcedureConfig *config,
  * String properties are set to the value of the image parasite if
  * @run_mode is %GIMP_RUN_INTERACTIVE or %GIMP_RUN_WITH_LAST_VALS, or
  * if the corresponding argument is an auxiliary argument. As a
- * special case, a propery named "gimp-comment" will default to
+ * special case, a property named "gimp-comment" will default to
  * gimp_get_default_comment() if there is no "gimp-comment" parasite.
  *
  * After calling this function, the @args passed to run() should be
@@ -576,8 +576,8 @@ gimp_procedure_config_begin_run (GimpProcedureConfig  *config,
  *
  * If the @run_mode passed to gimp_procedure_config_end_run() was
  * %GIMP_RUN_INTERACTIVE, @config is saved as last used values to be
- * used when the procedure runs again. Additionally, the @image passed
- * gimp_procedure_config_begin_run() was not %NULL, @config is
+ * used when the procedure runs again. Additionally, if the #GimpImage
+ * passed to gimp_procedure_config_begin_run() was not %NULL, @config is
  * attached to @image as last used values for this image using a
  * #GimpParasite and gimp_image_attach_parasite().
  *
@@ -846,6 +846,7 @@ gimp_procedure_config_save_metadata (GimpProcedureConfig *config,
   if (config->priv->metadata && ! config->priv->metadata_saved)
     {
       GObjectClass *object_class = G_OBJECT_GET_CLASS (config);
+      GError       *error        = NULL;
       gint          i;
 
       for (i = 0; i < G_N_ELEMENTS (metadata_properties); i++)
@@ -869,11 +870,22 @@ gimp_procedure_config_save_metadata (GimpProcedureConfig *config,
             }
         }
 
-      gimp_image_metadata_save_finish (exported_image,
-                                       config->priv->mime_type,
-                                       config->priv->metadata,
-                                       config->priv->metadata_flags,
-                                       file, NULL);
+      if (! gimp_image_metadata_save_finish (exported_image,
+                                             config->priv->mime_type,
+                                             config->priv->metadata,
+                                             config->priv->metadata_flags,
+                                             file, &error))
+        {
+          if (error)
+            {
+              /* Even though a failure to write metadata is not enough
+                 reason to say we failed to save the image, we should
+                 still notify the user about the problem. */
+              g_message ("%s: saving metadata failed: %s",
+                         G_STRFUNC, error->message);
+              g_error_free (error);
+            }
+        }
 
       config->priv->metadata_saved = TRUE;
     }
