@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -105,12 +105,6 @@ gimp_config_get_type (void)
   return config_iface_type;
 }
 
-GType
-gimp_config_interface_get_type (void)
-{
-  return gimp_config_get_type ();
-}
-
 static void
 gimp_config_iface_default_init (GimpConfigInterface *iface)
 {
@@ -226,8 +220,11 @@ gimp_config_iface_equal (GimpConfig *a,
 
       prop_spec = property_specs[i];
 
-      if (! (prop_spec->flags & G_PARAM_READABLE))
-        continue;
+      if (! (prop_spec->flags & G_PARAM_READABLE) ||
+            (prop_spec->flags & GIMP_CONFIG_PARAM_DONT_COMPARE))
+        {
+          continue;
+        }
 
       g_value_init (&a_value, prop_spec->value_type);
       g_value_init (&b_value, prop_spec->value_type);
@@ -283,47 +280,6 @@ gimp_config_iface_copy (GimpConfig  *src,
 /**
  * gimp_config_serialize_to_file:
  * @config: a #GObject that implements the #GimpConfigInterface.
- * @filename: the name of the file to write the configuration to.
- * @header: optional file header (must be ASCII only)
- * @footer: optional file footer (must be ASCII only)
- * @data: user data passed to the serialize implementation.
- * @error: return location for a possible error
- *
- * Serializes the object properties of @config to the file specified
- * by @filename. If a file with that name already exists, it is
- * overwritten. Basically this function opens @filename for you and
- * calls the serialize function of the @config's #GimpConfigInterface.
- *
- * Return value: %TRUE if serialization succeeded, %FALSE otherwise.
- *
- * Since: 2.4
- **/
-gboolean
-gimp_config_serialize_to_file (GimpConfig   *config,
-                               const gchar  *filename,
-                               const gchar  *header,
-                               const gchar  *footer,
-                               gpointer      data,
-                               GError      **error)
-{
-  GimpConfigWriter *writer;
-
-  g_return_val_if_fail (GIMP_IS_CONFIG (config), FALSE);
-  g_return_val_if_fail (filename != NULL, FALSE);
-  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-
-  writer = gimp_config_writer_new_file (filename, TRUE, header, error);
-  if (!writer)
-    return FALSE;
-
-  GIMP_CONFIG_GET_INTERFACE (config)->serialize (config, writer, data);
-
-  return gimp_config_writer_finish (writer, footer, error);
-}
-
-/**
- * gimp_config_serialize_to_gfile:
- * @config: a #GObject that implements the #GimpConfigInterface.
  * @file:   the #GFile to write the configuration to.
  * @header: optional file header (must be ASCII only)
  * @footer: optional file footer (must be ASCII only)
@@ -335,17 +291,17 @@ gimp_config_serialize_to_file (GimpConfig   *config,
  * overwritten. Basically this function opens @file for you and calls
  * the serialize function of the @config's #GimpConfigInterface.
  *
- * Return value: %TRUE if serialization succeeded, %FALSE otherwise.
+ * Returns: %TRUE if serialization succeeded, %FALSE otherwise.
  *
  * Since: 2.10
  **/
 gboolean
-gimp_config_serialize_to_gfile (GimpConfig   *config,
-                                GFile        *file,
-                                const gchar  *header,
-                                const gchar  *footer,
-                                gpointer      data,
-                                GError      **error)
+gimp_config_serialize_to_file (GimpConfig   *config,
+                               GFile        *file,
+                               const gchar  *header,
+                               const gchar  *footer,
+                               gpointer      data,
+                               GError      **error)
 {
   GimpConfigWriter *writer;
 
@@ -353,11 +309,11 @@ gimp_config_serialize_to_gfile (GimpConfig   *config,
   g_return_val_if_fail (G_IS_FILE (file), FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  writer = gimp_config_writer_new_gfile (file, TRUE, header, error);
+  writer = gimp_config_writer_new_from_file (file, TRUE, header, error);
   if (!writer)
     return FALSE;
 
-  GIMP_CONFIG_GET_INTERFACE (config)->serialize (config, writer, data);
+  GIMP_CONFIG_GET_IFACE (config)->serialize (config, writer, data);
 
   return gimp_config_writer_finish (writer, footer, error);
 }
@@ -374,7 +330,7 @@ gimp_config_serialize_to_gfile (GimpConfig   *config,
  * Serializes the object properties of @config to the stream specified
  * by @output.
  *
- * Return value: %TRUE if serialization succeeded, %FALSE otherwise.
+ * Returns: %TRUE if serialization succeeded, %FALSE otherwise.
  *
  * Since: 2.10
  **/
@@ -392,11 +348,11 @@ gimp_config_serialize_to_stream (GimpConfig     *config,
   g_return_val_if_fail (G_IS_OUTPUT_STREAM (output), FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  writer = gimp_config_writer_new_stream (output, header, error);
+  writer = gimp_config_writer_new_from_stream (output, header, error);
   if (!writer)
     return FALSE;
 
-  GIMP_CONFIG_GET_INTERFACE (config)->serialize (config, writer, data);
+  GIMP_CONFIG_GET_IFACE (config)->serialize (config, writer, data);
 
   return gimp_config_writer_finish (writer, footer, error);
 }
@@ -410,7 +366,7 @@ gimp_config_serialize_to_stream (GimpConfig     *config,
  * Serializes the object properties of @config to the given file
  * descriptor.
  *
- * Return value: %TRUE if serialization succeeded, %FALSE otherwise.
+ * Returns: %TRUE if serialization succeeded, %FALSE otherwise.
  *
  * Since: 2.4
  **/
@@ -424,11 +380,11 @@ gimp_config_serialize_to_fd (GimpConfig *config,
   g_return_val_if_fail (GIMP_IS_CONFIG (config), FALSE);
   g_return_val_if_fail (fd > 0, FALSE);
 
-  writer = gimp_config_writer_new_fd (fd);
+  writer = gimp_config_writer_new_from_fd (fd);
   if (!writer)
     return FALSE;
 
-  GIMP_CONFIG_GET_INTERFACE (config)->serialize (config, writer, data);
+  GIMP_CONFIG_GET_IFACE (config)->serialize (config, writer, data);
 
   return gimp_config_writer_finish (writer, NULL, NULL);
 }
@@ -440,7 +396,7 @@ gimp_config_serialize_to_fd (GimpConfig *config,
  *
  * Serializes the object properties of @config to a string.
  *
- * Return value: a newly allocated NUL-terminated string.
+ * Returns: a newly allocated NUL-terminated string.
  *
  * Since: 2.4
  **/
@@ -454,9 +410,9 @@ gimp_config_serialize_to_string (GimpConfig *config,
   g_return_val_if_fail (GIMP_IS_CONFIG (config), NULL);
 
   str = g_string_new (NULL);
-  writer = gimp_config_writer_new_string (str);
+  writer = gimp_config_writer_new_from_string (str);
 
-  GIMP_CONFIG_GET_INTERFACE (config)->serialize (config, writer, data);
+  GIMP_CONFIG_GET_IFACE (config)->serialize (config, writer, data);
 
   gimp_config_writer_finish (writer, NULL, NULL);
 
@@ -464,55 +420,47 @@ gimp_config_serialize_to_string (GimpConfig *config,
 }
 
 /**
- * gimp_config_deserialize_file:
- * @config: a #GObject that implements the #GimpConfigInterface.
- * @filename: the name of the file to read configuration from.
- * @data: user data passed to the deserialize implementation.
- * @error: return location for a possible error
+ * gimp_config_serialize_to_parasite:
+ * @config:         a #GObject that implements the #GimpConfigInterface.
+ * @parasite_name:  the new parasite's name
+ * @parasite_flags: the new parasite's flags
+ * @data:           user data passed to the serialize implementation.
  *
- * Opens the file specified by @filename, reads configuration data
- * from it and configures @config accordingly. Basically this function
- * creates a properly configured #GScanner for you and calls the
- * deserialize function of the @config's #GimpConfigInterface.
+ * Serializes the object properties of @config to a #GimpParasite.
  *
- * Return value: %TRUE if deserialization succeeded, %FALSE otherwise.
+ * Returns: (transfer full): the newly allocated #GimpParasite.
  *
- * Since: 2.4
+ * Since: 3.0
  **/
-gboolean
-gimp_config_deserialize_file (GimpConfig   *config,
-                              const gchar  *filename,
-                              gpointer      data,
-                              GError      **error)
+GimpParasite *
+gimp_config_serialize_to_parasite (GimpConfig  *config,
+                                   const gchar *parasite_name,
+                                   guint        parasite_flags,
+                                   gpointer     data)
 {
-  GScanner *scanner;
-  gboolean  success;
+  GimpParasite *parasite;
+  gchar        *str;
 
-  g_return_val_if_fail (GIMP_IS_CONFIG (config), FALSE);
-  g_return_val_if_fail (filename != NULL, FALSE);
-  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+  g_return_val_if_fail (GIMP_IS_CONFIG (config), NULL);
+  g_return_val_if_fail (parasite_name != NULL, NULL);
 
-  scanner = gimp_scanner_new_file (filename, error);
-  if (! scanner)
-    return FALSE;
+  str = gimp_config_serialize_to_string (config, data);
 
-  g_object_freeze_notify (G_OBJECT (config));
+  if (! str)
+    return NULL;
 
-  success = GIMP_CONFIG_GET_INTERFACE (config)->deserialize (config,
-                                                             scanner, 0, data);
+  parasite = gimp_parasite_new (parasite_name,
+                                parasite_flags,
+                                0, NULL);
 
-  g_object_thaw_notify (G_OBJECT (config));
+  parasite->size = strlen (str) + 1;
+  parasite->data = str;
 
-  gimp_scanner_destroy (scanner);
-
-  if (! success)
-    g_assert (error == NULL || *error != NULL);
-
-  return success;
+  return parasite;
 }
 
 /**
- * gimp_config_deserialize_gfile:
+ * gimp_config_deserialize_file:
  * @config: a #GObject that implements the #GimpConfigInterface.
  * @file: the #GFile to read configuration from.
  * @data: user data passed to the deserialize implementation.
@@ -523,15 +471,15 @@ gimp_config_deserialize_file (GimpConfig   *config,
  * a properly configured #GScanner for you and calls the deserialize
  * function of the @config's #GimpConfigInterface.
  *
- * Return value: %TRUE if deserialization succeeded, %FALSE otherwise.
+ * Returns: %TRUE if deserialization succeeded, %FALSE otherwise.
  *
  * Since: 2.10
  **/
 gboolean
-gimp_config_deserialize_gfile (GimpConfig  *config,
-                               GFile       *file,
-                               gpointer     data,
-                               GError     **error)
+gimp_config_deserialize_file (GimpConfig  *config,
+                              GFile       *file,
+                              gpointer     data,
+                              GError     **error)
 {
   GScanner *scanner;
   gboolean  success;
@@ -540,18 +488,18 @@ gimp_config_deserialize_gfile (GimpConfig  *config,
   g_return_val_if_fail (G_IS_FILE (file), FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  scanner = gimp_scanner_new_gfile (file, error);
+  scanner = gimp_scanner_new_file (file, error);
   if (! scanner)
     return FALSE;
 
   g_object_freeze_notify (G_OBJECT (config));
 
-  success = GIMP_CONFIG_GET_INTERFACE (config)->deserialize (config,
-                                                             scanner, 0, data);
+  success = GIMP_CONFIG_GET_IFACE (config)->deserialize (config,
+                                                         scanner, 0, data);
 
   g_object_thaw_notify (G_OBJECT (config));
 
-  gimp_scanner_destroy (scanner);
+  gimp_scanner_unref (scanner);
 
   if (! success)
     g_assert (error == NULL || *error != NULL);
@@ -571,7 +519,7 @@ gimp_config_deserialize_gfile (GimpConfig  *config,
  * #GScanner for you and calls the deserialize function of the
  * @config's #GimpConfigInterface.
  *
- * Return value: %TRUE if deserialization succeeded, %FALSE otherwise.
+ * Returns: %TRUE if deserialization succeeded, %FALSE otherwise.
  *
  * Since: 2.10
  **/
@@ -594,12 +542,12 @@ gimp_config_deserialize_stream (GimpConfig    *config,
 
   g_object_freeze_notify (G_OBJECT (config));
 
-  success = GIMP_CONFIG_GET_INTERFACE (config)->deserialize (config,
-                                                             scanner, 0, data);
+  success = GIMP_CONFIG_GET_IFACE (config)->deserialize (config,
+                                                         scanner, 0, data);
 
   g_object_thaw_notify (G_OBJECT (config));
 
-  gimp_scanner_destroy (scanner);
+  gimp_scanner_unref (scanner);
 
   if (! success)
     g_assert (error == NULL || *error != NULL);
@@ -610,7 +558,7 @@ gimp_config_deserialize_stream (GimpConfig    *config,
 /**
  * gimp_config_deserialize_string:
  * @config:   a #GObject that implements the #GimpConfigInterface.
- * @text:     string to deserialize (in UTF-8 encoding)
+ * @text: (array length=text_len): string to deserialize (in UTF-8 encoding)
  * @text_len: length of @text in bytes or -1
  * @data:     client data
  * @error:    return location for a possible error
@@ -624,7 +572,7 @@ gimp_config_deserialize_stream (GimpConfig    *config,
  * Since: 2.4
  **/
 gboolean
-gimp_config_deserialize_string (GimpConfig      *config,
+gimp_config_deserialize_string (GimpConfig   *config,
                                 const gchar  *text,
                                 gint          text_len,
                                 gpointer      data,
@@ -641,12 +589,12 @@ gimp_config_deserialize_string (GimpConfig      *config,
 
   g_object_freeze_notify (G_OBJECT (config));
 
-  success = GIMP_CONFIG_GET_INTERFACE (config)->deserialize (config,
-                                                             scanner, 0, data);
+  success = GIMP_CONFIG_GET_IFACE (config)->deserialize (config,
+                                                         scanner, 0, data);
 
   g_object_thaw_notify (G_OBJECT (config));
 
-  gimp_scanner_destroy (scanner);
+  gimp_scanner_unref (scanner);
 
   if (! success)
     g_assert (error == NULL || *error != NULL);
@@ -654,6 +602,41 @@ gimp_config_deserialize_string (GimpConfig      *config,
   return success;
 }
 
+/**
+ * gimp_config_deserialize_parasite:
+ * @config:   a #GObject that implements the #GimpConfigInterface.
+ * @parasite: parasite containing a serialized config string
+ * @data:     client data
+ * @error:    return location for a possible error
+ *
+ * Configures @config from @parasite. Basically this function creates
+ * a properly configured #GScanner for you and calls the deserialize
+ * function of the @config's #GimpConfigInterface.
+ *
+ * Returns: %TRUE if deserialization succeeded, %FALSE otherwise.
+ *
+ * Since: 3.0
+ **/
+gboolean
+gimp_config_deserialize_parasite (GimpConfig          *config,
+                                  const GimpParasite  *parasite,
+                                  gpointer             data,
+                                  GError             **error)
+{
+  g_return_val_if_fail (GIMP_IS_CONFIG (config), FALSE);
+  g_return_val_if_fail (parasite != NULL, FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  if (! gimp_parasite_data (parasite))
+    return TRUE;
+
+  return gimp_config_deserialize_string (config,
+                                         gimp_parasite_data (parasite),
+                                         gimp_parasite_data_size (parasite),
+                                         data,
+                                         error);
+}
+
 /**
  * gimp_config_deserialize_return:
  * @scanner:        a #GScanner
@@ -720,9 +703,7 @@ gimp_config_serialize (GimpConfig       *config,
 {
   g_return_val_if_fail (GIMP_IS_CONFIG (config), FALSE);
 
-  return GIMP_CONFIG_GET_INTERFACE (config)->serialize (config,
-                                                        writer,
-                                                        data);
+  return GIMP_CONFIG_GET_IFACE (config)->serialize (config, writer, data);
 }
 
 /**
@@ -746,10 +727,10 @@ gimp_config_deserialize (GimpConfig *config,
 {
   g_return_val_if_fail (GIMP_IS_CONFIG (config), FALSE);
 
-  return GIMP_CONFIG_GET_INTERFACE (config)->deserialize (config,
-                                                          scanner,
-                                                          nest_level,
-                                                          data);
+  return GIMP_CONFIG_GET_IFACE (config)->deserialize (config,
+                                                      scanner,
+                                                      nest_level,
+                                                      data);
 }
 
 /**
@@ -761,7 +742,7 @@ gimp_config_deserialize (GimpConfig *config,
  * only works for objects that are completely defined by their
  * properties.
  *
- * Return value: the duplicated #GimpConfig object
+ * Returns: the duplicated #GimpConfig object
  *
  * Since: 2.4
  **/
@@ -770,7 +751,7 @@ gimp_config_duplicate (GimpConfig *config)
 {
   g_return_val_if_fail (GIMP_IS_CONFIG (config), NULL);
 
-  return GIMP_CONFIG_GET_INTERFACE (config)->duplicate (config);
+  return GIMP_CONFIG_GET_IFACE (config)->duplicate (config);
 }
 
 /**
@@ -783,7 +764,7 @@ gimp_config_duplicate (GimpConfig *config)
  * works for objects that are completely defined by their
  * properties.
  *
- * Return value: %TRUE if the two objects are equal.
+ * Returns: %TRUE if the two objects are equal.
  *
  * Since: 2.4
  **/
@@ -796,7 +777,7 @@ gimp_config_is_equal_to (GimpConfig *a,
   g_return_val_if_fail (G_TYPE_FROM_INSTANCE (a) == G_TYPE_FROM_INSTANCE (b),
                         FALSE);
 
-  return GIMP_CONFIG_GET_INTERFACE (a)->equal (a, b);
+  return GIMP_CONFIG_GET_IFACE (a)->equal (a, b);
 }
 
 /**
@@ -816,7 +797,7 @@ gimp_config_reset (GimpConfig *config)
 
   g_object_freeze_notify (G_OBJECT (config));
 
-  GIMP_CONFIG_GET_INTERFACE (config)->reset (config);
+  GIMP_CONFIG_GET_IFACE (config)->reset (config);
 
   g_object_thaw_notify (G_OBJECT (config));
 }
@@ -833,7 +814,7 @@ gimp_config_reset (GimpConfig *config)
  *
  * Properties marked as "construct-only" are not touched.
  *
- * Return value: %TRUE if @dest was modified, %FALSE otherwise
+ * Returns: %TRUE if @dest was modified, %FALSE otherwise
  *
  * Since: 2.6
  **/
@@ -851,7 +832,7 @@ gimp_config_copy (GimpConfig  *src,
 
   g_object_freeze_notify (G_OBJECT (dest));
 
-  changed = GIMP_CONFIG_GET_INTERFACE (src)->copy (src, dest, flags);
+  changed = GIMP_CONFIG_GET_IFACE (src)->copy (src, dest, flags);
 
   g_object_thaw_notify (G_OBJECT (dest));
 
