@@ -50,6 +50,7 @@
 
 typedef struct
 {
+  gint          ref_count;
   gchar        *name;
   GMappedFile  *mapped;
   gchar        *text;
@@ -57,60 +58,39 @@ typedef struct
 } GimpScannerData;
 
 
+G_DEFINE_BOXED_TYPE (GimpScanner, gimp_scanner,
+                     gimp_scanner_ref, gimp_scanner_unref)
+
+
+
 /*  local function prototypes  */
 
-static GScanner * gimp_scanner_new     (const gchar  *name,
-                                        GMappedFile  *mapped,
-                                        gchar        *text,
-                                        GError      **error);
-static void       gimp_scanner_message (GScanner     *scanner,
-                                        gchar        *message,
-                                        gboolean      is_error);
+static GimpScanner * gimp_scanner_new     (const gchar  *name,
+                                           GMappedFile  *mapped,
+                                           gchar        *text,
+                                           GError      **error);
+static void          gimp_scanner_message (GimpScanner  *scanner,
+                                           gchar        *message,
+                                           gboolean      is_error);
 
 
 /*  public functions  */
 
 /**
  * gimp_scanner_new_file:
- * @filename:
- * @error:
- *
- * Return value:
- *
- * Since: 2.4
- **/
-GScanner *
-gimp_scanner_new_file (const gchar  *filename,
-                       GError      **error)
-{
-  GScanner *scanner;
-  GFile    *file;
-
-  g_return_val_if_fail (filename != NULL, NULL);
-  g_return_val_if_fail (error == NULL || *error == NULL, NULL);
-
-  file = g_file_new_for_path (filename);
-  scanner = gimp_scanner_new_gfile (file, error);
-  g_object_unref (file);
-
-  return scanner;
-}
-
-/**
- * gimp_scanner_new_gfile:
  * @file: a #GFile
  * @error: return location for #GError, or %NULL
  *
- * Return value: The new #GScanner.
+ * Returns: (transfer full): The new #GimpScanner.
  *
  * Since: 2.10
  **/
-GScanner *
-gimp_scanner_new_gfile (GFile   *file,
-                        GError **error)
+GimpScanner *
+gimp_scanner_new_file (GFile   *file,
+                       GError **error)
 {
-  GScanner *scanner;
-  gchar    *path;
+  GimpScanner *scanner;
+  gchar       *path;
 
   g_return_val_if_fail (G_IS_FILE (file), NULL);
   g_return_val_if_fail (error == NULL || *error == NULL, NULL);
@@ -179,15 +159,15 @@ gimp_scanner_new_gfile (GFile   *file,
  * @input: a #GInputStream
  * @error: return location for #GError, or %NULL
  *
- * Return value: The new #GScanner.
+ * Returns: (transfer full): The new #GimpScanner.
  *
  * Since: 2.10
  **/
-GScanner *
+GimpScanner *
 gimp_scanner_new_stream (GInputStream  *input,
                          GError       **error)
 {
-  GScanner    *scanner;
+  GimpScanner *scanner;
   GFile       *file;
   const gchar *path;
   GString     *string;
@@ -248,20 +228,20 @@ gimp_scanner_new_stream (GInputStream  *input,
 
 /**
  * gimp_scanner_new_string:
- * @text:
- * @text_len:
- * @error:
+ * @text: (array length=text_len):
+ * @text_len: The length of @text, or -1 if NULL-terminated
+ * @error: return location for #GError, or %NULL
  *
- * Return value:
+ * Returns: (transfer full): The new #GimpScanner.
  *
  * Since: 2.4
  **/
-GScanner *
+GimpScanner *
 gimp_scanner_new_string (const gchar  *text,
                          gint          text_len,
                          GError      **error)
 {
-  GScanner *scanner;
+  GimpScanner *scanner;
 
   g_return_val_if_fail (text != NULL || text_len <= 0, NULL);
   g_return_val_if_fail (error == NULL || *error == NULL, NULL);
@@ -276,23 +256,24 @@ gimp_scanner_new_string (const gchar  *text,
   return scanner;
 }
 
-static GScanner *
+static GimpScanner *
 gimp_scanner_new (const gchar  *name,
                   GMappedFile  *mapped,
                   gchar        *text,
                   GError      **error)
 {
-  GScanner        *scanner;
+  GimpScanner     *scanner;
   GimpScannerData *data;
 
   scanner = g_scanner_new (NULL);
 
   data = g_slice_new0 (GimpScannerData);
 
-  data->name   = g_strdup (name);
-  data->mapped = mapped;
-  data->text   = text;
-  data->error  = error;
+  data->ref_count = 1;
+  data->name      = g_strdup (name);
+  data->mapped    = mapped;
+  data->text      = text;
+  data->error     = error;
 
   scanner->user_data   = data;
   scanner->msg_handler = gimp_scanner_message;
@@ -308,14 +289,41 @@ gimp_scanner_new (const gchar  *name,
 }
 
 /**
- * gimp_scanner_destroy:
- * @scanner: A #GScanner created by gimp_scanner_new_file() or
+ * gimp_scanner_ref:
+ * @scanner: #GimpScanner to ref
+ *
+ * Adds a reference to a #GimpScanner.
+ *
+ * Returns: the same @scanner.
+ *
+ * Since: 3.0
+ */
+GimpScanner *
+gimp_scanner_ref (GimpScanner *scanner)
+{
+  GimpScannerData *data;
+
+  g_return_val_if_fail (scanner != NULL, NULL);
+
+  data = scanner->user_data;
+
+  data->ref_count++;
+
+  return scanner;
+}
+
+/**
+ * gimp_scanner_unref:
+ * @scanner: A #GimpScanner created by gimp_scanner_new_file() or
  *           gimp_scanner_new_string()
  *
- * Since: 2.4
+ * Unref a #GimpScanner. If the reference count drops to zero, the
+ * scanner is freed.
+ *
+ * Since: 3.0
  **/
 void
-gimp_scanner_destroy (GScanner *scanner)
+gimp_scanner_unref (GimpScanner *scanner)
 {
   GimpScannerData *data;
 
@@ -323,31 +331,36 @@ gimp_scanner_destroy (GScanner *scanner)
 
   data = scanner->user_data;
 
-  if (data->mapped)
-    g_mapped_file_unref (data->mapped);
+  data->ref_count--;
 
-  if (data->text)
-    g_free (data->text);
+  if (data->ref_count < 1)
+    {
+      if (data->mapped)
+        g_mapped_file_unref (data->mapped);
 
-  g_free (data->name);
-  g_slice_free (GimpScannerData, data);
+      if (data->text)
+        g_free (data->text);
 
-  g_scanner_destroy (scanner);
+      g_free (data->name);
+      g_slice_free (GimpScannerData, data);
+
+      g_scanner_destroy (scanner);
+    }
 }
 
 /**
  * gimp_scanner_parse_token:
- * @scanner: A #GScanner created by gimp_scanner_new_file() or
+ * @scanner: A #GimpScanner created by gimp_scanner_new_file() or
  *           gimp_scanner_new_string()
  * @token: the #GTokenType expected as next token.
  *
- * Return value: %TRUE if the next token is @token, %FALSE otherwise.
+ * Returns: %TRUE if the next token is @token, %FALSE otherwise.
  *
  * Since: 2.4
  **/
 gboolean
-gimp_scanner_parse_token (GScanner   *scanner,
-                          GTokenType  token)
+gimp_scanner_parse_token (GimpScanner *scanner,
+                          GTokenType   token)
 {
   if (g_scanner_peek_next_token (scanner) != token)
     return FALSE;
@@ -359,17 +372,17 @@ gimp_scanner_parse_token (GScanner   *scanner,
 
 /**
  * gimp_scanner_parse_identifier:
- * @scanner: A #GScanner created by gimp_scanner_new_file() or
+ * @scanner: A #GimpScanner created by gimp_scanner_new_file() or
  *           gimp_scanner_new_string()
- * @identifier: the expected identifier.
+ * @identifier: (out): the expected identifier.
  *
- * Return value: %TRUE if the next token is an identifier and if its
+ * Returns: %TRUE if the next token is an identifier and if its
  * value matches @identifier.
  *
  * Since: 2.4
  **/
 gboolean
-gimp_scanner_parse_identifier (GScanner    *scanner,
+gimp_scanner_parse_identifier (GimpScanner *scanner,
                                const gchar *identifier)
 {
   if (g_scanner_peek_next_token (scanner) != G_TOKEN_IDENTIFIER)
@@ -385,17 +398,17 @@ gimp_scanner_parse_identifier (GScanner    *scanner,
 
 /**
  * gimp_scanner_parse_string:
- * @scanner: A #GScanner created by gimp_scanner_new_file() or
+ * @scanner: A #GimpScanner created by gimp_scanner_new_file() or
  *           gimp_scanner_new_string()
- * @dest: Return location for the parsed string
+ * @dest: (out): Return location for the parsed string
  *
- * Return value: %TRUE on success
+ * Returns: %TRUE on success
  *
  * Since: 2.4
  **/
 gboolean
-gimp_scanner_parse_string (GScanner  *scanner,
-                           gchar    **dest)
+gimp_scanner_parse_string (GimpScanner  *scanner,
+                           gchar       **dest)
 {
   if (g_scanner_peek_next_token (scanner) != G_TOKEN_STRING)
     return FALSE;
@@ -422,17 +435,17 @@ gimp_scanner_parse_string (GScanner  *scanner,
 
 /**
  * gimp_scanner_parse_string_no_validate:
- * @scanner: A #GScanner created by gimp_scanner_new_file() or
+ * @scanner: A #GimpScanner created by gimp_scanner_new_file() or
  *           gimp_scanner_new_string()
- * @dest: Return location for the parsed string
+ * @dest: (out): Return location for the parsed string
  *
- * Return value: %TRUE on success
+ * Returns: %TRUE on success
  *
  * Since: 2.4
  **/
 gboolean
-gimp_scanner_parse_string_no_validate (GScanner  *scanner,
-                                       gchar    **dest)
+gimp_scanner_parse_string_no_validate (GimpScanner  *scanner,
+                                       gchar       **dest)
 {
   if (g_scanner_peek_next_token (scanner) != G_TOKEN_STRING)
     return FALSE;
@@ -449,19 +462,19 @@ gimp_scanner_parse_string_no_validate (GScanner  *scanner,
 
 /**
  * gimp_scanner_parse_data:
- * @scanner: A #GScanner created by gimp_scanner_new_file() or
+ * @scanner: A #GimpScanner created by gimp_scanner_new_file() or
  *           gimp_scanner_new_string()
  * @length: Length of the data to parse
- * @dest: Return location for the parsed data
+ * @dest: (out) (array): Return location for the parsed data
  *
- * Return value: %TRUE on success
+ * Returns: %TRUE on success
  *
  * Since: 2.4
  **/
 gboolean
-gimp_scanner_parse_data (GScanner  *scanner,
-                         gint       length,
-                         guint8   **dest)
+gimp_scanner_parse_data (GimpScanner  *scanner,
+                         gint          length,
+                         guint8      **dest)
 {
   if (g_scanner_peek_next_token (scanner) != G_TOKEN_STRING)
     return FALSE;
@@ -478,17 +491,17 @@ gimp_scanner_parse_data (GScanner  *scanner,
 
 /**
  * gimp_scanner_parse_int:
- * @scanner: A #GScanner created by gimp_scanner_new_file() or
+ * @scanner: A #GimpScanner created by gimp_scanner_new_file() or
  *           gimp_scanner_new_string()
- * @dest: Return location for the parsed integer
+ * @dest: (out): Return location for the parsed integer
  *
- * Return value: %TRUE on success
+ * Returns: %TRUE on success
  *
  * Since: 2.4
  **/
 gboolean
-gimp_scanner_parse_int (GScanner *scanner,
-                        gint     *dest)
+gimp_scanner_parse_int (GimpScanner *scanner,
+                        gint        *dest)
 {
   gboolean negate = FALSE;
 
@@ -513,17 +526,17 @@ gimp_scanner_parse_int (GScanner *scanner,
 
 /**
  * gimp_scanner_parse_int64:
- * @scanner: A #GScanner created by gimp_scanner_new_file() or
+ * @scanner: A #GimpScanner created by gimp_scanner_new_file() or
  *           gimp_scanner_new_string()
- * @dest: Return location for the parsed integer
+ * @dest: (out): Return location for the parsed integer
  *
- * Return value: %TRUE on success
+ * Returns: %TRUE on success
  *
  * Since: 2.8
  **/
 gboolean
-gimp_scanner_parse_int64 (GScanner *scanner,
-                          gint64   *dest)
+gimp_scanner_parse_int64 (GimpScanner *scanner,
+                          gint64      *dest)
 {
   gboolean negate = FALSE;
 
@@ -548,17 +561,17 @@ gimp_scanner_parse_int64 (GScanner *scanner,
 
 /**
  * gimp_scanner_parse_float:
- * @scanner: A #GScanner created by gimp_scanner_new_file() or
+ * @scanner: A #GimpScanner created by gimp_scanner_new_file() or
  *           gimp_scanner_new_string()
- * @dest: Return location for the parsed float
+ * @dest: (out): Return location for the parsed float
  *
- * Return value: %TRUE on success
+ * Returns: %TRUE on success
  *
  * Since: 2.4
  **/
 gboolean
-gimp_scanner_parse_float (GScanner *scanner,
-                          gdouble  *dest)
+gimp_scanner_parse_float (GimpScanner *scanner,
+                          gdouble     *dest)
 {
   gboolean negate = FALSE;
 
@@ -581,10 +594,15 @@ gimp_scanner_parse_float (GScanner *scanner,
     }
   else if (g_scanner_peek_next_token (scanner) == G_TOKEN_INT)
     {
+      /* v_int is unsigned so we need to cast to
+       *
+       * gint64 first for negative values.
+       */
+
       g_scanner_get_next_token (scanner);
 
       if (negate)
-        *dest = -scanner->value.v_int;
+        *dest = - (gint64) scanner->value.v_int;
       else
         *dest = scanner->value.v_int;
 
@@ -596,17 +614,17 @@ gimp_scanner_parse_float (GScanner *scanner,
 
 /**
  * gimp_scanner_parse_boolean:
- * @scanner: A #GScanner created by gimp_scanner_new_file() or
+ * @scanner: A #GimpScanner created by gimp_scanner_new_file() or
  *           gimp_scanner_new_string()
- * @dest: Return location for the parsed boolean
+ * @dest: (out): Return location for the parsed boolean
  *
- * Return value: %TRUE on success
+ * Returns: %TRUE on success
  *
  * Since: 2.4
  **/
 gboolean
-gimp_scanner_parse_boolean (GScanner *scanner,
-                            gboolean *dest)
+gimp_scanner_parse_boolean (GimpScanner *scanner,
+                            gboolean    *dest)
 {
   if (g_scanner_peek_next_token (scanner) != G_TOKEN_IDENTIFIER)
     return FALSE;
@@ -647,17 +665,17 @@ enum
 
 /**
  * gimp_scanner_parse_color:
- * @scanner: A #GScanner created by gimp_scanner_new_file() or
+ * @scanner: A #GimpScanner created by gimp_scanner_new_file() or
  *           gimp_scanner_new_string()
- * @dest: Pointer to a color to store the result
+ * @dest: (out caller-allocates): Pointer to a color to store the result
  *
- * Return value: %TRUE on success
+ * Returns: %TRUE on success
  *
  * Since: 2.4
  **/
 gboolean
-gimp_scanner_parse_color (GScanner *scanner,
-                          GimpRGB  *dest)
+gimp_scanner_parse_color (GimpScanner *scanner,
+                          GimpRGB     *dest)
 {
   guint      scope_id;
   guint      old_scope_id;
@@ -767,16 +785,16 @@ gimp_scanner_parse_color (GScanner *scanner,
 
 /**
  * gimp_scanner_parse_matrix2:
- * @scanner: A #GScanner created by gimp_scanner_new_file() or
+ * @scanner: A #GimpScanner created by gimp_scanner_new_file() or
  *           gimp_scanner_new_string()
  * @dest: Pointer to a matrix to store the result
  *
- * Return value: %TRUE on success
+ * Returns: %TRUE on success
  *
  * Since: 2.4
  **/
 gboolean
-gimp_scanner_parse_matrix2 (GScanner    *scanner,
+gimp_scanner_parse_matrix2 (GimpScanner *scanner,
                             GimpMatrix2 *dest)
 {
   guint        scope_id;
@@ -851,9 +869,9 @@ gimp_scanner_parse_matrix2 (GScanner    *scanner,
 /*  private functions  */
 
 static void
-gimp_scanner_message (GScanner *scanner,
-                      gchar    *message,
-                      gboolean  is_error)
+gimp_scanner_message (GimpScanner *scanner,
+                      gchar       *message,
+                      gboolean     is_error)
 {
   GimpScannerData *data = scanner->user_data;
 
