@@ -25,6 +25,8 @@
 
 #include <gtk/gtk.h>
 
+#include "libgimpbase/gimpbase.h"
+
 #include "gimpwidgetstypes.h"
 
 #include "gimpintcombobox.h"
@@ -46,32 +48,28 @@ enum
   PROP_0,
   PROP_ELLIPSIZE,
   PROP_LABEL,
-  PROP_LAYOUT
+  PROP_LAYOUT,
+  PROP_VALUE
 };
 
 
-typedef struct
+struct _GimpIntComboBoxPrivate
 {
-  GtkCellRenderer        *pixbuf_renderer;
   GtkCellRenderer        *text_renderer;
 
-  GtkCellRenderer        *menu_pixbuf_renderer;
-  GtkCellRenderer        *menu_text_renderer;
-
   PangoEllipsizeMode      ellipsize;
   gchar                  *label;
-  GtkCellRenderer        *label_renderer;
   GimpIntComboBoxLayout   layout;
 
   GimpIntSensitivityFunc  sensitivity_func;
   gpointer                sensitivity_data;
   GDestroyNotify          sensitivity_destroy;
-} GimpIntComboBoxPrivate;
+};
 
-#define GIMP_INT_COMBO_BOX_GET_PRIVATE(obj) \
-  ((GimpIntComboBoxPrivate *) ((GimpIntComboBox *) (obj))->priv)
+#define GET_PRIVATE(obj) (((GimpIntComboBox *) (obj))->priv)
 
 
+static void  gimp_int_combo_box_constructed  (GObject         *object);
 static void  gimp_int_combo_box_finalize     (GObject         *object);
 static void  gimp_int_combo_box_set_property (GObject         *object,
                                               guint            property_id,
@@ -82,6 +80,9 @@ static void  gimp_int_combo_box_get_property (GObject         *object,
                                               GValue          *value,
                                               GParamSpec      *pspec);
 
+static void  gimp_int_combo_box_changed      (GtkComboBox     *combo_box,
+                                              gpointer         user_data);
+
 static void  gimp_int_combo_box_create_cells (GimpIntComboBox *combo_box);
 static void  gimp_int_combo_box_data_func    (GtkCellLayout   *layout,
                                               GtkCellRenderer *cell,
@@ -101,6 +102,7 @@ gimp_int_combo_box_class_init (GimpIntComboBoxClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  object_class->constructed  = gimp_int_combo_box_constructed;
   object_class->finalize     = gimp_int_combo_box_finalize;
   object_class->set_property = gimp_int_combo_box_set_property;
   object_class->get_property = gimp_int_combo_box_get_property;
@@ -150,6 +152,21 @@ gimp_int_combo_box_class_init (GimpIntComboBoxClass *klass)
                                                       GIMP_TYPE_INT_COMBO_BOX_LAYOUT,
                                                       GIMP_INT_COMBO_BOX_LAYOUT_ABBREVIATED,
                                                       GIMP_PARAM_READWRITE));
+
+  /**
+   * GimpIntComboBox:value:
+   *
+   * The active value (different from the "active" property of
+   * GtkComboBox which is the active index).
+   *
+   * Since: 3.0
+   */
+  g_object_class_install_property (object_class, PROP_VALUE,
+                                   g_param_spec_int ("value",
+                                                     "Value",
+                                                     "Value of active item",
+                                                     G_MININT, G_MAXINT, 0,
+                                                     GIMP_PARAM_READWRITE));
 }
 
 static void
@@ -160,19 +177,29 @@ gimp_int_combo_box_init (GimpIntComboBox *combo_box)
 
   combo_box->priv = priv = gimp_int_combo_box_get_instance_private (combo_box);
 
-  store = gimp_int_store_new ();
+  store = g_object_new (GIMP_TYPE_INT_STORE, NULL);
   gtk_combo_box_set_model (GTK_COMBO_BOX (combo_box), GTK_TREE_MODEL (store));
   g_object_unref (store);
 
   priv->layout = GIMP_INT_COMBO_BOX_LAYOUT_ABBREVIATED;
 
-  gimp_int_combo_box_create_cells (GIMP_INT_COMBO_BOX (combo_box));
+  g_signal_connect (combo_box, "changed",
+                    G_CALLBACK (gimp_int_combo_box_changed),
+                    NULL);
+}
+
+static void
+gimp_int_combo_box_constructed (GObject *object)
+{
+  G_OBJECT_CLASS (parent_class)->constructed (object);
+
+  gimp_int_combo_box_create_cells (GIMP_INT_COMBO_BOX (object));
 }
 
 static void
 gimp_int_combo_box_finalize (GObject *object)
 {
-  GimpIntComboBoxPrivate *priv = GIMP_INT_COMBO_BOX_GET_PRIVATE (object);
+  GimpIntComboBoxPrivate *priv = GET_PRIVATE (object);
 
   g_clear_pointer (&priv->label, g_free);
 
@@ -193,7 +220,7 @@ gimp_int_combo_box_set_property (GObject      *object,
                                  const GValue *value,
                                  GParamSpec   *pspec)
 {
-  GimpIntComboBoxPrivate *priv = GIMP_INT_COMBO_BOX_GET_PRIVATE (object);
+  GimpIntComboBoxPrivate *priv = GET_PRIVATE (object);
 
   switch (property_id)
     {
@@ -213,6 +240,10 @@ gimp_int_combo_box_set_property (GObject      *object,
       gimp_int_combo_box_set_layout (GIMP_INT_COMBO_BOX (object),
                                      g_value_get_enum (value));
       break;
+    case PROP_VALUE:
+      gimp_int_combo_box_set_active (GIMP_INT_COMBO_BOX (object),
+                                     g_value_get_int (value));
+      break;
 
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -226,7 +257,7 @@ gimp_int_combo_box_get_property (GObject    *object,
                                  GValue     *value,
                                  GParamSpec *pspec)
 {
-  GimpIntComboBoxPrivate *priv = GIMP_INT_COMBO_BOX_GET_PRIVATE (object);
+  GimpIntComboBoxPrivate *priv = GET_PRIVATE (object);
 
   switch (property_id)
     {
@@ -239,6 +270,14 @@ gimp_int_combo_box_get_property (GObject    *object,
     case PROP_LAYOUT:
       g_value_set_enum (value, priv->layout);
       break;
+    case PROP_VALUE:
+      {
+        gint v;
+
+        gimp_int_combo_box_get_active (GIMP_INT_COMBO_BOX (object), &v);
+        g_value_set_int (value, v);
+      }
+    break;
 
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -248,7 +287,7 @@ gimp_int_combo_box_get_property (GObject    *object,
 
 
 /**
- * gimp_int_combo_box_new:
+ * gimp_int_combo_box_new: (skip)
  * @first_label: the label of the first item
  * @first_value: the value of the first item
  * @...: a %NULL terminated list of more label, value pairs
@@ -260,7 +299,7 @@ gimp_int_combo_box_get_property (GObject    *object,
  * If you need to construct an empty #GimpIntComboBox, it's best to use
  * g_object_new (GIMP_TYPE_INT_COMBO_BOX, NULL).
  *
- * Return value: a new #GimpIntComboBox.
+ * Returns: a new #GimpIntComboBox.
  *
  * Since: 2.2
  **/
@@ -288,9 +327,9 @@ gimp_int_combo_box_new (const gchar *first_label,
  * @values: a va_list with more values
  *
  * A variant of gimp_int_combo_box_new() that takes a va_list of
- * label/value pairs. Probably only useful for language bindings.
+ * label/value pairs.
  *
- * Return value: a new #GimpIntComboBox.
+ * Returns: a new #GimpIntComboBox.
  *
  * Since: 2.2
  **/
@@ -301,25 +340,14 @@ gimp_int_combo_box_new_valist (const gchar *first_label,
 {
   GtkWidget    *combo_box;
   GtkListStore *store;
-  const gchar  *label;
-  gint          value;
 
-  combo_box = g_object_new (GIMP_TYPE_INT_COMBO_BOX, NULL);
+  store = gimp_int_store_new_valist (first_label, first_value, values);
 
-  store = GTK_LIST_STORE (gtk_combo_box_get_model (GTK_COMBO_BOX (combo_box)));
+  combo_box = g_object_new (GIMP_TYPE_INT_COMBO_BOX,
+                            "model", store,
+                            NULL);
 
-  for (label = first_label, value = first_value;
-       label;
-       label = va_arg (values, const gchar *), value = va_arg (values, gint))
-    {
-      GtkTreeIter  iter = { 0, };
-
-      gtk_list_store_append (store, &iter);
-      gtk_list_store_set (store, &iter,
-                          GIMP_INT_STORE_VALUE, value,
-                          GIMP_INT_STORE_LABEL, label,
-                          -1);
-    }
+  g_object_unref (store);
 
   return combo_box;
 }
@@ -332,7 +360,7 @@ gimp_int_combo_box_new_valist (const gchar *first_label,
  * A variant of gimp_int_combo_box_new() that takes an array of labels.
  * The array indices are used as values.
  *
- * Return value: a new #GimpIntComboBox.
+ * Returns: a new #GimpIntComboBox.
  *
  * Since: 2.2
  **/
@@ -369,7 +397,7 @@ gimp_int_combo_box_new_array (gint         n_values,
 }
 
 /**
- * gimp_int_combo_box_prepend:
+ * gimp_int_combo_box_prepend: (skip)
  * @combo_box: a #GimpIntComboBox
  * @...:       pairs of column number and value, terminated with -1
  *
@@ -402,7 +430,7 @@ gimp_int_combo_box_prepend (GimpIntComboBox *combo_box,
 }
 
 /**
- * gimp_int_combo_box_append:
+ * gimp_int_combo_box_append: (skip)
  * @combo_box: a #GimpIntComboBox
  * @...:       pairs of column number and value, terminated with -1
  *
@@ -442,8 +470,8 @@ gimp_int_combo_box_append (GimpIntComboBox *combo_box,
  * Looks up the item that belongs to the given @value and makes it the
  * selected item in the @combo_box.
  *
- * Return value: %TRUE on success or %FALSE if there was no item for
- *               this value.
+ * Returns: %TRUE on success (value changed or not) or %FALSE if there
+ *          was no item for this value.
  *
  * Since: 2.2
  **/
@@ -453,12 +481,21 @@ gimp_int_combo_box_set_active (GimpIntComboBox *combo_box,
 {
   GtkTreeModel *model;
   GtkTreeIter   iter;
+  gint          current_value;
 
   g_return_val_if_fail (GIMP_IS_INT_COMBO_BOX (combo_box), FALSE);
 
   model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo_box));
 
-  if (gimp_int_store_lookup_by_value (model, value, &iter))
+  if (gimp_int_combo_box_get_active (combo_box, &current_value) &&
+      value == current_value)
+    {
+      /* Guard for identical value to not loop forever between
+       * GimpIntComboBox "value" and GtkComboBox "active" properties.
+       */
+      return TRUE;
+    }
+  else if (gimp_int_store_lookup_by_value (model, value, &iter))
     {
       gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combo_box), &iter);
       return TRUE;
@@ -470,11 +507,11 @@ gimp_int_combo_box_set_active (GimpIntComboBox *combo_box,
 /**
  * gimp_int_combo_box_get_active:
  * @combo_box: a #GimpIntComboBox
- * @value:     return location for the integer value
+ * @value: (out): return location for the integer value
  *
  * Retrieves the value of the selected (active) item in the @combo_box.
  *
- * Return value: %TRUE if @value has been set or %FALSE if no item was
+ * Returns: %TRUE if @value has been set or %FALSE if no item was
  *               active.
  *
  * Since: 2.2
@@ -508,7 +545,7 @@ gimp_int_combo_box_get_active (GimpIntComboBox *combo_box,
  * Looks up the item that has the given @user_data and makes it the
  * selected item in the @combo_box.
  *
- * Return value: %TRUE on success or %FALSE if there was no item for
+ * Returns: %TRUE on success or %FALSE if there was no item for
  *               this user-data.
  *
  * Since: 2.10
@@ -536,11 +573,11 @@ gimp_int_combo_box_set_active_by_user_data (GimpIntComboBox *combo_box,
 /**
  * gimp_int_combo_box_get_active_user_data:
  * @combo_box: a #GimpIntComboBox
- * @user_data: return location for the gpointer value
+ * @user_data: (out): return location for the gpointer value
  *
  * Retrieves the user-data of the selected (active) item in the @combo_box.
  *
- * Return value: %TRUE if @user_data has been set or %FALSE if no item was
+ * Returns: %TRUE if @user_data has been set or %FALSE if no item was
  *               active.
  *
  * Since: 2.10
@@ -568,10 +605,11 @@ gimp_int_combo_box_get_active_user_data (GimpIntComboBox *combo_box,
 
 /**
  * gimp_int_combo_box_connect:
- * @combo_box: a #GimpIntComboBox
- * @value:     the value to set
- * @callback:  a callback to connect to the @combo_box's "changed" signal
- * @data:      a pointer passed as data to g_signal_connect()
+ * @combo_box:    a #GimpIntComboBox
+ * @value:        the value to set
+ * @callback:     a callback to connect to the @combo_box's "changed" signal
+ * @data:         a pointer passed as data to g_signal_connect()
+ * @data_destroy: Destroy function for @data.
  *
  * A convenience function that sets the initial @value of a
  * #GimpIntComboBox and connects @callback to the "changed"
@@ -583,7 +621,7 @@ gimp_int_combo_box_get_active_user_data (GimpIntComboBox *combo_box,
  * gimp_drawable_combo_box_new(). If you pass an invalid initial
  * @value, the @callback will be called with the default item active.
  *
- * Return value: the signal handler ID as returned by g_signal_connect()
+ * Returns: the signal handler ID as returned by g_signal_connect()
  *
  * Since: 2.2
  **/
@@ -591,7 +629,8 @@ gulong
 gimp_int_combo_box_connect (GimpIntComboBox *combo_box,
                             gint             value,
                             GCallback        callback,
-                            gpointer         data)
+                            gpointer         data,
+                            GDestroyNotify   data_destroy)
 {
   gulong handler = 0;
 
@@ -600,6 +639,10 @@ gimp_int_combo_box_connect (GimpIntComboBox *combo_box,
   if (callback)
     handler = g_signal_connect (combo_box, "changed", callback, data);
 
+  if (data_destroy)
+    g_object_weak_ref (G_OBJECT (combo_box), (GWeakNotify) data_destroy,
+                       data);
+
   if (! gimp_int_combo_box_set_active (combo_box, value))
     g_signal_emit_by_name (combo_box, "changed", NULL);
 
@@ -626,14 +669,29 @@ gimp_int_combo_box_set_label (GimpIntComboBox *combo_box,
 
   g_return_if_fail (GIMP_IS_INT_COMBO_BOX (combo_box));
 
-  priv = GIMP_INT_COMBO_BOX_GET_PRIVATE (combo_box);
+  priv = GET_PRIVATE (combo_box);
 
   if (label == priv->label)
     return;
 
-  g_free (priv->label);
+  if (priv->label)
+    {
+      g_free (priv->label);
+      priv->label = NULL;
 
-  priv->label = g_strdup (label);
+      g_signal_handlers_disconnect_by_func (combo_box,
+                                            gimp_int_combo_box_create_cells,
+                                            NULL);
+    }
+
+  if (label)
+    {
+      priv->label = g_strdup (label);
+
+      g_signal_connect (combo_box, "notify::popup-shown",
+                        G_CALLBACK (gimp_int_combo_box_create_cells),
+                        NULL);
+    }
 
   gimp_int_combo_box_create_cells (combo_box);
 
@@ -647,7 +705,7 @@ gimp_int_combo_box_set_label (GimpIntComboBox *combo_box,
  * Returns the label previously set with gimp_int_combo_box_set_label(),
  * or %NULL,
  *
- * Return value: the @combo_box' label.
+ * Returns: the @combo_box' label.
  *
  * Since: 2.10
  **/
@@ -656,7 +714,7 @@ gimp_int_combo_box_get_label (GimpIntComboBox *combo_box)
 {
   g_return_val_if_fail (GIMP_IS_INT_COMBO_BOX (combo_box), NULL);
 
-  return GIMP_INT_COMBO_BOX_GET_PRIVATE (combo_box)->label;
+  return GET_PRIVATE (combo_box)->label;
 }
 
 /**
@@ -676,7 +734,7 @@ gimp_int_combo_box_set_layout (GimpIntComboBox       *combo_box,
 
   g_return_if_fail (GIMP_IS_INT_COMBO_BOX (combo_box));
 
-  priv = GIMP_INT_COMBO_BOX_GET_PRIVATE (combo_box);
+  priv = GET_PRIVATE (combo_box);
 
   if (layout == priv->layout)
     return;
@@ -694,7 +752,7 @@ gimp_int_combo_box_set_layout (GimpIntComboBox       *combo_box,
  *
  * Returns the layout of @combo_box
  *
- * Return value: the @combo_box's layout.
+ * Returns: the @combo_box's layout.
  *
  * Since: 2.10
  **/
@@ -704,7 +762,7 @@ gimp_int_combo_box_get_layout (GimpIntComboBox *combo_box)
   g_return_val_if_fail (GIMP_IS_INT_COMBO_BOX (combo_box),
                         GIMP_INT_COMBO_BOX_LAYOUT_ABBREVIATED);
 
-  return GIMP_INT_COMBO_BOX_GET_PRIVATE (combo_box)->layout;
+  return GET_PRIVATE (combo_box)->layout;
 }
 
 /**
@@ -733,7 +791,7 @@ gimp_int_combo_box_set_sensitivity (GimpIntComboBox        *combo_box,
 
   g_return_if_fail (GIMP_IS_INT_COMBO_BOX (combo_box));
 
-  priv = GIMP_INT_COMBO_BOX_GET_PRIVATE (combo_box);
+  priv = GET_PRIVATE (combo_box);
 
   if (priv->sensitivity_destroy)
     {
@@ -754,181 +812,147 @@ gimp_int_combo_box_set_sensitivity (GimpIntComboBox        *combo_box,
 /*  private functions  */
 
 static void
-queue_resize_cell_view (GtkContainer *container)
+gimp_int_combo_box_changed (GtkComboBox *combo_box,
+                            gpointer     user_data)
 {
-  GList *children = gtk_container_get_children (container);
-  GList *list;
+  gint value;
 
-  for (list = children; list; list = g_list_next (list))
-    {
-      if (GTK_IS_CELL_VIEW (list->data))
-        {
-          gtk_widget_queue_resize (list->data);
-          break;
-        }
-      else if (GTK_IS_CONTAINER (list->data))
-        {
-          queue_resize_cell_view (list->data);
-        }
-    }
-
-  g_list_free (children);
+  gimp_int_combo_box_get_active (GIMP_INT_COMBO_BOX (combo_box),
+                                 &value);
+  g_object_set (combo_box, "value", value, NULL);
 }
 
 static void
 gimp_int_combo_box_create_cells (GimpIntComboBox *combo_box)
 {
-  GimpIntComboBoxPrivate *priv = GIMP_INT_COMBO_BOX_GET_PRIVATE (combo_box);
-  GtkCellLayout          *layout;
+  GimpIntComboBoxPrivate *priv   = GET_PRIVATE (combo_box);
+  GtkCellLayout          *layout = GTK_CELL_LAYOUT (combo_box);
+  GtkCellRenderer        *text_renderer;
+  GtkCellRenderer        *pixbuf_renderer;
+  gboolean                popup_shown;
 
-  /*  menu layout  */
-
-  layout = GTK_CELL_LAYOUT (combo_box);
+  g_object_get (combo_box, "popup-shown", &popup_shown, NULL);
 
   gtk_cell_layout_clear (layout);
 
-  priv->menu_pixbuf_renderer = gtk_cell_renderer_pixbuf_new ();
-  g_object_set (priv->menu_pixbuf_renderer,
-                "xpad", 2,
-                NULL);
+  priv->text_renderer = NULL;
 
-  priv->menu_text_renderer = gtk_cell_renderer_text_new ();
-
-  gtk_cell_layout_pack_start (layout,
-                              priv->menu_pixbuf_renderer, FALSE);
-  gtk_cell_layout_pack_start (layout,
-                              priv->menu_text_renderer, TRUE);
-
-  gtk_cell_layout_set_attributes (layout,
-                                  priv->menu_pixbuf_renderer,
-                                  "icon-name", GIMP_INT_STORE_ICON_NAME,
-                                  "pixbuf",    GIMP_INT_STORE_PIXBUF,
-                                  NULL);
-  gtk_cell_layout_set_attributes (layout,
-                                  priv->menu_text_renderer,
-                                  "text", GIMP_INT_STORE_LABEL,
-                                  NULL);
-
-  if (priv->sensitivity_func)
+  if (popup_shown)
     {
-      gtk_cell_layout_set_cell_data_func (layout,
-                                          priv->menu_pixbuf_renderer,
-                                          gimp_int_combo_box_data_func,
-                                          priv, NULL);
+      /*  menu layout  */
 
-      gtk_cell_layout_set_cell_data_func (layout,
-                                          priv->menu_text_renderer,
-                                          gimp_int_combo_box_data_func,
-                                          priv, NULL);
-    }
-
-  /*  combo box layout  */
-
-  layout = GTK_CELL_LAYOUT (gtk_bin_get_child (GTK_BIN (combo_box)));
-
-  gtk_cell_layout_clear (layout);
-
-  if (priv->layout != GIMP_INT_COMBO_BOX_LAYOUT_ICON_ONLY)
-    {
-      priv->text_renderer = gtk_cell_renderer_text_new ();
-      g_object_set (priv->text_renderer,
-                    "ellipsize", priv->ellipsize,
-                    NULL);
-    }
-  else
-    {
-      priv->text_renderer = NULL;
-    }
-
-  priv->pixbuf_renderer = gtk_cell_renderer_pixbuf_new ();
-
-  if (priv->text_renderer)
-    {
-      g_object_set (priv->pixbuf_renderer,
+      pixbuf_renderer = gtk_cell_renderer_pixbuf_new ();
+      g_object_set (pixbuf_renderer,
                     "xpad", 2,
                     NULL);
-    }
 
-  if (priv->label)
-    {
-      priv->label_renderer = gtk_cell_renderer_text_new ();
-      g_object_set (priv->label_renderer,
-                    "text", priv->label,
-                    NULL);
+      text_renderer = gtk_cell_renderer_text_new ();
 
-      gtk_cell_layout_pack_start (layout,
-                                  priv->label_renderer, FALSE);
+      gtk_cell_layout_pack_start (layout, pixbuf_renderer, FALSE);
+      gtk_cell_layout_pack_start (layout, text_renderer, TRUE);
 
-      gtk_cell_layout_pack_end (layout,
-                                priv->pixbuf_renderer, FALSE);
-
-      if (priv->text_renderer)
-        {
-          gtk_cell_layout_pack_end (layout,
-                                    priv->text_renderer, TRUE);
-
-          g_object_set (priv->text_renderer,
-                        "xalign", 1.0,
-                        NULL);
-        }
-    }
-  else
-    {
-      gtk_cell_layout_pack_start (layout,
-                                  priv->pixbuf_renderer, FALSE);
-
-      if (priv->text_renderer)
-        {
-          gtk_cell_layout_pack_start (layout,
-                                      priv->text_renderer, TRUE);
-        }
-    }
-
-  gtk_cell_layout_set_attributes (layout,
-                                  priv->pixbuf_renderer,
-                                  "icon-name", GIMP_INT_STORE_ICON_NAME,
-                                  NULL);
-
-  if (priv->text_renderer)
-    {
       gtk_cell_layout_set_attributes (layout,
-                                      priv->text_renderer,
+                                      pixbuf_renderer,
+                                      "icon-name", GIMP_INT_STORE_ICON_NAME,
+                                      "pixbuf",    GIMP_INT_STORE_PIXBUF,
+                                      NULL);
+      gtk_cell_layout_set_attributes (layout,
+                                      text_renderer,
                                       "text", GIMP_INT_STORE_LABEL,
                                       NULL);
-    }
 
-  if (priv->layout == GIMP_INT_COMBO_BOX_LAYOUT_ABBREVIATED ||
-      priv->sensitivity_func)
-    {
-      gtk_cell_layout_set_cell_data_func (layout,
-                                          priv->pixbuf_renderer,
-                                          gimp_int_combo_box_data_func,
-                                          priv, NULL);
-
-      if (priv->text_renderer)
+      if (priv->sensitivity_func)
         {
           gtk_cell_layout_set_cell_data_func (layout,
-                                              priv->text_renderer,
+                                              pixbuf_renderer,
+                                              gimp_int_combo_box_data_func,
+                                              priv, NULL);
+
+          gtk_cell_layout_set_cell_data_func (layout,
+                                              text_renderer,
                                               gimp_int_combo_box_data_func,
                                               priv, NULL);
         }
     }
-
-  /* HACK: GtkCellView doesn't invalidate itself when stuff is
-   * added/removed, work around this bug until GTK+ 2.24.19
-   */
-  if (gtk_check_version (2, 24, 19))
+  else
     {
-      GList *attached_menus;
+      /*  combo box layout  */
 
-      queue_resize_cell_view (GTK_CONTAINER (combo_box));
+      if (priv->layout != GIMP_INT_COMBO_BOX_LAYOUT_ICON_ONLY)
+        {
+          priv->text_renderer = text_renderer = gtk_cell_renderer_text_new ();
+          g_object_set (priv->text_renderer,
+                        "ellipsize", priv->ellipsize,
+                        NULL);
+        }
+      else
+        {
+          text_renderer = NULL;
+        }
 
-      /* HACK HACK HACK OMG */
-      attached_menus = g_object_get_data (G_OBJECT (combo_box),
-                                          "gtk-attached-menus");
+      pixbuf_renderer = gtk_cell_renderer_pixbuf_new ();
 
-      for (; attached_menus; attached_menus = g_list_next (attached_menus))
-        queue_resize_cell_view (attached_menus->data);
+      if (text_renderer)
+        {
+          g_object_set (pixbuf_renderer,
+                        "xpad", 2,
+                        NULL);
+        }
+
+      if (priv->label)
+        {
+          GtkCellRenderer *label_renderer;
+
+          label_renderer = gtk_cell_renderer_text_new ();
+          g_object_set (label_renderer,
+                        "text", priv->label,
+                        NULL);
+
+          gtk_cell_layout_pack_start (layout, label_renderer, FALSE);
+          gtk_cell_layout_pack_end (layout, pixbuf_renderer, FALSE);
+
+          if (text_renderer)
+            {
+              gtk_cell_layout_pack_end (layout, text_renderer, TRUE);
+
+              g_object_set (text_renderer,
+                            "xalign", 1.0,
+                            NULL);
+            }
+        }
+      else
+        {
+          gtk_cell_layout_pack_start (layout, pixbuf_renderer, FALSE);
+
+          if (priv->text_renderer)
+            gtk_cell_layout_pack_start (layout, text_renderer, TRUE);
+        }
+
+      gtk_cell_layout_set_attributes (layout,
+                                      pixbuf_renderer,
+                                      "icon-name", GIMP_INT_STORE_ICON_NAME,
+                                      NULL);
+
+      if (text_renderer)
+        gtk_cell_layout_set_attributes (layout,
+                                        text_renderer,
+                                        "text", GIMP_INT_STORE_LABEL,
+                                        NULL);
+
+      if (priv->layout == GIMP_INT_COMBO_BOX_LAYOUT_ABBREVIATED ||
+          priv->sensitivity_func)
+        {
+          gtk_cell_layout_set_cell_data_func (layout,
+                                              pixbuf_renderer,
+                                              gimp_int_combo_box_data_func,
+                                              priv, NULL);
+
+          if (text_renderer)
+            gtk_cell_layout_set_cell_data_func (layout,
+                                                text_renderer,
+                                                gimp_int_combo_box_data_func,
+                                                priv, NULL);
+        }
     }
 }
 
