@@ -47,6 +47,7 @@ typedef struct _GimpColorProfileComboBox      GimpColorProfileComboBox;
 typedef struct _GimpColorProfileStore         GimpColorProfileStore;
 typedef struct _GimpColorProfileView          GimpColorProfileView;
 typedef struct _GimpColorScale                GimpColorScale;
+typedef struct _GimpColorScaleEntry           GimpColorScaleEntry;
 typedef struct _GimpColorScales               GimpColorScales;
 typedef struct _GimpColorSelector             GimpColorSelector;
 typedef struct _GimpColorSelect               GimpColorSelect;
@@ -61,6 +62,8 @@ typedef struct _GimpFrame                     GimpFrame;
 typedef struct _GimpHintBox                   GimpHintBox;
 typedef struct _GimpIntComboBox               GimpIntComboBox;
 typedef struct _GimpIntStore                  GimpIntStore;
+typedef struct _GimpLabeled                   GimpLabeled;
+typedef struct _GimpLabelSpin                 GimpLabelSpin;
 typedef struct _GimpMemsizeEntry              GimpMemsizeEntry;
 typedef struct _GimpNumberPairEntry           GimpNumberPairEntry;
 typedef struct _GimpOffsetArea                GimpOffsetArea;
@@ -70,6 +73,7 @@ typedef struct _GimpPickButton                GimpPickButton;
 typedef struct _GimpPreview                   GimpPreview;
 typedef struct _GimpPreviewArea               GimpPreviewArea;
 typedef struct _GimpRuler                     GimpRuler;
+typedef struct _GimpScaleEntry                GimpScaleEntry;
 typedef struct _GimpScrolledPreview           GimpScrolledPreview;
 typedef struct _GimpSizeEntry                 GimpSizeEntry;
 typedef struct _GimpSpinButton                GimpSpinButton;
