@@ -218,110 +218,76 @@ png_create_procedure (GimpPlugIn  *plug_in,
                                       "Nick Lamb <njl195@zepler.org.uk>",
                                       PLUG_IN_VERSION);
 
+      gimp_file_procedure_set_format_name (GIMP_FILE_PROCEDURE (procedure),
+                                           _("PNG"));
       gimp_file_procedure_set_mime_types (GIMP_FILE_PROCEDURE (procedure),
                                           "image/png");
       gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                           "png");
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "interlaced",
-                             "Interlaced",
+                             "_Interlacing (Adam7)",
                              "Use Adam7 interlacing?",
                              FALSE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_INT (procedure, "compression",
-                         "Compression",
+                         "Co_mpression level",
                          "Deflate Compression factor (0..9)",
                          0, 9, 9,
                          G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "bkgd",
-                             "bKGD",
-                             "Write bKGD chunk?",
+                             "Save _background color",
+                             "Write bKGD chunk (PNG metadata)",
                              TRUE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "gama",
-                             "gAMA",
-                             "Write gAMA chunk?",
+                             "Save _gamma",
+                             "Write gAMA chunk (PNG metadata)",
                              FALSE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "offs",
-                             "oFFs",
-                             "Write oFFs chunk?",
+                             "Save layer o_ffset",
+                             "Write oFFs chunk (PNG metadata)",
                              FALSE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "phys",
-                             "pHYs",
-                             "Write pHYs chunk?",
+                             "Save resol_ution",
+                             "Write pHYs chunk (PNG metadata)",
                              TRUE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "time",
-                             "tIME",
-                             "Write tIME chunk?",
+                             "Save creation _time",
+                             "Write tIME chunk (PNG metadata)",
                              TRUE,
                              G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_BOOLEAN (procedure, "save-comment",
-                             "Save comment",
-                             "Write comment?",
-                             gimp_export_comment (),
-                             G_PARAM_READWRITE);
-
-      GIMP_PROC_AUX_ARG_STRING (procedure, "gimp-comment",
-                                "Comment",
-                                "Image comment",
-                                gimp_get_default_comment (),
-                                G_PARAM_READWRITE);
-
-      gimp_procedure_set_argument_sync (procedure, "gimp-comment",
-                                        GIMP_ARGUMENT_SYNC_PARASITE);
-
       GIMP_PROC_ARG_BOOLEAN (procedure, "save-transparent",
-                             "Save transparent",
+                             "Save color _values from transparent pixels",
                              "Preserve color of transparent pixels?",
                              FALSE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_AUX_ARG_INT (procedure, "format",
-                             "Format",
+                             "_Pixel format",
                              "PNG export format",
                              PNG_FORMAT_AUTO, PNG_FORMAT_GRAYA16,
                              PNG_FORMAT_AUTO,
                              G_PARAM_READWRITE);
 
-      GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-exif",
-                                 "Save Exif",
-                                 "Save Exif",
-                                 gimp_export_exif (),
-                                 G_PARAM_READWRITE);
-
-      GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-xmp",
-                                 "Save XMP",
-                                 "Save XMP",
-                                 gimp_export_xmp (),
-                                 G_PARAM_READWRITE);
-
-      GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-iptc",
-                                 "Save IPTC",
-                                 "Save IPTC",
-                                 gimp_export_iptc (),
-                                 G_PARAM_READWRITE);
-
-      GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-thumbnail",
-                                 "Save thumbnail",
-                                 "Save thumbnail",
-                                 TRUE,
-                                 G_PARAM_READWRITE);
-
-      GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-color-profile",
-                                 "Save profile",
-                                 "Save color profile",
-                                 gimp_export_color_profile (),
-                                 G_PARAM_READWRITE);
+      gimp_save_procedure_set_support_exif      (GIMP_SAVE_PROCEDURE (procedure), TRUE);
+      gimp_save_procedure_set_support_iptc      (GIMP_SAVE_PROCEDURE (procedure), TRUE);
+      gimp_save_procedure_set_support_xmp       (GIMP_SAVE_PROCEDURE (procedure), TRUE);
+#if defined(PNG_iCCP_SUPPORTED)
+      gimp_save_procedure_set_support_profile   (GIMP_SAVE_PROCEDURE (procedure), TRUE);
+#endif
+      gimp_save_procedure_set_support_thumbnail (GIMP_SAVE_PROCEDURE (procedure), TRUE);
+      gimp_save_procedure_set_support_comment   (GIMP_SAVE_PROCEDURE (procedure), TRUE);
     }
 
   return procedure;
@@ -905,19 +871,23 @@ load_image (GFile        *file,
       gint offset_x = png_get_x_offset_pixels (pp, info);
       gint offset_y = png_get_y_offset_pixels (pp, info);
 
-      if (! interactive)
+      if (offset_x != 0 ||
+          offset_y != 0)
         {
-          gimp_layer_set_offsets (layer, offset_x, offset_y);
-        }
-      else if (offsets_dialog (offset_x, offset_y))
-        {
-          gimp_layer_set_offsets (layer, offset_x, offset_y);
-
-          if (abs (offset_x) > width ||
-              abs (offset_y) > height)
+          if (! interactive)
             {
-              g_message (_("The PNG file specifies an offset that caused "
-                           "the layer to be positioned outside the image."));
+              gimp_layer_set_offsets (layer, offset_x, offset_y);
+            }
+          else if (offsets_dialog (offset_x, offset_y))
+            {
+              gimp_layer_set_offsets (layer, offset_x, offset_y);
+
+              if (abs (offset_x) > width ||
+                  abs (offset_y) > height)
+                {
+                  g_message (_("The PNG file specifies an offset that caused "
+                               "the layer to be positioned outside the image."));
+                }
             }
         }
     }
@@ -1273,10 +1243,6 @@ save_image (GFile        *file,
   gboolean        save_transp_pixels;
   gint            compression_level;
   PngExportFormat export_format;
-  gboolean        save_exif;
-  gboolean        save_xmp;
-  gboolean        save_iptc;
-  gboolean        save_thumbnail;
   gboolean        save_profile;
 
 #if !defined(PNG_iCCP_SUPPORTED)
@@ -1297,10 +1263,6 @@ save_image (GFile        *file,
                 "save-transparent",   &save_transp_pixels,
                 "compression",        &compression_level,
                 "format",             &export_format,
-                "save-exif",          &save_exif,
-                "save-xmp",           &save_xmp,
-                "save-iptc",          &save_iptc,
-                "save-thumbnail",     &save_thumbnail,
                 "save-color-profile", &save_profile,
                 NULL);
 
@@ -2201,109 +2163,16 @@ save_dialog (GimpImage     *image,
              gboolean       alpha)
 {
   GtkWidget    *dialog;
-  GtkWidget    *main_vbox;
-  GtkWidget    *grid;
-  GtkWidget    *button;
   GtkListStore *store;
-  GtkWidget    *combo;
-  GimpParasite *parasite;
-  gint          col;
-  gint          row;
   gboolean      run;
 
-  dialog = gimp_procedure_dialog_new (procedure,
-                                      GIMP_PROCEDURE_CONFIG (config),
-                                      _("Export Image as PNG"));
+  dialog = gimp_save_procedure_dialog_new (GIMP_SAVE_PROCEDURE (procedure),
+                                           GIMP_PROCEDURE_CONFIG (config),
+                                           image);
 
-  main_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-  gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
-  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
-                      main_vbox, TRUE, TRUE, 0);
-  gtk_widget_show (main_vbox);
+  gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dialog),
+                                    "compression", GIMP_TYPE_SCALE_ENTRY);
 
-  grid = gtk_grid_new ();
-  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
-  gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
-  gtk_box_pack_start (GTK_BOX (main_vbox), grid, FALSE, FALSE, 0);
-  gtk_widget_show (grid);
-
-  /* Toggles */
-  col = 0;
-  row = 0;
-
-  button = gimp_prop_check_button_new (config, "interlaced",
-                                       _("_Interlacing (Adam7)"));
-  gtk_grid_attach (GTK_GRID (grid), button, col, row++, 1, 1);
-
-  button = gimp_prop_check_button_new (config, "bkgd",
-                                       _("Save _background color"));
-  gtk_grid_attach (GTK_GRID (grid), button, col, row++, 1, 1);
-
-  button = gimp_prop_check_button_new (config, "gama",
-                                       _("Save _gamma"));
-  gtk_grid_attach (GTK_GRID (grid), button, col, row++, 1, 1);
-
-  button = gimp_prop_check_button_new (config, "offs",
-                                       _("Save layer o_ffset"));
-  gtk_grid_attach (GTK_GRID (grid), button, col, row++, 1, 1);
-
-  button = gimp_prop_check_button_new (config, "phys",
-                                       _("Save _resolution"));
-  gtk_grid_attach (GTK_GRID (grid), button, col, row++, 1, 1);
-
-  button = gimp_prop_check_button_new (config, "time",
-                                       _("Save creation _time"));
-  gtk_grid_attach (GTK_GRID (grid), button, col, row++, 1, 1);
-
-  button = gimp_prop_check_button_new (config, "save-transparent",
-                                       _("Save color _values from "
-                                         "transparent pixels"));
-  gtk_grid_attach (GTK_GRID (grid), button, col, row++, 2, 1);
-
-  gtk_widget_set_sensitive (button, alpha);
-
-  col = 1;
-  row = 0;
-
-  button = gimp_prop_check_button_new (config, "save-comment",
-                                       _("Save comme_nt"));
-  gtk_grid_attach (GTK_GRID (grid), button, col, row++, 1, 1);
-
-  parasite = gimp_image_get_parasite (image, "gimp-comment");
-  gtk_widget_set_sensitive (button, parasite != NULL);
-  gimp_parasite_free (parasite);
-
-  button = gimp_prop_check_button_new (config, "save-exif",
-                                       _("Save E_xif data"));
-  gtk_grid_attach (GTK_GRID (grid), button, col, row++, 1, 1);
-
-  button = gimp_prop_check_button_new (config, "save-xmp",
-                                       _("Save XMP data"));
-  gtk_grid_attach (GTK_GRID (grid), button, col, row++, 1, 1);
-
-  button = gimp_prop_check_button_new (config, "save-iptc",
-                                       _("Save IPTC data"));
-  gtk_grid_attach (GTK_GRID (grid), button, col, row++, 1, 1);
-
-  button = gimp_prop_check_button_new (config, "save-thumbnail",
-                                       _("Save thumbnail"));
-  gtk_grid_attach (GTK_GRID (grid), button, col, row++, 1, 1);
-
-  button = gimp_prop_check_button_new (config, "save-color-profile",
-                                       _("Save color profile"));
-  gtk_grid_attach (GTK_GRID (grid), button, col, row++, 1, 1);
-
-#if !defined(PNG_iCCP_SUPPORTED)
-  gtk_widget_hide (button);
-#endif
-
-  grid = gtk_grid_new ();
-  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
-  gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
-  gtk_box_pack_start (GTK_BOX (main_vbox), grid, FALSE, FALSE, 0);
-  gtk_widget_show (grid);
-
-  /* Pixel format combo */
   store = gimp_int_store_new (_("Automatic"),    PNG_FORMAT_AUTO,
                               _("8 bpc RGB"),    PNG_FORMAT_RGB8,
                               _("8 bpc GRAY"),   PNG_FORMAT_GRAY8,
@@ -2314,24 +2183,28 @@ save_dialog (GimpImage     *image,
                               _("16 bpc RGBA"),  PNG_FORMAT_RGBA16,
                               _("16 bpc GRAYA"), PNG_FORMAT_GRAYA16,
                               NULL);
-  combo = gimp_prop_int_combo_box_new (config, "format",
-                                       GIMP_INT_STORE (store));
+  gimp_procedure_dialog_get_int_combo (GIMP_PROCEDURE_DIALOG (dialog),
+                                       "format", GIMP_INT_STORE (store));
   g_object_unref (store);
 
-  gimp_grid_attach_aligned (GTK_GRID (grid), 0, 0,
-                            _("_Pixel format:"), 0.0, 0.5,
-                            combo, 2);
+  gtk_widget_set_sensitive (gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dialog),
+                                                              "save-transparent",
+                                                              G_TYPE_NONE),
+                            alpha);
 
-  /* Compression level scale */
-  gimp_prop_scale_entry_new (config, "compression",
-                             GTK_GRID (grid), 0, 1,
-                             _("Co_mpression level:"),
-                             1, 1, 0,
-                             FALSE, 0, 0);
-
-  gtk_widget_show (dialog);
+  gimp_save_procedure_dialog_add_metadata (GIMP_SAVE_PROCEDURE_DIALOG (dialog), "bkgd");
+  gimp_save_procedure_dialog_add_metadata (GIMP_SAVE_PROCEDURE_DIALOG (dialog), "gama");
+  gimp_save_procedure_dialog_add_metadata (GIMP_SAVE_PROCEDURE_DIALOG (dialog), "offs");
+  gimp_save_procedure_dialog_add_metadata (GIMP_SAVE_PROCEDURE_DIALOG (dialog), "phys");
+  gimp_save_procedure_dialog_add_metadata (GIMP_SAVE_PROCEDURE_DIALOG (dialog), "time");
+  gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dialog),
+                              "format", "compression",
+                              "interlaced", "save-transparent",
+                              NULL);
 
   run = gimp_procedure_dialog_run (GIMP_PROCEDURE_DIALOG (dialog));
 
+  gtk_widget_destroy (dialog);
+
   return run;
 }
