@@ -1303,7 +1303,7 @@ load_default_hotspot (GimpImage     *image,
 
   if (/* if we cannot load hotspot correctly */
       ! get_hotspot_from_parasite (image, &hot_spot_x, &hot_spot_y) ||
-      /* or hostspot is out of range */
+      /* or hotspot is out of range */
       ! pix_in_region (hot_spot_x, hot_spot_y, hotspot_range))
     {
       /* then use top left point of hotspot_range as fallback. */
@@ -1351,7 +1351,7 @@ save_image (GFile         *file,
   GList           *list;
   GList           *orig_list;
   gchar           *framename;              /* framename of a layer */
-  GeglRectangle    save_rgn;               /* region to save */
+  GeglRectangle    save_rgn = { 0 };       /* region to save */
   gint             layer_xoffset, layer_yoffset;
   /* temporary buffer which store pixel data (guchar * bpp = guint32) */
   guint32          pixelbuf[SQR (MAX_SAVE_DIMENSION)];
