@@ -107,6 +107,9 @@ static void             nlfiltRow                 (guchar           *srclast,
                                                    gint              bpp,
                                                    gint              filtno);
 
+static void    nlfilter_scale_entry_update_double (GimpLabelSpin    *entry,
+                                                   gdouble          *value);
+
 
 G_DEFINE_TYPE (Nlfilter, nlfilter, GIMP_TYPE_PLUG_IN)
 
@@ -395,7 +398,7 @@ nlfiltRow (guchar *srclast, guchar *srcthis, guchar *srcnext, guchar *dst,
 /* the pixel in question needs to be read. Given this, we only */
 /* need 3 or 4 weightings per hexagon, as follows: */
 /*                  _ _                         */
-/* Virtical hex:   |_|_|  1 2                   */
+/* Vertical hex:   |_|_|  1 2                   */
 /*                 |X|_|  0 3                   */
 /*                                       _      */
 /*              _                      _|_|   1 */
@@ -509,10 +512,10 @@ nlfiltInit (gdouble alpha, gdouble radius, FilterType filter)
       gdouble tabscale,mtabscale;
       gdouble v0,v1,v2,v3,m0,m1,m2,h0,h1,h2,h3;
 
-          /* horizontal offset of virtical hex centers */
+          /* horizontal offset of vertical hex centers */
       hexhoff = radius/2;
 
-          /* vertical offset of virtical hex centers */
+          /* vertical offset of vertical hex centers */
       hexvoff = 3.0 * radius/sqrt(12.0);
 
           /*
@@ -1079,16 +1082,16 @@ nlfilter_preview (GimpDrawable *drawable,
 static gboolean
 nlfilter_dialog (GimpDrawable *drawable)
 {
-  GtkWidget     *dialog;
-  GtkWidget     *main_vbox;
-  GtkWidget     *preview;
-  GtkWidget     *frame;
-  GtkWidget     *alpha_trim;
-  GtkWidget     *opt_est;
-  GtkWidget     *edge_enhance;
-  GtkWidget     *grid;
-  GtkAdjustment *adj;
-  gboolean       run;
+  GtkWidget *dialog;
+  GtkWidget *main_vbox;
+  GtkWidget *preview;
+  GtkWidget *frame;
+  GtkWidget *alpha_trim;
+  GtkWidget *opt_est;
+  GtkWidget *edge_enhance;
+  GtkWidget *grid;
+  GtkWidget *scale;
+  gboolean   run;
 
   gimp_ui_init (PLUG_IN_BINARY);
 
@@ -1154,29 +1157,25 @@ nlfilter_dialog (GimpDrawable *drawable)
   gtk_box_pack_start (GTK_BOX (main_vbox), grid, FALSE, FALSE, 0);
   gtk_widget_show (grid);
 
-  adj = gimp_scale_entry_new (GTK_GRID (grid), 0, 0,
-                              _("A_lpha:"), 0, 0,
-                              nlfvals.alpha, 0.0, 1.0, 0.05, 0.1, 2,
-                              TRUE, 0, 0,
-                              NULL, NULL);
-  g_signal_connect (adj, "value-changed",
-                    G_CALLBACK (gimp_double_adjustment_update),
+  scale = gimp_scale_entry_new (_("A_lpha:"), nlfvals.alpha, 0.0, 1.0, 2);
+  g_signal_connect (scale, "value-changed",
+                    G_CALLBACK (nlfilter_scale_entry_update_double),
                     &nlfvals.alpha);
-  g_signal_connect_swapped (adj, "value-changed",
+  g_signal_connect_swapped (scale, "value-changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
+  gtk_grid_attach (GTK_GRID (grid), scale, 0, 0, 3, 1);
+  gtk_widget_show (scale);
 
-  adj = gimp_scale_entry_new (GTK_GRID (grid), 0, 1,
-                              _("_Radius:"), 0, 0,
-                              nlfvals.radius, 1.0 / 3.0, 1.0, 0.05, 0.1, 2,
-                              TRUE, 0, 0,
-                              NULL, NULL);
-  g_signal_connect (adj, "value-changed",
-                    G_CALLBACK (gimp_double_adjustment_update),
+  scale = gimp_scale_entry_new (_("_Radius:"), nlfvals.radius, 1.0 / 3.0, 1.0, 2);
+  g_signal_connect (scale, "value-changed",
+                    G_CALLBACK (nlfilter_scale_entry_update_double),
                     &nlfvals.radius);
-  g_signal_connect_swapped (adj, "value-changed",
+  g_signal_connect_swapped (scale, "value-changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
+  gtk_grid_attach (GTK_GRID (grid), scale, 0, 1, 3, 1);
+  gtk_widget_show (scale);
 
   gtk_widget_show (dialog);
 
@@ -1186,3 +1185,10 @@ nlfilter_dialog (GimpDrawable *drawable)
 
   return run;
 }
+
+static void
+nlfilter_scale_entry_update_double (GimpLabelSpin *entry,
+                                    gdouble       *value)
+{
+  *value = gimp_label_spin_get_value (entry);
+}
