@@ -108,10 +108,20 @@ static void               load_paths       (TIFF              *tif,
                                             gint               offset_y);
 
 static void               fill_bit2byte    (void);
+static void               fill_2bit2byte   (void);
+static void               fill_4bit2byte   (void);
 static void               convert_bit2byte (const guchar      *src,
                                             guchar            *dest,
                                             gint               width,
                                             gint               height);
+static void              convert_2bit2byte (const guchar      *src,
+                                            guchar            *dest,
+                                            gint               width,
+                                            gint               height);
+static void              convert_4bit2byte (const guchar      *src,
+                                            guchar            *dest,
+                                            gint               width,
+                                            gint               height);
 
 static void               convert_int2uint (guchar            *buffer,
                                             gint               bps,
@@ -161,6 +171,7 @@ load_image (GFile        *file,
   gint               max_row            = 0;
   gint               max_col            = 0;
   gboolean           save_transp_pixels = FALSE;
+  GimpColorProfile  *first_profile      = NULL;
   gint               li;
 
   *image = NULL;
@@ -181,12 +192,37 @@ load_image (GFile        *file,
   pages.n_pages = pages.o_pages = TIFFNumberOfDirectories (tif);
   if (pages.n_pages == 0)
     {
-      TIFFClose (tif);
-      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
-                   _("TIFF '%s' does not contain any directories"),
-                   gimp_file_get_utf8_name (file));
+      /* See #5837.
+       * It seems we might be able to rescue some data even though the
+       * TIFF is possibly syntactically wrong.
+       */
 
-      return GIMP_PDB_EXECUTION_ERROR;
+      /* libtiff says max number of directory is 65535. */
+      for (li = 0; li < 65536; li++)
+        {
+          if (TIFFSetDirectory (tif, li) == 0)
+            break;
+        }
+      pages.n_pages = li;
+      if (pages.n_pages == 0)
+        {
+          TIFFClose (tif);
+          g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                       _("TIFF '%s' does not contain any directories"),
+                       gimp_file_get_utf8_name (file));
+
+          return GIMP_PDB_EXECUTION_ERROR;
+        }
+
+      TIFFSetDirectory (tif, 0);
+      g_message (ngettext ("TIFF '%s' directory count by header failed "
+                           "though there seems to be %d page."
+                           " Attempting to load the file with this assumption.",
+                           "TIFF '%s' directory count by header failed "
+                           "though there seem to be %d pages."
+                           " Attempting to load the file with this assumption.",
+                           pages.n_pages),
+                 gimp_file_get_utf8_name (file), pages.n_pages);
     }
 
   pages.pages = NULL;
@@ -215,7 +251,9 @@ load_image (GFile        *file,
           gushort  extra;
           gushort *extra_types;
 
-          TIFFSetDirectory (tif, li);
+          if (TIFFSetDirectory (tif, li) == 0)
+            continue;
+
           TIFFGetFieldDefaulted (tif, TIFFTAG_SAMPLESPERPIXEL, &spp);
           if (! TIFFGetField (tif, TIFFTAG_PHOTOMETRIC, &photomet))
             {
@@ -322,7 +360,7 @@ load_image (GFile        *file,
       gushort          *extra_types;
       ChannelData      *channel = NULL;
       uint16            planar  = PLANARCONFIG_CONTIG;
-      gboolean          is_bw;
+      gboolean          is_bw; /* FIXME Might need to change the name since we use it for palette too now */
       gboolean          is_signed;
       gint              i;
       gboolean          worst_case = FALSE;
@@ -344,12 +382,38 @@ load_image (GFile        *file,
       TIFFGetFieldDefaulted (tif, TIFFTAG_SAMPLEFORMAT, &sampleformat);
 
       profile = load_profile (tif);
+      if (! profile && first_profile)
+        {
+          profile = first_profile;
+          g_object_ref (profile);
+        }
+
       if (profile)
         {
+          profile_linear = gimp_color_profile_is_linear (profile);
+
+          if (! first_profile)
+            {
+              first_profile = profile;
+              g_object_ref (first_profile);
+
+              if (profile_linear && li > 0 && pages.target != GIMP_PAGE_SELECTOR_TARGET_IMAGES)
+                g_message (_("This image has a linear color profile but "
+                             "it was not set on the first layer. "
+                             "The layers below layer # %d will be "
+                             "interpreted as non linear."), li+1);
+            }
+          else if (pages.target != GIMP_PAGE_SELECTOR_TARGET_IMAGES &&
+                   ! gimp_color_profile_is_equal (first_profile, profile))
+            {
+              g_message (_("This image has multiple color profiles. "
+                           "We will use the first one. If this leads "
+                           "to incorrect results you should consider "
+                           "loading each layer as a separate image."));
+            }
+
           if (! *image)
             *profile_loaded = TRUE;
-
-          profile_linear = gimp_color_profile_is_linear (profile);
         }
 
       if (bps > 8 && bps != 8 && bps != 16 && bps != 32 && bps != 64)
@@ -358,6 +422,8 @@ load_image (GFile        *file,
       switch (bps)
         {
         case 1:
+        case 2:
+        case 4:
         case 8:
           if (profile_linear)
             image_precision = GIMP_PRECISION_U8_LINEAR;
@@ -563,13 +629,18 @@ load_image (GFile        *file,
         {
         case PHOTOMETRIC_MINISBLACK:
         case PHOTOMETRIC_MINISWHITE:
-          if (bps == 1 && ! alpha && spp == 1)
+          if ((bps == 1 || bps == 2 || bps == 4) && ! alpha && spp == 1)
             {
               image_type = GIMP_INDEXED;
               layer_type = GIMP_INDEXED_IMAGE;
 
               is_bw = TRUE;
-              fill_bit2byte ();
+              if (bps == 1)
+                fill_bit2byte ();
+              else if (bps == 2)
+                fill_2bit2byte ();
+              else if (bps == 4)
+                fill_4bit2byte ();
             }
           else
             {
@@ -716,6 +787,14 @@ load_image (GFile        *file,
         case PHOTOMETRIC_PALETTE:
           image_type = GIMP_INDEXED;
           layer_type = alpha ? GIMP_INDEXEDA_IMAGE : GIMP_INDEXED_IMAGE;
+
+          is_bw = TRUE;
+          if (bps == 1)
+            fill_bit2byte ();
+          else if (bps == 2)
+            fill_2bit2byte ();
+          else if (bps == 4)
+            fill_4bit2byte ();
           break;
 
         default:
@@ -836,10 +915,11 @@ load_image (GFile        *file,
         }
 
       /* attach color profile */
-
       if (profile)
         {
-          gimp_image_set_color_profile (*image, profile);
+          if (pages.target == GIMP_PAGE_SELECTOR_TARGET_IMAGES || profile == first_profile)
+            gimp_image_set_color_profile (*image, profile);
+
           g_object_unref (profile);
         }
 
@@ -887,6 +967,74 @@ load_image (GFile        *file,
           }
       }
 
+
+      /* Attach GeoTIFF Tags as Parasite, If available */
+      {
+        GimpParasite *parasite    = NULL;
+        void         *geotag_data = NULL;
+        uint32        count       = 0;
+
+        if (TIFFGetField (tif, GEOTIFF_MODELPIXELSCALE, &count, &geotag_data))
+          {
+            parasite = gimp_parasite_new ("Gimp_GeoTIFF_ModelPixelScale",
+                                          GIMP_PARASITE_PERSISTENT,
+                                          (TIFFDataWidth (TIFF_DOUBLE) * count),
+                                          geotag_data);
+            gimp_image_attach_parasite (*image, parasite);
+            gimp_parasite_free (parasite);
+          }
+
+        if (TIFFGetField (tif, GEOTIFF_MODELTIEPOINT, &count, &geotag_data))
+          {
+            parasite = gimp_parasite_new ("Gimp_GeoTIFF_ModelTiePoint",
+                                          GIMP_PARASITE_PERSISTENT,
+                                          (TIFFDataWidth (TIFF_DOUBLE) * count),
+                                          geotag_data);
+            gimp_image_attach_parasite (*image, parasite);
+            gimp_parasite_free (parasite);
+          }
+
+        if (TIFFGetField (tif, GEOTIFF_MODELTRANSFORMATION, &count, &geotag_data))
+          {
+            parasite = gimp_parasite_new ("Gimp_GeoTIFF_ModelTransformation",
+                                          GIMP_PARASITE_PERSISTENT,
+                                          (TIFFDataWidth (TIFF_DOUBLE) * count),
+                                          geotag_data);
+            gimp_image_attach_parasite (*image, parasite);
+            gimp_parasite_free (parasite);
+          }
+
+        if (TIFFGetField (tif, GEOTIFF_KEYDIRECTORY, &count, &geotag_data) )
+          {
+            parasite = gimp_parasite_new ("Gimp_GeoTIFF_KeyDirectory",
+                                          GIMP_PARASITE_PERSISTENT,
+                                          (TIFFDataWidth (TIFF_SHORT) * count),
+                                          geotag_data);
+            gimp_image_attach_parasite (*image, parasite);
+            gimp_parasite_free (parasite);
+          }
+
+        if (TIFFGetField (tif, GEOTIFF_DOUBLEPARAMS, &count, &geotag_data))
+          {
+            parasite = gimp_parasite_new ("Gimp_GeoTIFF_DoubleParams",
+                                          GIMP_PARASITE_PERSISTENT,
+                                          (TIFFDataWidth (TIFF_DOUBLE) * count),
+                                          geotag_data);
+            gimp_image_attach_parasite (*image, parasite);
+            gimp_parasite_free (parasite);
+          }
+
+        if (TIFFGetField (tif, GEOTIFF_ASCIIPARAMS, &count, &geotag_data))
+          {
+            parasite = gimp_parasite_new ("Gimp_GeoTIFF_Asciiparams",
+                                          GIMP_PARASITE_PERSISTENT,
+                                          (TIFFDataWidth (TIFF_ASCII) * count),
+                                          geotag_data);
+            gimp_image_attach_parasite (*image, parasite);
+            gimp_parasite_free (parasite);
+          }
+      }
+
       /* any resolution info in the file? */
       {
         gfloat   xres = 72.0;
@@ -987,17 +1135,75 @@ load_image (GFile        *file,
         {
           guchar cmap[768];
 
-          if (is_bw)
+          if (is_bw && photomet != PHOTOMETRIC_PALETTE)
             {
               if (photomet == PHOTOMETRIC_MINISWHITE)
                 {
-                  cmap[0] = cmap[1] = cmap[2] = 255;
-                  cmap[3] = cmap[4] = cmap[5] = 0;
+                  if (bps == 1)
+                    {
+                      cmap[0] = cmap[1] = cmap[2] = 255;
+                      cmap[3] = cmap[4] = cmap[5] = 0;
+                    }
+                  else if (bps == 2)
+                    {
+                      cmap[0] = cmap[1]  = cmap[2]  = 255;
+                      cmap[3] = cmap[4]  = cmap[5]  = 170;
+                      cmap[6] = cmap[7]  = cmap[8]  = 85;
+                      cmap[9] = cmap[10] = cmap[11] = 0;
+                    }
+                  else if (bps == 4)
+                    {
+                      cmap[0]  = cmap[1]  = cmap[2]  = 255;
+                      cmap[3]  = cmap[4]  = cmap[5]  = 238;
+                      cmap[6]  = cmap[7]  = cmap[8]  = 221;
+                      cmap[9]  = cmap[10] = cmap[11] = 204;
+                      cmap[12] = cmap[13] = cmap[14] = 187;
+                      cmap[15] = cmap[16] = cmap[17] = 170;
+                      cmap[18] = cmap[19] = cmap[20] = 153;
+                      cmap[21] = cmap[22] = cmap[23] = 136;
+                      cmap[24] = cmap[25] = cmap[26] = 119;
+                      cmap[27] = cmap[28] = cmap[29] = 102;
+                      cmap[30] = cmap[31] = cmap[32] = 85;
+                      cmap[33] = cmap[34] = cmap[35] = 68;
+                      cmap[36] = cmap[37] = cmap[38] = 51;
+                      cmap[39] = cmap[40] = cmap[41] = 34;
+                      cmap[42] = cmap[43] = cmap[44] = 17;
+                      cmap[45] = cmap[46] = cmap[47] = 0;
+                    }
                 }
               else
                 {
-                  cmap[0] = cmap[1] = cmap[2] = 0;
-                  cmap[3] = cmap[4] = cmap[5] = 255;
+                  if (bps == 1)
+                    {
+                      cmap[0] = cmap[1] = cmap[2] = 0;
+                      cmap[3] = cmap[4] = cmap[5] = 255;
+                    }
+                  else if (bps == 2)
+                    {
+                      cmap[0] = cmap[1]  = cmap[2]  = 0;
+                      cmap[3] = cmap[4]  = cmap[5]  = 85;
+                      cmap[6] = cmap[7]  = cmap[8]  = 170;
+                      cmap[9] = cmap[10] = cmap[11] = 255;
+                    }
+                  else if (bps == 4)
+                    {
+                      cmap[0]  = cmap[1]  = cmap[2]  = 0;
+                      cmap[3]  = cmap[4]  = cmap[5]  = 17;
+                      cmap[6]  = cmap[7]  = cmap[8]  = 34;
+                      cmap[9]  = cmap[10] = cmap[11] = 51;
+                      cmap[12] = cmap[13] = cmap[14] = 68;
+                      cmap[15] = cmap[16] = cmap[17] = 85;
+                      cmap[18] = cmap[19] = cmap[20] = 102;
+                      cmap[21] = cmap[22] = cmap[23] = 119;
+                      cmap[24] = cmap[25] = cmap[26] = 136;
+                      cmap[27] = cmap[28] = cmap[29] = 153;
+                      cmap[30] = cmap[31] = cmap[32] = 170;
+                      cmap[33] = cmap[34] = cmap[35] = 187;
+                      cmap[36] = cmap[37] = cmap[38] = 204;
+                      cmap[39] = cmap[40] = cmap[41] = 221;
+                      cmap[42] = cmap[43] = cmap[44] = 238;
+                      cmap[45] = cmap[46] = cmap[47] = 255;
+                    }
                 }
             }
           else
@@ -1218,6 +1424,7 @@ load_image (GFile        *file,
 
       gimp_progress_update (1.0);
     }
+  g_clear_object (&first_profile);
 
   if (pages.target == GIMP_PAGE_SELECTOR_TARGET_IMAGES)
     {
@@ -1239,6 +1446,16 @@ load_image (GFile        *file,
     }
   else
     {
+      if (! (*image))
+        {
+          TIFFClose (tif);
+          g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                       _("No data could be read from TIFF '%s'. The file is probably corrupted."),
+                       gimp_file_get_utf8_name (file));
+
+          return GIMP_PDB_EXECUTION_ERROR;
+        }
+
       if (pages.keep_empty_space)
         {
           /* unfortunately we have no idea about empty space
@@ -1619,7 +1836,12 @@ load_contiguous (TIFF        *tif,
 
           if (is_bw)
             {
-              convert_bit2byte (buffer, bw_buffer, cols, rows);
+              if (bps == 1)
+                convert_bit2byte (buffer, bw_buffer, cols, rows);
+              else if (bps == 2)
+                convert_2bit2byte (buffer, bw_buffer, cols, rows);
+              else if (bps == 4)
+                convert_4bit2byte (buffer, bw_buffer, cols, rows);
             }
           else if (is_signed)
             {
@@ -1794,7 +2016,12 @@ load_separate (TIFF        *tif,
 
                   if (is_bw)
                     {
-                      convert_bit2byte (buffer, bw_buffer, cols, rows);
+                      if (bps == 1)
+                        convert_bit2byte (buffer, bw_buffer, cols, rows);
+                      else if (bps == 2)
+                        convert_2bit2byte (buffer, bw_buffer, cols, rows);
+                      else if (bps == 4)
+                        convert_4bit2byte (buffer, bw_buffer, cols, rows);
                     }
                   else if (is_signed)
                     {
@@ -1851,7 +2078,9 @@ load_separate (TIFF        *tif,
 }
 
 
-static guchar bit2byte[256 * 8];
+static guchar   bit2byte[256 * 8];
+static guchar _2bit2byte[256 * 4];
+static guchar _4bit2byte[256 * 2];
 
 static void
 fill_bit2byte (void)
@@ -1873,6 +2102,54 @@ fill_bit2byte (void)
   filled = TRUE;
 }
 
+static void
+fill_2bit2byte (void)
+{
+  static gboolean filled2 = FALSE;
+
+  guchar *dest;
+  gint    i, j;
+
+  if (filled2)
+    return;
+
+  dest = _2bit2byte;
+
+  for (j = 0; j < 256; j++)
+    {
+    for (i = 3; i >= 0; i--)
+      {
+        *(dest++) = ((j & (3 << (2*i))) >> (2*i));
+      }
+    }
+
+  filled2 = TRUE;
+}
+
+static void
+fill_4bit2byte (void)
+{
+  static gboolean filled4 = FALSE;
+
+  guchar *dest;
+  gint    i, j;
+
+  if (filled4)
+    return;
+
+  dest = _4bit2byte;
+
+  for (j = 0; j < 256; j++)
+    {
+    for (i = 1; i >= 0; i--)
+      {
+        *(dest++) = ((j & (15 << (4*i))) >> (4*i));
+      }
+    }
+
+  filled4 = TRUE;
+}
+
 static void
 convert_bit2byte (const guchar *src,
                   guchar       *dest,
@@ -1902,6 +2179,64 @@ convert_bit2byte (const guchar *src,
     }
 }
 
+static void
+convert_2bit2byte (const guchar *src,
+                   guchar       *dest,
+                   gint          width,
+                   gint          height)
+{
+  gint y;
+
+  for (y = 0; y < height; y++)
+    {
+      gint x = width;
+
+      while (x >= 4)
+        {
+          memcpy (dest, _2bit2byte + *src * 4, 4);
+          dest += 4;
+          x -= 4;
+          src++;
+        }
+
+      if (x > 0)
+        {
+          memcpy (dest, _2bit2byte + *src * 4, x);
+          dest += x;
+          src++;
+        }
+    }
+}
+
+static void
+convert_4bit2byte (const guchar *src,
+                   guchar       *dest,
+                   gint          width,
+                   gint          height)
+{
+  gint y;
+
+  for (y = 0; y < height; y++)
+    {
+      gint x = width;
+
+      while (x >= 2)
+        {
+          memcpy (dest, _4bit2byte + *src * 2, 2);
+          dest += 2;
+          x -= 2;
+          src++;
+        }
+
+      if (x > 0)
+        {
+          memcpy (dest, _4bit2byte + *src * 2, x);
+          dest += x;
+          src++;
+        }
+    }
+}
+
 static void
 convert_int2uint (guchar *buffer,
                   gint    bps,
