@@ -320,10 +320,12 @@ save_layer (TIFF        *tif,
   gint              offset_x, offset_y;
   gint              config_compression;
   gboolean          config_save_transp_pixels;
+  gboolean          config_save_geotiff_tags;
 
   g_object_get (config,
                 "compression",             &config_compression,
                 "save-transparent-pixels", &config_save_transp_pixels,
+                "save-geotiff",            &config_save_geotiff_tags,
                 NULL);
 
   compression = gimp_compression_to_tiff_compression (config_compression);
@@ -771,6 +773,72 @@ save_layer (TIFF        *tif,
                               * (gdouble) row / (gdouble) rows);
     }
 
+  /* Save GeoTIFF tags to file, if available */
+  if (config_save_geotiff_tags)
+    {
+      GimpParasite *parasite = NULL;
+
+      parasite = gimp_image_get_parasite (image,"Gimp_GeoTIFF_ModelPixelScale");
+
+      if (parasite)
+        {
+          TIFFSetField (tif,
+                        GEOTIFF_MODELPIXELSCALE,
+                        (gimp_parasite_data_size (parasite) / TIFFDataWidth (TIFF_DOUBLE)),
+                        gimp_parasite_data (parasite));
+          gimp_parasite_free (parasite);
+        }
+
+      parasite = gimp_image_get_parasite (image,"Gimp_GeoTIFF_ModelTiePoint");
+      if (parasite)
+        {
+          TIFFSetField (tif,
+                        GEOTIFF_MODELTIEPOINT,
+                        (gimp_parasite_data_size (parasite) / TIFFDataWidth (TIFF_DOUBLE)),
+                        gimp_parasite_data (parasite));
+          gimp_parasite_free (parasite);
+        }
+
+      parasite = gimp_image_get_parasite (image,"Gimp_GeoTIFF_ModelTransformation");
+      if (parasite)
+        {
+          TIFFSetField (tif,
+                        GEOTIFF_MODELTRANSFORMATION,
+                        (gimp_parasite_data_size (parasite) / TIFFDataWidth (TIFF_DOUBLE)),
+                        gimp_parasite_data (parasite));
+          gimp_parasite_free (parasite);
+        }
+
+      parasite = gimp_image_get_parasite (image,"Gimp_GeoTIFF_KeyDirectory");
+      if (parasite)
+        {
+          TIFFSetField (tif,
+                        GEOTIFF_KEYDIRECTORY,
+                        (gimp_parasite_data_size (parasite) / TIFFDataWidth (TIFF_SHORT)),
+                        gimp_parasite_data (parasite));
+          gimp_parasite_free (parasite);
+        }
+
+      parasite = gimp_image_get_parasite (image,"Gimp_GeoTIFF_DoubleParams");
+      if (parasite)
+        {
+          TIFFSetField (tif,
+                        GEOTIFF_DOUBLEPARAMS,
+                        (gimp_parasite_data_size (parasite) / TIFFDataWidth (TIFF_DOUBLE)),
+                        gimp_parasite_data (parasite));
+          gimp_parasite_free (parasite);
+        }
+
+      parasite = gimp_image_get_parasite (image,"Gimp_GeoTIFF_Asciiparams");
+      if (parasite)
+        {
+          TIFFSetField (tif,
+                        GEOTIFF_ASCIIPARAMS,
+                        gimp_parasite_data (parasite));
+          gimp_parasite_free (parasite);
+        }
+    }
+
   TIFFWriteDirectory (tif);
 
   gimp_progress_update (progress_base + progress_fraction);
@@ -1208,32 +1276,15 @@ save_dialog (GimpImage     *image,
              gboolean       is_multi_layer)
 {
   GtkWidget       *dialog;
-  GtkWidget       *main_vbox;
-  GtkWidget       *grid;
   GtkListStore    *store;
   GtkWidget       *combo;
-  GtkWidget       *button;
   GtkWidget       *frame;
-  GtkWidget       *entry;
   GimpCompression  compression;
-  gint             row = 0;
   gboolean         run;
 
-  dialog = gimp_procedure_dialog_new (procedure,
-                                      GIMP_PROCEDURE_CONFIG (config),
-                                      _("Export Image as TIFF"));
-
-  main_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-  gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
-  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
-                      main_vbox, TRUE, TRUE, 0);
-  gtk_widget_show (main_vbox);
-
-  grid = gtk_grid_new ();
-  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
-  gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
-  gtk_box_pack_start (GTK_BOX (main_vbox), grid, FALSE, FALSE, 0);
-  gtk_widget_show (grid);
+  dialog = gimp_save_procedure_dialog_new (GIMP_SAVE_PROCEDURE (procedure),
+                                           GIMP_PROCEDURE_CONFIG (config),
+                                           image);
 
   store =
     gimp_int_store_new (_("None"),              GIMP_COMPRESSION_NONE,
@@ -1244,23 +1295,43 @@ save_dialog (GimpImage     *image,
                         _("CCITT Group 3 fax"), GIMP_COMPRESSION_CCITTFAX3,
                         _("CCITT Group 4 fax"), GIMP_COMPRESSION_CCITTFAX4,
                         NULL);
-
-  combo = gimp_prop_int_combo_box_new (config, "compression",
-                                       GIMP_INT_STORE (store));
+  combo = gimp_procedure_dialog_get_int_combo (GIMP_PROCEDURE_DIALOG (dialog),
+                                               "compression", GIMP_INT_STORE (store));
   g_object_unref (store);
-
-  gimp_grid_attach_aligned (GTK_GRID (grid), 0, row++,
-                            _("_Compression:"), 0.0, 0.5,
-                            combo, 1);
-
+  combo = gimp_label_int_widget_get_widget (GIMP_LABEL_INT_WIDGET (combo));
   gimp_int_combo_box_set_sensitivity (GIMP_INT_COMBO_BOX (combo),
                                       combo_sensitivity_func,
                                       combo, NULL);
-
   combo_set_item_sensitive (combo, GIMP_COMPRESSION_CCITTFAX3, is_monochrome);
   combo_set_item_sensitive (combo, GIMP_COMPRESSION_CCITTFAX4, is_monochrome);
   combo_set_item_sensitive (combo, GIMP_COMPRESSION_JPEG,      ! is_indexed);
 
+  frame = gimp_procedure_dialog_fill_frame (GIMP_PROCEDURE_DIALOG (dialog),
+                                            "layers-frame", "save-layers", FALSE,
+                                            "crop-layers");
+  /* TODO: if single-layer TIFF, set the toggle insensitive and show it
+   * as unchecked though I don't actually change the config value to
+   * keep storing previously chosen value.
+   * This used to be so before. We probably need to add some logics in
+   * the GimpProcedureDialog generation for such case.
+   */
+  gtk_widget_set_sensitive (frame, is_multi_layer);
+  /* TODO: same for "save-transparent-pixels", we probably want to show
+   * it unchecked even though it doesn't matter for processing.
+   */
+  gtk_widget_set_sensitive (gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dialog),
+                                                              "save-transparent-pixels",
+                                                              G_TYPE_NONE),
+                            has_alpha && ! is_indexed);
+
+  gimp_save_procedure_dialog_add_metadata (GIMP_SAVE_PROCEDURE_DIALOG (dialog), "save-geotiff");
+
+  gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dialog),
+                              "compression",
+                              "layers-frame",
+                              "save-transparent-pixels",
+                              NULL);
+
   g_object_get (config,
                 "compression", &compression,
                 NULL);
@@ -1283,95 +1354,6 @@ save_dialog (GimpImage     *image,
                 "compression", compression,
                 NULL);
 
-  if (is_multi_layer)
-    {
-      button = gimp_prop_check_button_new (config, "save-layers",
-                                           _("Save layers"));
-    }
-  else
-    {
-      /* If single-layer TIFF, set the toggle insensitive and show it as
-       * unchecked though I don't actually change the config value to
-       * keep storing previously chosen value.
-       */
-      button = gtk_check_button_new_with_mnemonic (_("Save layers"));
-      gtk_widget_set_sensitive (button, FALSE);
-      gtk_widget_show (button);
-    }
-
-  gtk_grid_attach (GTK_GRID (grid), button, 0, row++, 2, 1);
-
-  frame = gimp_frame_new (NULL);
-  gtk_grid_attach (GTK_GRID (grid), frame, 0, row++, 2, 1);
-  gtk_widget_show (frame);
-
-  if (is_multi_layer)
-    {
-      g_object_bind_property (config, "save-layers",
-                              frame,  "sensitive",
-                              G_BINDING_SYNC_CREATE);
-
-      button = gimp_prop_check_button_new (config, "crop-layers",
-                                           _("Crop layers to image bounds"));
-      gtk_container_add (GTK_CONTAINER (frame), button);
-    }
-  else
-    {
-      /* If single-layer TIFF, set the layers frame insensitive and
-       * show its toggles as unchecked though I don't actually change
-       * the config value to keep storing previously chosen values.
-       */
-      gtk_widget_set_sensitive (frame, FALSE);
-
-      button = gtk_check_button_new_with_mnemonic (_("Crop layers"));
-      gtk_container_add (GTK_CONTAINER (frame), button);
-      gtk_widget_show (button);
-    }
-
-  if (has_alpha && ! is_indexed)
-    {
-      button = gimp_prop_check_button_new (config, "save-transparent-pixels",
-                                           _("Save color values from transparent pixels"));
-    }
-  else
-    {
-      button = gtk_check_button_new_with_mnemonic (_("Save color values from transparent pixels"));
-      gtk_widget_set_sensitive (button, FALSE);
-      gtk_widget_show (button);
-    }
-
-  gtk_grid_attach (GTK_GRID (grid), button, 0, row++, 2, 1);
-
-  button = gimp_prop_check_button_new (config, "save-exif",
-                                       _("Save Exif data"));
-  gtk_grid_attach (GTK_GRID (grid), button, 0, row++, 2, 1);
-
-  button = gimp_prop_check_button_new (config, "save-xmp",
-                                       _("Save XMP data"));
-  gtk_grid_attach (GTK_GRID (grid), button, 0, row++, 2, 1);
-
-  button = gimp_prop_check_button_new (config, "save-iptc",
-                                       _("Save IPTC data"));
-  gtk_grid_attach (GTK_GRID (grid), button, 0, row++, 2, 1);
-
-  button = gimp_prop_check_button_new (config, "save-thumbnail",
-                                       _("Save thumbnail"));
-  gtk_grid_attach (GTK_GRID (grid), button, 0, row++, 2, 1);
-
-#ifdef TIFFTAG_ICCPROFILE
-  button = gimp_prop_check_button_new (config, "save-color-profile",
-                                       _("Save color profile"));
-  gtk_grid_attach (GTK_GRID (grid), button, 0, row++, 2, 1);
-#endif
-
-  frame = gimp_frame_new (_("Comment"));
-  gtk_grid_attach (GTK_GRID (grid), frame, 0, row++, 2, 1);
-  gtk_widget_show (frame);
-
-  entry = gimp_prop_entry_new (config, "gimp-comment", -1);
-  gtk_container_add (GTK_CONTAINER (frame), entry);
-  gtk_widget_show (entry);
-
   gtk_widget_show (dialog);
 
   run = gimp_procedure_dialog_run (GIMP_PROCEDURE_DIALOG (dialog));
