@@ -188,13 +188,15 @@ tiff_create_procedure (GimpPlugIn  *plug_in,
 
       gimp_file_procedure_set_handles_remote (GIMP_FILE_PROCEDURE (procedure),
                                               TRUE);
+      gimp_file_procedure_set_format_name (GIMP_FILE_PROCEDURE (procedure),
+                                           _("TIFF"));
       gimp_file_procedure_set_mime_types (GIMP_FILE_PROCEDURE (procedure),
                                           "image/tiff");
       gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                           "tif,tiff");
 
       GIMP_PROC_ARG_INT (procedure, "compression",
-                         "Compression",
+                         "Co_mpression",
                          "Compression type: { NONE (0), LZW (1), PACKBITS (2), "
                          "DEFLATE (3), JPEG (4), CCITT G3 Fax (5), "
                          "CCITT G4 Fax (6) }",
@@ -202,62 +204,38 @@ tiff_create_procedure (GimpPlugIn  *plug_in,
                          G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "save-transparent-pixels",
-                             "Save transparent pixels",
+                             "Save color _values from transparent pixels",
                              "Keep the color data masked by an alpha channel "
                              "intact (do not store premultiplied components)",
                              TRUE,
                              G_PARAM_READWRITE);
 
-      GIMP_PROC_AUX_ARG_STRING (procedure, "gimp-comment",
-                                "Comment",
-                                "Image comment",
-                                gimp_get_default_comment (),
-                                G_PARAM_READWRITE);
-
-      gimp_procedure_set_argument_sync (procedure, "gimp-comment",
-                                        GIMP_ARGUMENT_SYNC_PARASITE);
-
       GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-layers",
-                                 "Save Layers",
+                                 "Save La_yers",
                                  "Save Layers",
                                  TRUE,
                                  G_PARAM_READWRITE);
 
       GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "crop-layers",
-                                 "Crop Layers",
+                                 "Crop L_ayers",
                                  "Crop Layers",
                                  TRUE,
                                  G_PARAM_READWRITE);
 
-      GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-exif",
-                                 "Save Exif",
-                                 "Save Exif",
-                                 gimp_export_exif (),
-                                 G_PARAM_READWRITE);
-
-      GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-xmp",
-                                 "Save XMP",
-                                 "Save XMP",
-                                 gimp_export_xmp (),
-                                 G_PARAM_READWRITE);
-
-      GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-iptc",
-                                 "Save IPTC",
-                                 "Save IPTC",
-                                 gimp_export_iptc (),
-                                 G_PARAM_READWRITE);
-
-      GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-thumbnail",
-                                 "Save thumbnail",
-                                 "Save thumbnail",
+      GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-geotiff",
+                                 "Save _GeoTIFF data",
+                                 "Save GeoTIFF data",
                                  TRUE,
                                  G_PARAM_READWRITE);
 
-      GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-color-profile",
-                                 "Save profile",
-                                 "Save color profile",
-                                 gimp_export_color_profile (),
-                                 G_PARAM_READWRITE);
+      gimp_save_procedure_set_support_exif      (GIMP_SAVE_PROCEDURE (procedure), TRUE);
+      gimp_save_procedure_set_support_iptc      (GIMP_SAVE_PROCEDURE (procedure), TRUE);
+      gimp_save_procedure_set_support_xmp       (GIMP_SAVE_PROCEDURE (procedure), TRUE);
+#ifdef TIFFTAG_ICCPROFILE
+      gimp_save_procedure_set_support_profile   (GIMP_SAVE_PROCEDURE (procedure), TRUE);
+#endif
+      gimp_save_procedure_set_support_thumbnail (GIMP_SAVE_PROCEDURE (procedure), TRUE);
+      gimp_save_procedure_set_support_comment   (GIMP_SAVE_PROCEDURE (procedure), TRUE);
     }
 
   return procedure;
