@@ -27,13 +27,6 @@
 #include "metadata-tags.h"
 
 
-/* The meaning of "single" and "multi" here denotes whether it is used in a
- * single line or a multi line edit field.
- * Depending on it's xmp type multi line can be saved as either:
- * - one tag of type text, possibly including newlines
- * - an array of tags of the same type for seq and bag, where each line in
- *   the multi line edit will be one item in the array
- */
 const metadata_tag default_metadata_tags[] =
 {
   /* Description */
@@ -141,16 +134,12 @@ const metadata_tag default_metadata_tags[] =
 };
 const gint n_default_metadata_tags = G_N_ELEMENTS (default_metadata_tags);
 
-/* Then meaning of "single" and "multi" below is a little different than above.
- * "single" - for iptc tags that can appear only once,
- * "multi"  - for iptc tags that are repeatable, i.e. can appear multiple times.
- */
 const metadata_tag equivalent_metadata_tags[] =
 {
   { "Iptc.Application2.DateCreated",             "single", 10,  TAG_TYPE_IPTC, GIMP_XMP_NONE }, //  0
   { "Iptc.Application2.TransmissionReference",   "single", 12,  TAG_TYPE_IPTC, GIMP_XMP_NONE }, //  1
   { "Iptc.Application2.SpecialInstructions",     "single", 13,  TAG_TYPE_IPTC, GIMP_XMP_NONE }, //  2
-  { "Iptc.Application2.Headline",                "single", 11,  TAG_TYPE_IPTC, GIMP_XMP_NONE }, //  3
+  { "Iptc.Application2.Headline",                "multi",  11,  TAG_TYPE_IPTC, GIMP_XMP_NONE }, //  3
   { "Iptc.Application2.Category",                "single", 56,  TAG_TYPE_IPTC, GIMP_XMP_NONE }, //  4
   { "Iptc.Application2.City",                    "single", 20,  TAG_TYPE_IPTC, GIMP_XMP_NONE }, //  5
   { "Iptc.Application2.ProvinceState",           "single", 21,  TAG_TYPE_IPTC, GIMP_XMP_NONE }, //  6
@@ -161,7 +150,7 @@ const metadata_tag equivalent_metadata_tags[] =
   { "Iptc.Application2.Urgency",                 "combo",  26,  TAG_TYPE_IPTC, GIMP_XMP_NONE }, // 11
   { "Iptc.Application2.SubLocation",             "single", 28,  TAG_TYPE_IPTC, GIMP_XMP_NONE }, // 12
   { "Iptc.Application2.Byline",                  "single",  1,  TAG_TYPE_IPTC, GIMP_XMP_NONE }, // 13
-  { "Iptc.Application2.Caption",                 "single",  2,  TAG_TYPE_IPTC, GIMP_XMP_NONE }, // 14
+  { "Iptc.Application2.Caption",                 "multi",   2,  TAG_TYPE_IPTC, GIMP_XMP_NONE }, // 14
   { "Iptc.Application2.Keywords",                "multi",   3,  TAG_TYPE_IPTC, GIMP_XMP_NONE }, // 15
   { "Iptc.Application2.ObjectName",              "single",  0,  TAG_TYPE_IPTC, GIMP_XMP_NONE }, // 16
   { "Iptc.Application2.Copyright",               "single",  4,  TAG_TYPE_IPTC, GIMP_XMP_NONE }, // 17
@@ -177,7 +166,6 @@ const gint n_equivalent_metadata_tags = G_N_ELEMENTS (equivalent_metadata_tags);
  */
 const combobox_str_tag digitalsourcetype[] =
 {
-  { "",                                                              N_("Select a value")                                   },
   { "http://cv.iptc.org/newscodes/digitalsourcetype/digitalCapture", N_("Original digital capture of a real life scene")    },
   { "http://cv.iptc.org/newscodes/digitalsourcetype/negativeFilm",   N_("Digitized from a negative on film")                },
   { "http://cv.iptc.org/newscodes/digitalsourcetype/positiveFilm",   N_("Digitized from a positive on film")                },
@@ -191,11 +179,10 @@ const gint n_digitalsourcetype = G_N_ELEMENTS (digitalsourcetype);
  */
 const combobox_str_tag modelreleasestatus[] =
 {
-  { "",                                       N_("Select a value")                       },
   { "http://ns.useplus.org/ldf/vocab/MR-NON", N_("None")                                 },
   { "http://ns.useplus.org/ldf/vocab/MR-NAP", N_("Not Applicable")                       },
-  { "http://ns.useplus.org/ldf/vocab/MR-UMR", N_("Unlimited Model Releases")             },
-  { "http://ns.useplus.org/ldf/vocab/MR-LMR", N_("Limited or Incomplete Model Releases") }
+  { "http://ns.useplus.org/ldf/vocab/MR-NAP", N_("Unlimited Model Releases")             },
+  { "http://ns.useplus.org/ldf/vocab/MR-LPR", N_("Limited or Incomplete Model Releases") }
 };
 const gint n_modelreleasestatus = G_N_ELEMENTS (modelreleasestatus);
 
@@ -206,7 +193,7 @@ const combobox_str_tag propertyreleasestatus[] =
 {
   { "http://ns.useplus.org/ldf/vocab/PR-NON", N_("None")                                    },
   { "http://ns.useplus.org/ldf/vocab/PR-NAP", N_("Not Applicable")                          },
-  { "http://ns.useplus.org/ldf/vocab/PR-UPR", N_("Unlimited Property Releases")             },
+  { "http://ns.useplus.org/ldf/vocab/PR-NAP", N_("Unlimited Property Releases")             },
   { "http://ns.useplus.org/ldf/vocab/PR-LPR", N_("Limited or Incomplete Property Releases") }
 };
 const gint n_propertyreleasestatus = G_N_ELEMENTS (propertyreleasestatus);
@@ -361,18 +348,6 @@ const gchar *licensor[] =
 };
 const gint n_licensor = G_N_ELEMENTS (licensor);
 
-const gint licensor_special_handling[] =
-{
-  METADATA_NONE,
-  METADATA_NONE,
-  METADATA_NONE,
-  METADATA_PHONETYPE,
-  METADATA_NONE,
-  METADATA_PHONETYPE,
-  METADATA_NONE,
-  METADATA_NONE
-};
-
 #ifdef USE_TAGS
 const gchar *imagesupplier[] =
 {
@@ -403,33 +378,17 @@ const gchar *registryid[] =
 };
 const gint n_registryid = G_N_ELEMENTS (registryid);
 
-const gchar *registryid_alternative[] =
-{
-  "/iptcExt:RegOrgId",
-  "/iptcExt:RegItemId"
-};
-
 const gchar *artworkorobject[] =
 {
-  "/Iptc4xmpExt:AOTitle",
   "/Iptc4xmpExt:AODateCreated",
-  "/Iptc4xmpExt:AOCreator",
   "/Iptc4xmpExt:AOSource",
   "/Iptc4xmpExt:AOSourceInvNo",
+  "/Iptc4xmpExt:AOTitle",
   "/Iptc4xmpExt:AOCopyrightNotice",
+  "/Iptc4xmpExt:AOCreator"
 };
 const gint n_artworkorobject = G_N_ELEMENTS (artworkorobject);
 
-const gchar *artworkorobject_alternative[] =
-{
-  "/iptcExt:AOTitle",
-  "/iptcExt:AODateCreated",
-  "/iptcExt:AOCreator",
-  "/iptcExt:AOSource",
-  "/iptcExt:AOSourceInvNo",
-  "/iptcExt:AOCopyrightNotice",
-};
-
 const gchar *locationshown[] =
 {
   "/Iptc4xmpExt:Sublocation",
@@ -441,17 +400,6 @@ const gchar *locationshown[] =
 };
 const gint n_locationshown = G_N_ELEMENTS (locationshown);
 
-const gchar *locationshown_alternative[] =
-{
-  "/iptcExt:Sublocation",
-  "/iptcExt:City",
-  "/iptcExt:ProvinceState",
-  "/iptcExt:CountryName",
-  "/iptcExt:CountryCode",
-  "/iptcExt:WorldRegion"
-};
-
-
 #ifdef USE_TAGS
 const gchar *locationcreated[] =
 {
