@@ -30,6 +30,7 @@
 
 #include "libgimp/stdplugins-intl.h"
 
+#include "metadata-tags.h"
 
 #define PLUG_IN_PROC   "plug-in-metadata-viewer"
 #define PLUG_IN_BINARY "metadata-viewer"
@@ -98,17 +99,34 @@ static GimpValueArray * metadata_run              (GimpProcedure        *procedu
                                                    gpointer              run_data);
 
 static gboolean  metadata_viewer_dialog           (GimpImage      *image,
-                                                   GimpMetadata   *g_metadata);
+                                                   GimpMetadata   *g_metadata,
+                                                   GError        **error);
 static void      metadata_dialog_set_metadata     (GExiv2Metadata *metadata,
                                                    GtkBuilder     *builder);
 static void      metadata_dialog_append_tags      (GExiv2Metadata  *metadata,
                                                    gchar          **tags,
                                                    GtkListStore    *store,
                                                    gint             tag_column,
-                                                   gint             value_column);
+                                                   gint             value_column,
+                                                   gboolean         load_iptc);
+static void metadata_dialog_add_tag               (GtkListStore    *store,
+                                                   GtkTreeIter      iter,
+                                                   gint             tag_column,
+                                                   gint             value_column,
+                                                   const gchar     *tag,
+                                                   const gchar     *value);
+static void metadata_dialog_add_translated_tag    (GExiv2Metadata  *metadata,
+                                                   GtkListStore    *store,
+                                                   GtkTreeIter      iter,
+                                                   gint             tag_column,
+                                                   gint             value_column,
+                                                   const gchar     *tag);
 static gchar   * metadata_dialog_format_tag_value (GExiv2Metadata  *metadata,
                                                    const gchar     *tag,
                                                    gboolean         truncate);
+static gchar   * metadata_format_string_value     (const gchar     *value,
+                                                   gboolean         truncate);
+static inline gboolean metadata_tag_is_string     (const gchar     *tag);
 
 
 G_DEFINE_TYPE (Metadata, metadata, GIMP_TYPE_PLUG_IN)
@@ -191,6 +209,7 @@ metadata_run (GimpProcedure        *procedure,
 {
   GimpImage    *image;
   GimpMetadata *metadata;
+  GError       *error  = NULL;
 
   INIT_I18N ();
 
@@ -211,14 +230,16 @@ metadata_run (GimpProcedure        *procedure,
       gimp_image_set_metadata (image, metadata);
     }
 
-  metadata_viewer_dialog (image, metadata);
-
-  return gimp_procedure_new_return_values (procedure, GIMP_PDB_SUCCESS, NULL);
+  if (metadata_viewer_dialog (image, metadata, &error))
+    return gimp_procedure_new_return_values (procedure, GIMP_PDB_SUCCESS, NULL);
+  else
+    return gimp_procedure_new_return_values (procedure, GIMP_PDB_EXECUTION_ERROR, error);
 }
 
 static gboolean
-metadata_viewer_dialog (GimpImage    *image,
-                        GimpMetadata *g_metadata)
+metadata_viewer_dialog (GimpImage     *image,
+                        GimpMetadata  *g_metadata,
+                        GError       **error)
 {
   GtkBuilder     *builder;
   GtkWidget      *dialog;
@@ -227,7 +248,7 @@ metadata_viewer_dialog (GimpImage    *image,
   gchar          *ui_file;
   gchar          *title;
   gchar          *name;
-  GError         *error = NULL;
+  GError         *local_error = NULL;
   GExiv2Metadata *metadata;
 
   metadata = GEXIV2_METADATA(g_metadata);
@@ -237,11 +258,13 @@ metadata_viewer_dialog (GimpImage    *image,
   ui_file = g_build_filename (gimp_data_directory (),
                               "ui", "plug-ins", "plug-in-metadata-viewer.ui", NULL);
 
-  if (! gtk_builder_add_from_file (builder, ui_file, &error))
+  if (! gtk_builder_add_from_file (builder, ui_file, &local_error))
     {
-      g_printerr ("Error occurred while loading UI file!\n");
-      g_printerr ("Message: %s\n", error->message);
-      g_clear_error (&error);
+      if (! local_error)
+        local_error = g_error_new_literal (G_FILE_ERROR, 0,
+                                           _("Error loading metadata-viewer dialog."));
+      g_propagate_error (error, local_error);
+
       g_free (ui_file);
       g_object_unref (builder);
       return FALSE;
@@ -296,7 +319,7 @@ metadata_dialog_set_metadata (GExiv2Metadata *metadata,
   tags  = gexiv2_metadata_get_exif_tags (metadata);
   store = GTK_LIST_STORE (gtk_builder_get_object (builder, "exif-liststore"));
 
-  metadata_dialog_append_tags (metadata, tags, store, C_EXIF_TAG, C_EXIF_VALUE);
+  metadata_dialog_append_tags (metadata, tags, store, C_EXIF_TAG, C_EXIF_VALUE, FALSE);
 
   g_strfreev (tags);
 
@@ -304,7 +327,7 @@ metadata_dialog_set_metadata (GExiv2Metadata *metadata,
   tags  = gexiv2_metadata_get_xmp_tags (metadata);
   store = GTK_LIST_STORE (gtk_builder_get_object (builder, "xmp-liststore"));
 
-  metadata_dialog_append_tags (metadata, tags, store, C_XMP_TAG, C_XMP_VALUE);
+  metadata_dialog_append_tags (metadata, tags, store, C_XMP_TAG, C_XMP_VALUE, FALSE);
 
   g_strfreev (tags);
 
@@ -312,36 +335,237 @@ metadata_dialog_set_metadata (GExiv2Metadata *metadata,
   tags  = gexiv2_metadata_get_iptc_tags (metadata);
   store = GTK_LIST_STORE (gtk_builder_get_object (builder, "iptc-liststore"));
 
-  metadata_dialog_append_tags (metadata, tags, store, C_IPTC_TAG, C_IPTC_VALUE);
+  metadata_dialog_append_tags (metadata, tags, store, C_IPTC_TAG, C_IPTC_VALUE, TRUE);
 
   g_strfreev (tags);
 }
 
+static gchar *
+metadata_format_string_value (const gchar *value,
+                              gboolean     truncate)
+{
+  glong  size;
+  gchar *result;
+
+  size = g_utf8_strlen (value, -1);
+
+  if (! truncate || size <= TAG_VALUE_MAX_SIZE)
+    {
+      result = g_strdup(value);
+    }
+  else
+    {
+      gchar   *value_utf8_trunc;
+      GString *str;
+
+      value_utf8_trunc = g_utf8_substring (value, 0, TAG_VALUE_MAX_SIZE);
+      str = g_string_new (value_utf8_trunc);
+
+      g_free (value_utf8_trunc);
+
+      g_string_append (str, "... ");
+      g_string_append_printf (str,
+                              _("(%lu more character(s))"),
+                              size - TAG_VALUE_MAX_SIZE);
+
+      result = g_string_free (str, FALSE);
+    }
+  return result;
+}
+
+static inline gboolean
+metadata_tag_is_string (const gchar *tag)
+{
+  const gchar *tag_type;
+
+  tag_type = gexiv2_metadata_get_tag_type (tag);
+
+  return (g_strcmp0 (tag_type, "Byte")      != 0 &&
+          g_strcmp0 (tag_type, "Undefined") != 0 &&
+          g_strcmp0 (tag_type, NULL)        != 0);
+}
+
+static void
+metadata_dialog_add_tag (GtkListStore    *store,
+                         GtkTreeIter      iter,
+                         gint             tag_column,
+                         gint             value_column,
+                         const gchar     *tag,
+                         const gchar     *value)
+{
+  if (value)
+    {
+      gtk_list_store_append (store, &iter);
+      gtk_list_store_set (store, &iter,
+                          tag_column,   tag,
+                          value_column, value,
+                          -1);
+    }
+}
+
+static void
+metadata_dialog_add_translated_tag (GExiv2Metadata  *metadata,
+                                    GtkListStore    *store,
+                                    GtkTreeIter      iter,
+                                    gint             tag_column,
+                                    gint             value_column,
+                                    const gchar     *tag)
+{
+  gchar *value;
+
+  value = gexiv2_metadata_get_tag_interpreted_string (metadata, tag);
+  metadata_dialog_add_tag (store, iter, tag_column, value_column,
+                           tag, gettext (value));
+  g_free (value);
+}
+
 static void
 metadata_dialog_append_tags (GExiv2Metadata  *metadata,
                              gchar          **tags,
                              GtkListStore    *store,
                              gint             tag_column,
-                             gint             value_column)
+                             gint             value_column,
+                             gboolean         load_iptc)
 {
   GtkTreeIter  iter;
   const gchar *tag;
+  const gchar *last_tag = NULL;
+  gboolean     gps_done = FALSE;
 
   while ((tag = *tags++))
     {
-      gchar       *value;
+      gchar  *value;
+      gchar **values;
 
-      gtk_list_store_append (store, &iter);
+      /* We need special handling for iptc tags like "Keywords" which
+       * can appear multiple times. For now assuming that this can
+       * only happen for iptc tags of String and related types.
+       * See also: https://exiv2.org/iptc.html which only lists
+       * one Date type as repeatable (Iptc.Application2.ReferenceDate),
+       * and Date is handled here as string.
+       */
+      if (load_iptc && metadata_tag_is_string (tag))
+        {
+          if (last_tag && ! strcmp (tag, last_tag))
+            {
+              continue;
+            }
+          last_tag = tag;
 
-      value = metadata_dialog_format_tag_value (metadata, tag,
-                                                /* truncate = */ TRUE);
+          values = gexiv2_metadata_get_tag_multiple (GEXIV2_METADATA (metadata),
+                                                     tag);
 
-      gtk_list_store_set (store, &iter,
-                          tag_column,   tag,
-                          value_column, value,
-                          -1);
+          if (values)
+            {
+              gint i;
 
-      g_free (value);
+              for (i = 0; values[i] != NULL; i++)
+                {
+                  gtk_list_store_append (store, &iter);
+
+                  value = metadata_format_string_value (values[i], /* truncate = */ TRUE);
+
+                  gtk_list_store_set (store, &iter,
+                                      tag_column,   tag,
+                                      value_column, value,
+                                      -1);
+
+                  g_free (value);
+                }
+
+              g_strfreev (values);
+            }
+        }
+      else if (! strcmp ("Exif.GPSInfo.GPSLongitude",    tag) ||
+               ! strcmp ("Exif.GPSInfo.GPSLongitudeRef", tag) ||
+               ! strcmp ("Exif.GPSInfo.GPSLatitude",     tag) ||
+               ! strcmp ("Exif.GPSInfo.GPSLatitudeRef",  tag) ||
+               ! strcmp ("Exif.GPSInfo.GPSAltitude",     tag) ||
+               ! strcmp ("Exif.GPSInfo.GPSAltitudeRef",  tag))
+        {
+          /* We need special handling for some of the GPS tags to
+           * be able to show better values than the default. */
+          if (! gps_done)
+            {
+              gdouble  lng, lat, alt;
+              gchar   *str;
+              gchar  *value;
+
+              if (gexiv2_metadata_get_gps_longitude (GEXIV2_METADATA (metadata),
+                                                     &lng))
+                {
+                  str = metadata_format_gps_longitude_latitude (lng);
+                  metadata_dialog_add_tag (store, iter,
+                                           tag_column, value_column,
+                                           "Exif.GPSInfo.GPSLongitude",
+                                           str);
+                  g_free (str);
+                }
+              metadata_dialog_add_translated_tag (metadata, store, iter,
+                                                  tag_column, value_column,
+                                                  "Exif.GPSInfo.GPSLongitudeRef");
+
+              if (gexiv2_metadata_get_gps_latitude (GEXIV2_METADATA (metadata),
+                                                    &lat))
+                {
+                  str = metadata_format_gps_longitude_latitude (lat);
+                  metadata_dialog_add_tag (store, iter,
+                                           tag_column, value_column,
+                                           "Exif.GPSInfo.GPSLatitude",
+                                           str);
+                  g_free (str);
+                }
+              metadata_dialog_add_translated_tag (metadata, store, iter,
+                                                  tag_column, value_column,
+                                                  "Exif.GPSInfo.GPSLatitudeRef");
+
+              if (gexiv2_metadata_get_gps_altitude (GEXIV2_METADATA (metadata),
+                                                    &alt))
+                {
+                  gchar *str2, *str3;
+
+                  str  = metadata_format_gps_altitude (alt, TRUE,  _(" meter"));
+                  str2 = metadata_format_gps_altitude (alt, FALSE, _(" feet"));
+                  str3 = g_strdup_printf ("%s (%s)", str, str2);
+                  metadata_dialog_add_tag (store, iter,
+                                           tag_column, value_column,
+                                           "Exif.GPSInfo.GPSAltitude",
+                                           str3);
+                  g_free (str);
+                  g_free (str2);
+                  g_free (str3);
+                  value = gexiv2_metadata_get_tag_string (metadata,
+                                                          "Exif.GPSInfo.GPSAltitudeRef");
+
+                  if (value)
+                    {
+                      gint index;
+
+                      if (value[0] == '0')
+                        index = 1;
+                      else if (value[0] == '1')
+                        index = 2;
+                      else
+                        index = 0;
+                      metadata_dialog_add_tag (store, iter,
+                                              tag_column, value_column,
+                                              "Exif.GPSInfo.GPSAltitudeRef",
+                                              gettext (gpsaltref[index]));
+                      g_free (value);
+                    }
+                }
+              gps_done = TRUE;
+            }
+        }
+      else
+        {
+          value = metadata_dialog_format_tag_value (metadata, tag,
+                                                    /* truncate = */ TRUE);
+          metadata_dialog_add_tag (store, iter,
+                                   tag_column, value_column,
+                                   tag, value);
+          g_free (value);
+        }
     }
 }
 
@@ -350,51 +574,26 @@ metadata_dialog_format_tag_value (GExiv2Metadata *metadata,
                                   const gchar    *tag,
                                   gboolean        truncate)
 {
-  const gchar *tag_type;
   gchar       *result;
 
-  tag_type = gexiv2_metadata_get_tag_type (tag);
-
-  if (g_strcmp0 (tag_type, "Byte")      != 0 &&
-      g_strcmp0 (tag_type, "Undefined") != 0 &&
-      g_strcmp0 (tag_type, NULL)        != 0)
+  if (metadata_tag_is_string(tag))
     {
       gchar *value;
       gchar *value_utf8;
-      glong  size;
 
       value      = gexiv2_metadata_get_tag_interpreted_string (metadata, tag);
-      value_utf8 = g_locale_to_utf8 (value, -1, NULL, NULL, NULL);
-
-      g_free (value);
-
-      size = g_utf8_strlen (value_utf8, -1);
-
-      if (! truncate || size <= TAG_VALUE_MAX_SIZE)
+      if (! g_utf8_validate (value, -1, NULL))
         {
-          result = value_utf8;
+          value_utf8 = g_locale_to_utf8 (value, -1, NULL, NULL, NULL);
         }
       else
         {
-          gchar   *value_utf8_trunc;
-          GString *str;
-
-          value_utf8_trunc = g_utf8_substring (value_utf8,
-                                               0, TAG_VALUE_MAX_SIZE);
-
-          g_free (value_utf8);
-
-          str = g_string_new (value_utf8_trunc);
-
-          g_free (value_utf8_trunc);
-
-          g_string_append (str, "... ");
-          g_string_append_printf (str,
-                                  _("(%lu more character(s))"),
-                                  size - TAG_VALUE_MAX_SIZE);
-
-          result = g_string_free (str, FALSE);
+          value_utf8 = g_strdup (value);
         }
+
+      g_free (value);
+
+      result = metadata_format_string_value (value_utf8, truncate);
     }
   else
     {
